@@ -1,6 +1,9 @@
 /* Suffix for config filenames */
 #define FVWMRC ".fvwm2rc"
 
+/* Additional version information, like date */
+#define VERSIONINFO ""
+
 /* Define if gdk-imlib is used */
 #undef GDK_IMLIB
 
@@ -77,6 +80,10 @@
 /* Define if nl_langinfo is available */
 #undef HAVE_CODESET
 
+/* Define to 1 if translation of program messages to the user's native
+   language is requested. */
+#undef HAVE_NLS
+
 /* Define if you want the Shaped window extensions.
  * Shaped window extensions seem to increase the window managers RSS
  * by about 60 Kbytes. They provide for leaving a title-bar on the window
@@ -208,16 +215,30 @@
 
 @TOP@
 
-#if 0
-/* migo: Commented; this is not removed with autoconf-2.50 anymore. */
-#error The stuff above TOP goes to the top of config.h.in
-#error What appears below BOTTOM goes to the bottom
-#error This text should not appear anywhere!
-#endif
+/*
+The stuff above TOP goes to the top of config.h.in
+What appears below BOTTOM goes to the bottom
+This text should not appear anywhere (but it does with autoconf-2.50+).
+*/
 
 @BOTTOM@
 
 
+/* Allow GCC extensions to work, if you have GCC. */
+#ifndef __attribute__
+/* This feature is available in gcc versions 2.5 and later. */
+#  if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 5) || __STRICT_ANSI__
+#    define __attribute__(x)
+#  endif
+/* The __-protected variants of `format' and `printf' attributes
+ * are accepted by gcc versions 2.6.4 (effectively 2.7) and later. */
+#  if __GNUC__ < 2 || (__GNUC__ == 2 && __GNUC_MINOR__ < 7)
+#    define __format__ format
+#    define __printf__ printf
+#  endif
+#endif
+
+
 #if HAVE_ALLOCA_H
 #  include <alloca.h>
 #else
@@ -281,3 +302,8 @@
 #endif
 
 #include "libs/defaults.h"
+
+#ifndef FEVENT_C
+#include <X11/Xlib.h>
+#include "libs/FEvent.h"
+#endif
