@@ -53,19 +53,20 @@
 #include "libs/fvwmlib.h"
 #include "libs/FShape.h"
 #include "libs/FScreen.h"
-#include <libs/gravity.h>
 #include "libs/Picture.h"
+#include "libs/PictureUtils.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
-#include "functions.h"
+#include "execcontext.h"
 #include "commands.h"
 #include "bindings.h"
 #include "misc.h"
 #include "screen.h"
-#include "defaults.h"
 #include "add_window.h"
 #include "events.h"
+#include "eventhandler.h"
+#include "eventmask.h"
 #include "module_interface.h"
 #include "stack.h"
 #include "update.h"
@@ -116,12 +117,15 @@ char NoResource[] = "NoResource"; /* Class if no res_name in class hints */
  ***********************************************************************/
 
 static void CaptureOneWindow(
-	FvwmWindow *fw, Window window, Window keep_on_top_win,
-	Window parent_win, Bool is_recapture)
+	const exec_context_t *exc, FvwmWindow *fw, Window window,
+	Window keep_on_top_win, Window parent_win, Bool is_recapture)
 {
 	Window w;
 	unsigned long data[1];
 	initial_window_options_type win_opts;
+	evh_args_t ea;
+	exec_context_changes_t ecc;
+	XEvent e;
 
 	if (fw == NULL)
 	{
@@ -131,8 +135,8 @@ static void CaptureOneWindow(
 	{
 		/* Fvwm might crash in complex functions if we really try to
 		 * the dying window here because AddWindow() may fail and leave
-		 * a destroyed window in Fw.  Any, by the way, it is
-		 * pretty useless to recapture a * window that will vanish in a
+		 * a destroyed window in some structures.  By the way, it is
+		 * pretty useless to recapture a window that will vanish in a
 		 * moment. */
 		return;
 	}
@@ -188,10 +192,23 @@ static void CaptureOneWindow(
 		RestoreWithdrawnLocation(fw, is_recapture, parent_win);
 		SET_DO_REUSE_DESTROYED(fw, 1); /* RBW - 1999/03/20 */
 		destroy_window(fw);
-		Event.xmaprequest.window = w;
 		win_opts.flags.is_menu =
 			(is_recapture && fw != NULL && IS_TEAR_OFF_MENU(fw));
-		HandleMapRequestKeepRaised(keep_on_top_win, fw, &win_opts);
+
+
+		fev_make_null_event(&e, dpy);
+		e.xmaprequest.window = w;
+		e.xmaprequest.parent = Scr.Root;
+		ecc.x.etrigger = &e;
+		ecc.w.fw = NULL;
+		ecc.w.w = w;
+		ecc.w.wcontext = C_ROOT;
+		ea.exc = exc_clone_context(
+			exc, &ecc, ECC_ETRIGGER | ECC_FW | ECC_W |
+			ECC_WCONTEXT);
+		HandleMapRequestKeepRaised(&ea, keep_on_top_win, fw, &win_opts);
+		exc_destroy_context(ea.exc);
+
 		/* HandleMapRequestKeepRaised may have destroyed the fw if the
 		 * window vanished while in AddWindow(), so don't access fw
 		 * anymore before checking if it is a valid window. */
@@ -346,23 +363,17 @@ static int MappedNotOverride(
 
 static void do_recapture(F_CMD_ARGS, Bool fSingle)
 {
-	if (fSingle)
-	{
-		if (DeferExecution(eventp, &w, &fw, &context, CRS_SELECT,
-				   ButtonRelease))
-		{
-			return;
-		}
-	}
+	FvwmWindow *fw = exc->w.fw;
+
 	MyXGrabServer(dpy);
 	if (fSingle)
 	{
 		CaptureOneWindow(
-			fw, FW_W(fw), None, None, True);
+			exc, fw, FW_W(fw), None, None, True);
 	}
 	else
 	{
-		CaptureAllWindows(True);
+		CaptureAllWindows(exc, True);
 	}
 	/* Throw away queued up events. We don't want user input during a
 	 * recapture.  The window the user clicks in might disapper at the very
@@ -687,7 +698,7 @@ static void setup_mini_icon(FvwmWindow *fw, window_style *pstyle)
  * Copy icon size limits from window_style structure to FvwmWindow
  * structure.
  */
-static void setup_icon_size_limits(FvwmWindow *fw, window_style *pstyle)
+void setup_icon_size_limits(FvwmWindow *fw, window_style *pstyle)
 {
 	if (SHAS_ICON_SIZE_LIMITS(&pstyle->flags))
 	{
@@ -717,7 +728,7 @@ static void setup_frame_window(
 	attributes.backing_store = NotUseful;
 	attributes.background_pixmap = None;
 	attributes.cursor = Scr.FvwmCursors[CRS_DEFAULT];
-	attributes.event_mask = XEVMASK_FRAMEW;
+	attributes.event_mask = XEVMASK_FRAMEW_CAPTURE;
 	attributes.save_under = False;
 	/* create the frame window, child of root, grandparent of client */
 	FW_W_FRAME(fw) = XCreateWindow(
@@ -1086,27 +1097,10 @@ static void get_default_window_attributes(
 	FvwmWindow *fw, unsigned long *pvaluemask,
 	XSetWindowAttributes *pattributes)
 {
-	if (Pdepth < 2)
-	{
-		*pvaluemask = CWBackPixmap;
-		if (IS_STICKY(fw))
-		{
-			pattributes->background_pixmap = Scr.sticky_gray_pixmap;
-		}
-		else
-		{
-			pattributes->background_pixmap = Scr.light_gray_pixmap;
-		}
-	}
-	else
-	{
-		*pvaluemask = CWBackPixel;
-		pattributes->background_pixel = fw->colors.back;
-		pattributes->background_pixmap = None;
-	}
-
 	*pvaluemask |= CWBackingStore | CWCursor | CWSaveUnder | CWBorderPixel |
-		CWColormap;
+		CWColormap | CWBackPixmap;
+	pattributes->background_pixel = 0;
+	pattributes->background_pixmap = None;
 	pattributes->backing_store = NotUseful;
 	pattributes->cursor = Scr.FvwmCursors[CRS_DEFAULT];
 	pattributes->save_under = False;
@@ -1253,30 +1247,51 @@ static void setup_icon(FvwmWindow *fw, window_style *pstyle)
 static void destroy_icon(FvwmWindow *fw)
 {
 	free_window_names(fw, False, True);
+	if (IS_PIXMAP_OURS(fw))
+	{
+		XFreePixmap(dpy, fw->iconPixmap);
+		fw->iconPixmap = None;
+		if (fw->icon_maskPixmap != None)
+		{
+			XFreePixmap(dpy, fw->icon_maskPixmap);
+			fw->icon_maskPixmap = None;
+		}
+		if (fw->icon_alphaPixmap != None)
+		{
+			XFreePixmap(dpy, fw->icon_alphaPixmap);
+			fw->icon_alphaPixmap = None;
+		}
+		if (fw->icon_alloc_pixels != NULL)
+		{
+			if (fw->icon_nalloc_pixels != 0)
+			{
+				PictureFreeColors(
+					dpy, Pcmap, fw->icon_alloc_pixels,
+					fw->icon_nalloc_pixels, 0,
+					fw->icon_no_limit);
+			}
+			free(fw->icon_alloc_pixels);
+			fw->icon_alloc_pixels = NULL;
+			fw->icon_nalloc_pixels = 0;
+			fw->icon_no_limit = 0;
+		}
+	}
 	if (FW_W_ICON_TITLE(fw))
 	{
-		if (IS_PIXMAP_OURS(fw))
-		{
-			XFreePixmap(dpy, fw->iconPixmap);
-			if (fw->icon_maskPixmap != None)
-			{
-				XFreePixmap(dpy, fw->icon_maskPixmap);
-			}
-			if (fw->icon_alphaPixmap != None)
-			{
-				XFreePixmap(dpy, fw->icon_alphaPixmap);
-			}
-		}
-		XDeleteContext(dpy, FW_W_ICON_TITLE(fw), FvwmContext);
 		XDestroyWindow(dpy, FW_W_ICON_TITLE(fw));
+		XDeleteContext(dpy, FW_W_ICON_TITLE(fw), FvwmContext);
 		XFlush(dpy);
 	}
-	if ((IS_ICON_OURS(fw))&&(FW_W_ICON_PIXMAP(fw) != None))
-	{
-		XDestroyWindow(dpy, FW_W_ICON_PIXMAP(fw));
-	}
 	if (FW_W_ICON_PIXMAP(fw) != None)
 	{
+		if (IS_ICON_OURS(fw))
+		{
+			XDestroyWindow(dpy, FW_W_ICON_PIXMAP(fw));
+		}
+		else
+		{
+			XUnmapWindow(dpy, FW_W_ICON_PIXMAP(fw));
+		}
 		XDeleteContext(dpy, FW_W_ICON_PIXMAP(fw), FvwmContext);
 	}
 	clear_icon(fw);
@@ -1369,6 +1384,9 @@ static Bool setup_window_placement(
 		{"-xrn", NULL, XrmoptionResArg, (caddr_t) NULL},
 		{"-xrm", NULL, XrmoptionResArg, (caddr_t) NULL},
 	};
+	Bool rc;
+	const exec_context_t *exc;
+	exec_context_changes_t ecc;
 
 	/* Find out if the client requested a specific desk on the command
 	 * line. */
@@ -1448,10 +1466,16 @@ static Bool setup_window_placement(
 	{
 		win_opts->initial_state = IconicState;
 	}
-	return PlaceWindow(
-		fw, &pstyle->flags, attr_g, SGET_START_DESK(*pstyle),
+	ecc.type = EXCT_NULL;
+	ecc.w.fw = fw;
+	exc = exc_create_context(&ecc, ECC_TYPE | ECC_FW);
+	rc = PlaceWindow(
+		exc, &pstyle->flags, attr_g, SGET_START_DESK(*pstyle),
 		SGET_START_PAGE_X(*pstyle), SGET_START_PAGE_Y(*pstyle),
 		SGET_START_SCREEN(*pstyle), PLACE_INITIAL, win_opts);
+	exc_destroy_context(exc);
+
+	return rc;
 }
 
 static void destroy_mini_icon(FvwmWindow *fw)
@@ -1472,16 +1496,11 @@ static void setup_key_and_button_grabs(FvwmWindow *fw)
 	 * the other contexts, new windows have problems when bindings are
 	 * removed.  Therefore, grab all keys in a single pass through the
 	 * list. */
-	GrabAllWindowKeysAndButtons(
-		dpy, FW_W_PARENT(fw), Scr.AllBindings, C_WINDOW,
-		GetUnusedModifiers(), None, True);
 	GrabAllWindowKeys(
 		dpy, FW_W_FRAME(fw), Scr.AllBindings,
-		C_TITLE|C_RALL|C_LALL|C_SIDEBAR, GetUnusedModifiers(), True);
+		C_WINDOW|C_TITLE|C_RALL|C_LALL|C_SIDEBAR, GetUnusedModifiers(),
+		True);
 #endif
-	GrabAllWindowButtons(
-		dpy, FW_W_PARENT(fw), Scr.AllBindings, C_WINDOW,
-		GetUnusedModifiers(), None, True);
 	GrabAllWindowKeys(
 		dpy, FW_W_FRAME(fw), Scr.AllBindings,
 		C_TITLE|C_RALL|C_LALL|C_SIDEBAR|C_WINDOW, GetUnusedModifiers(),
@@ -1694,7 +1713,7 @@ void setup_icon_font(
 		resize_icon_title_height(fw, height - fw->icon_font->height);
 		/* this repositions the icon even if the window is not
 		 * iconified */
-		DrawIconWindow(fw);
+		DrawIconWindow(fw, True, True, False, NULL);
 	}
 
 	return;
@@ -1976,6 +1995,43 @@ void setup_focus_policy(FvwmWindow *fw)
 	return;
 }
 
+Bool validate_transientfor(FvwmWindow *fw)
+{
+	XWindowAttributes wa;
+	Window w;
+
+	w = FW_W_TRANSIENTFOR(fw);
+	if (w == None || w == FW_W(fw))
+	{
+		FW_W_TRANSIENTFOR(fw) = Scr.Root;
+		return False;
+	}
+	else if (!XGetWindowAttributes(dpy, w, &wa) ||
+		 wa.map_state != IsViewable)
+	{
+		/* transientfor does not exist or is not viewable or unmapped */
+		FW_W_TRANSIENTFOR(fw) = Scr.Root;
+		return False;
+	}
+
+	return True;
+}
+
+Bool setup_transientfor(FvwmWindow *fw)
+{
+	Bool rc;
+
+	rc = XGetTransientForHint(dpy, FW_W(fw), &FW_W_TRANSIENTFOR(fw));
+	SET_TRANSIENT(fw, rc);
+	if (rc == False)
+	{
+		FW_W_TRANSIENTFOR(fw) = Scr.Root;
+	}
+	validate_transientfor(fw);
+
+	return rc;
+}
+
 /***********************************************************************
  *
  *  Procedure:
@@ -1983,7 +2039,8 @@ void setup_focus_policy(FvwmWindow *fw)
  *
  ***********************************************************************/
 FvwmWindow *AddWindow(
-	Window w, FvwmWindow *ReuseWin, initial_window_options_type *win_opts)
+	const exec_context_t *exc, FvwmWindow *ReuseWin,
+	initial_window_options_type * win_opts)
 {
 	/* new fvwm window structure */
 	register FvwmWindow *fw;
@@ -2003,6 +2060,9 @@ FvwmWindow *AddWindow(
 	size_borders b;
 	frame_move_resize_args mr_args;
 	mwtsm_state_args state_args;
+	Window w = exc->w.w;
+	const exec_context_t *exc2;
+	exec_context_changes_t ecc;
 
 	/****** init window structure ******/
 	setup_window_structure(&tmp, w, ReuseWin);
@@ -2036,9 +2096,7 @@ FvwmWindow *AddWindow(
 	/* get merged styles */
 	lookup_style(fw, &style);
 	sflags = SGET_FLAGS_POINTER(style);
-	SET_TRANSIENT(
-		fw, !!XGetTransientForHint(
-			dpy, FW_W(fw), &FW_W_TRANSIENTFOR (fw)));
+	setup_transientfor(fw);
 	if (win_opts->flags.is_menu)
 	{
 		SET_TEAR_OFF_MENU(fw, 1);
@@ -2122,7 +2180,7 @@ FvwmWindow *AddWindow(
 		/****** calculate frame size ******/
 		setup_frame_size_limits(fw, &style);
 		constrain_size(
-			fw, (unsigned int *)&fw->frame_g.width,
+			fw, NULL, (unsigned int *)&fw->frame_g.width,
 			(unsigned int *)&fw->frame_g.height, 0, 0, 0);
 
 		/****** maximize ******/
@@ -2130,7 +2188,7 @@ FvwmWindow *AddWindow(
 		{
 			SET_MAXIMIZED(fw, 1);
 			constrain_size(
-				fw, (unsigned int *)&fw->max_g.width,
+				fw, NULL, (unsigned int *)&fw->max_g.width,
 				(unsigned int *)&fw->max_g.height, 0, 0,
 				CS_UPDATE_MAX_DEFECT);
 			get_relative_geometry(&fw->frame_g, &fw->max_g);
@@ -2228,21 +2286,27 @@ FvwmWindow *AddWindow(
 		SET_FORCE_NEXT_CR(fw, 1);
 		SET_FORCE_NEXT_PN(fw, 1);
 		mr_args = frame_create_move_resize_args(
-			fw, FRAME_MR_FORCE_SETUP_NO_W, NULL, &fw->frame_g, 0,
-			DIR_NONE);
+			fw, FRAME_MR_FORCE_SETUP_NO_W | FRAME_MR_DONT_DRAW,
+			NULL, &fw->frame_g, 0, DIR_NONE);
 	}
 	else
 	{
 		mr_args = frame_create_move_resize_args(
-			fw, FRAME_MR_FORCE_SETUP, NULL, &fw->frame_g, 0,
-			DIR_NONE);
+			fw, FRAME_MR_FORCE_SETUP | FRAME_MR_DONT_DRAW, NULL,
+			&fw->frame_g, 0, DIR_NONE);
 	}
 	frame_move_resize(fw, mr_args);
 	frame_free_move_resize_args(fw, mr_args);
+	/* draw later */
+	SET_WAS_NEVER_DRAWN(fw, 1);
 
 	/****** grab keys and buttons ******/
 	setup_key_and_button_grabs(fw);
 
+	/****** inform modules of new window ******/
+	BroadcastConfig(M_ADD_WINDOW,fw);
+	BroadcastWindowIconNames(fw, True, False);
+
 	/****** place the window in the stack ring ******/
 	if (!position_new_window_in_stack_ring(fw, SDO_START_LOWERED(sflags)))
 	{
@@ -2253,10 +2317,6 @@ FvwmWindow *AddWindow(
 			dpy, FW_W_FRAME(fw), CWSibling|CWStackMode, &xwc);
 	}
 
-	/****** inform modules of new window ******/
-	BroadcastConfig(M_ADD_WINDOW,fw);
-	BroadcastWindowIconNames(fw, True, False);
-
 	/* these are sent and broadcast before res_{class,name} for the benefit
 	 * of FvwmIconBox which can't handle M_ICON_FILE after M_RES_NAME */
 	/****** icon and mini icon ******/
@@ -2273,39 +2333,62 @@ FvwmWindow *AddWindow(
 		      (unsigned long)fw,fw->class.res_name);
 
 	/****** stick window ******/
-	if (IS_STICKY(fw) && (!(fw->hints.flags & USPosition) || used_sm))
+	if (!(fw->hints.flags & USPosition) || used_sm)
 	{
-		/* If it's sticky and the user didn't ask for an explicit
-		 * position, force it on screen now.  Don't do that with
-		 * USPosition because we have to assume the user knows what
-		 * (s)he is doing.  This is necessary e.g. if we want a sticky
-		 * 'panel' in FvwmButtons but don't want to see when it's
-		 * mapped in the void. */
-		if (!IsRectangleOnThisPage(&fw->frame_g, Scr.CurrentDesk))
+		int stick_page;
+		int stick_desk;
+
+		stick_page = is_window_sticky_on_page(fw);
+		stick_desk = is_window_sticky_on_page(fw);
+		if ((stick_page &&
+		     !IsRectangleOnThisPage(&fw->frame_g, Scr.CurrentDesk)) ||
+		    (stick_desk && fw->Desk != Scr.CurrentDesk))
 		{
-			SET_STICKY(fw, 0);
+			/* If it's sticky and the user didn't ask for an
+			 * explicit position, force it on screen now.  Don't do
+			 * that with USPosition because we have to assume the
+			 * user knows what (s)he is doing.  This is necessary
+			 * e.g. if we want a sticky 'panel' in FvwmButtons but
+			 * don't want to see when it's mapped in the void. */
+			ecc.w.fw = fw;
+			ecc.w.w = FW_W_FRAME(fw);
+			ecc.w.wcontext = C_FRAME;
+			exc2 = exc_clone_context(
+				exc, &ecc, ECC_FW | ECC_W | ECC_WCONTEXT);
+			SET_STICKY_ON_PAGE(fw, 0);
+			SET_STICKY_ON_DESK(fw, 0);
 			handle_stick(
-				NULL, &Event, FW_W_FRAME(fw), fw, C_FRAME, "",
-				0, 1);
+				NULL, exc2, "", stick_page, stick_desk, 1, 0);
+			exc_destroy_context(exc2);
 		}
 	}
 
 	/****** resize window ******/
 	if (do_resize_too)
 	{
-		XWarpPointer(dpy, Scr.Root, Scr.Root, 0, 0, Scr.MyDisplayWidth,
-			     Scr.MyDisplayHeight,
-			     fw->frame_g.x + (fw->frame_g.width>>1),
-			     fw->frame_g.y + (fw->frame_g.height>>1));
-		Event.xany.type = ButtonPress;
-		Event.xbutton.button = 1;
-		Event.xbutton.x_root = fw->frame_g.x + (fw->frame_g.width>>1);
-		Event.xbutton.y_root = fw->frame_g.y + (fw->frame_g.height>>1);
-		Event.xbutton.x = (fw->frame_g.width>>1);
-		Event.xbutton.y = (fw->frame_g.height>>1);
-		Event.xbutton.subwindow = None;
-		Event.xany.window = FW_W(fw);
-		CMD_Resize(NULL, &Event , FW_W(fw), fw, C_WINDOW, "", 0);
+		XEvent e;
+
+		FWarpPointer(
+			dpy, Scr.Root, Scr.Root, 0, 0, Scr.MyDisplayWidth,
+			Scr.MyDisplayHeight,
+			fw->frame_g.x + (fw->frame_g.width>>1),
+			fw->frame_g.y + (fw->frame_g.height>>1));
+		e.xany.type = ButtonPress;
+		e.xbutton.button = 1;
+		e.xbutton.x_root = fw->frame_g.x + (fw->frame_g.width>>1);
+		e.xbutton.y_root = fw->frame_g.y + (fw->frame_g.height>>1);
+		e.xbutton.x = (fw->frame_g.width>>1);
+		e.xbutton.y = (fw->frame_g.height>>1);
+		e.xbutton.subwindow = None;
+		e.xany.window = FW_W(fw);
+		fev_fake_event(&e);
+		ecc.x.etrigger = &e;
+		ecc.w.fw = fw;
+		ecc.w.wcontext = C_WINDOW;
+		exc2 = exc_clone_context(
+			exc, &ecc, ECC_ETRIGGER | ECC_FW | ECC_WCONTEXT);
+		CMD_Resize(NULL, exc2, "");
+		exc_destroy_context(exc2);
 	}
 
 	/****** window colormap ******/
@@ -2337,8 +2420,8 @@ FvwmWindow *AddWindow(
 		get_shaded_geometry_with_dir(
 			fw, &new_g, &new_g, state_args.shade_dir);
 		mr_args = frame_create_move_resize_args(
-			fw, FRAME_MR_SHRINK, &big_g, &new_g, 0,
-			state_args.shade_dir);
+			fw, FRAME_MR_SHRINK | FRAME_MR_DONT_DRAW, &big_g,
+			&new_g, 0, state_args.shade_dir);
 		frame_move_resize(fw, mr_args);
 		SET_SHADED(fw, 1);
 		SET_SHADED_DIR(fw, state_args.shade_dir);
@@ -2838,7 +2921,10 @@ void destroy_window(FvwmWindow *fw)
 	/****** remove from window list ******/
 
 	/* first of all, remove the window from the list of all windows! */
-	fw->prev->next = fw->next;
+	if (fw->prev != NULL)
+	{
+		fw->prev->next = fw->next;
+	}
 	if (fw->next != NULL)
 	{
 		fw->next->prev = fw->prev;
@@ -2855,7 +2941,9 @@ void destroy_window(FvwmWindow *fw)
 
 	/****** check if we have to delay window destruction ******/
 
-	if (Scr.flags.is_executing_complex_function && !DO_REUSE_DESTROYED(fw))
+	if ((Scr.flags.is_executing_complex_function ||
+	     Scr.flags.is_executing_menu_function) &&
+	    !DO_REUSE_DESTROYED(fw))
 	{
 		/* mark window for destruction */
 		SET_SCHEDULED_FOR_DESTROY(fw, 1);
@@ -3000,6 +3088,13 @@ void RestoreWithdrawnLocation(
 		return;
 	}
 
+	if (HAS_NEW_WM_NORMAL_HINTS(fw))
+	{
+		/* get the latest size hints */
+		XSync(dpy, 0);
+		GetWindowSizeHints(fw);
+		SET_HAS_NEW_WM_NORMAL_HINTS(fw, 0);
+	}
 	get_unshaded_geometry(fw, &unshaded_g);
 	gravity_get_naked_geometry(
 		fw->hints.win_gravity, fw, &naked_g, &unshaded_g);
@@ -3131,7 +3226,7 @@ void Reborder(void)
 	return;
 }
 
-void CaptureAllWindows(Bool is_recapture)
+void CaptureAllWindows(const exec_context_t *exc, Bool is_recapture)
 {
 	int i,j;
 	unsigned int nchildren;
@@ -3150,9 +3245,11 @@ void CaptureAllWindows(Bool is_recapture)
 	win_opts.flags.is_recapture = 1;
 	if (!(Scr.flags.windows_captured)) /* initial capture? */
 	{
-		/*
-		** weed out icon windows
-		*/
+		evh_args_t ea;
+		exec_context_changes_t ecc;
+		XEvent e;
+
+		/* weed out icon windows */
 		for (i = 0; i < nchildren; i++)
 		{
 			if (children[i])
@@ -3179,18 +3276,27 @@ void CaptureAllWindows(Bool is_recapture)
 				}
 			}
 		}
-		/*
-		** map all of the non-override, non-icon windows
-		*/
-		for (i=0;i<nchildren;i++)
+		/* map all of the non-override, non-icon windows */
+		e.type = MapRequest;
+		ecc.x.etrigger = &e;
+		for (i = 0; i < nchildren; i++)
 		{
+
 			if (children[i] &&
 			    MappedNotOverride(children[i], &win_opts))
 			{
 				XUnmapWindow(dpy, children[i]);
-				Event.xmaprequest.window = children[i];
+				e.xmaprequest.window = children[i];
+				e.xmaprequest.parent = Scr.Root;
+				ecc.w.fw = NULL;
+				ecc.w.w = children[i];
+				ecc.w.wcontext = C_ROOT;
+				ea.exc = exc_clone_context(
+					exc, &ecc, ECC_ETRIGGER | ECC_FW |
+					ECC_W | ECC_WCONTEXT);
 				HandleMapRequestKeepRaised(
-					None, NULL, &win_opts);
+					&ea, None, NULL, &win_opts);
+				exc_destroy_context(ea.exc);
 			}
 		}
 		Scr.flags.windows_captured = 1;
@@ -3213,7 +3319,7 @@ void CaptureAllWindows(Bool is_recapture)
 				    (caddr_t *)&fw) != XCNOENT)
 			{
 				CaptureOneWindow(
-					fw, children[i], keep_on_top_win,
+					exc, fw, children[i], keep_on_top_win,
 					parent_win, is_recapture);
 			}
 		}
