@@ -35,6 +35,9 @@ void setup_visible_name(FvwmWindow *fw, Bool is_icon);
 void setup_wm_hints(FvwmWindow *fw);
 void setup_placement_penalty(FvwmWindow *fw, window_style *pstyle);
 void setup_focus_policy(FvwmWindow *fw);
+Bool setup_transientfor(FvwmWindow *fw);
+void setup_icon_size_limits(FvwmWindow *fw, window_style *pstyle);
+Bool validate_transientfor(FvwmWindow *fw);
 void setup_title_geometry(
 	FvwmWindow *fw, window_style *pstyle);
 void setup_window_font(
@@ -62,7 +65,8 @@ void change_icon_boxes(
 void FetchWmProtocols(
 	FvwmWindow *);
 FvwmWindow *AddWindow(
-	Window w, FvwmWindow *ReuseWin, initial_window_options_type * win_opts);
+	const exec_context_t *exc, FvwmWindow *ReuseWin,
+	initial_window_options_type * win_opts);
 void GetWindowSizeHints(
 	FvwmWindow *);
 void free_window_names(
@@ -72,5 +76,6 @@ void destroy_window(
 void RestoreWithdrawnLocation(
 	FvwmWindow *tmp, Bool is_restart_or_recapture, Window parent);
 void Reborder(void);
+void CaptureAllWindows(const exec_context_t *exc, Bool is_recapture);
 
 #endif /* ADD_WINDOW_H */
