@@ -457,7 +457,8 @@ static Bool __is_bpress_window_handled(const exec_context_t *exc)
 
 	if (exc->w.fw == NULL)
 	{
-		if (te->xbutton.window != Scr.Root &&
+		if ((te->xbutton.window != Scr.Root ||
+		     te->xbutton.subwindow != None) &&
 		    !is_pan_frame(te->xbutton.window))
 		{
 			/* Ignore events in unmanaged windows or subwindows of
@@ -595,7 +596,7 @@ static void __handle_bpress_on_root(const exec_context_t *exc)
 	action = CheckBinding(
 		Scr.AllBindings, STROKE_ARG(0) exc->x.etrigger->xbutton.button,
 		exc->x.etrigger->xbutton.state, GetUnusedModifiers(), C_ROOT,
-		MOUSE_BINDING);
+		BIND_BUTTONPRESS);
 	if (action && *action)
 	{
 		const exec_context_t *exc2;
@@ -668,7 +669,7 @@ static void __handle_bpress_on_managed(const exec_context_t *exc)
 		action = CheckBinding(
 			Scr.AllBindings, STROKE_ARG(0) e->xbutton.button,
 			e->xbutton.state, GetUnusedModifiers(), exc->w.wcontext,
-			MOUSE_BINDING);
+			BIND_BUTTONPRESS);
 		if (__handle_bpress_action(exc, action))
 		{
 			f.do_swallow_click = 1;
@@ -743,7 +744,7 @@ void HandleButtonRelease(const evh_args_t *ea)
 	/* need to search for an appropriate stroke binding */
 	action = CheckBinding(
 		Scr.AllBindings, sequence, te->xbutton.button, real_modifier,
-		GetUnusedModifiers(), ea->exc->w.wcontext, STROKE_BINDING);
+		GetUnusedModifiers(), ea->exc->w.wcontext, BIND_STROKE);
 	/* got a match, now process it */
 	if (action != NULL && (action[0] != 0))
 	{
@@ -1773,11 +1774,6 @@ void HandleExpose(const evh_args_t *ea)
 		/* refresh the contents of the torn out menu */
 		menu_expose(&e, NULL);
 	}
-else if (WAS_NEVER_DRAWN(fw))
-{
-border_draw_decorations(
-	fw, PART_ALL, (Scr.Hilite == fw), True, CLEAR_ALL, NULL, NULL);
-}
 
 	return;
 }
@@ -1936,15 +1932,17 @@ void HandleFocusOut(const evh_args_t *ea)
 	return;
 }
 
-void HandleKeyPress(const evh_args_t *ea)
+void __handle_key_event(const evh_args_t *ea, Bool is_release)
 {
 	char *action;
 	FvwmWindow *sf;
-	const XEvent *te = ea->exc->x.etrigger;
 	KeyCode kc;
+	int context2;
+	binding_t type1;
+	binding_t type2;
+	const XEvent *te = ea->exc->x.etrigger;
 	const FvwmWindow * const fw = ea->exc->w.fw;
-
-	DBUG("HandleKeyPress","Routine Entered");
+	Bool is_second_binding;
 
 	PressedW = None;
 
@@ -1955,12 +1953,60 @@ void HandleKeyPress(const evh_args_t *ea)
 		dpy, XKeycodeToKeysym(dpy, te->xkey.keycode, 0));
 
 	/* Check if there is something bound to the key */
-	action = CheckBinding(
-		Scr.AllBindings, STROKE_ARG(0) kc, te->xkey.state,
-		GetUnusedModifiers(), ea->exc->w.wcontext, KEY_BINDING);
+	sf = get_focus_window();
+	if (sf != NULL && sf != ea->exc->w.fw)
+	{
+		context2 = C_WINDOW | C_ROOT;
+	}
+	else
+	{
+		context2 = ea->exc->w.wcontext;
+	}
+	if (is_release == False)
+	{
+		if (CheckTwoBindings(
+			    &is_second_binding, Scr.AllBindings, STROKE_ARG(0)
+			    kc, te->xkey.state, GetUnusedModifiers(),
+			    ea->exc->w.wcontext, BIND_KEYRELEASE, context2,
+			    BIND_PKEYRELEASE) != NULL)
+		{
+			MyXGrabKey(dpy);
+		}
+		type1 = BIND_KEYPRESS;
+		type2 = BIND_PKEYPRESS;
+	}
+	else
+	{
+		type1 = BIND_KEYRELEASE;
+		type2 = BIND_PKEYRELEASE;
+	}
+	action = CheckTwoBindings(
+		&is_second_binding, Scr.AllBindings, STROKE_ARG(0) kc,
+		te->xkey.state, GetUnusedModifiers(), ea->exc->w.wcontext,
+		type1, context2, type2);
 	if (action != NULL)
 	{
-		execute_function(NULL, ea->exc, action, 0);
+		const exec_context_t *exc;
+		exec_context_changes_t ecc;
+
+		exc = ea->exc;
+		if (is_second_binding == False)
+		{
+			ecc.w.fw = sf;
+			ecc.w.wcontext = context2;
+			exc = exc_clone_context(
+				ea->exc, &ecc, ECC_FW | ECC_WCONTEXT);
+		}
+		execute_function(NULL, exc, action, 0);
+		if (is_second_binding == False)
+		{
+			exc_destroy_context(exc);
+		}
+		if (is_release == True)
+		{
+			MyXUngrabKey(dpy);
+		}
+		XAllowEvents(dpy, AsyncKeyboard, CurrentTime);
 		return;
 	}
 
@@ -1973,7 +2019,10 @@ void HandleKeyPress(const evh_args_t *ea)
 
 		e = *te;
 		e.xkey.window = FW_W(sf);
-		FSendEvent(dpy, FW_W(sf), False, KeyPressMask, &e);
+		FSendEvent(
+			dpy, e.xkey.window, False,
+			(is_release == True) ? KeyReleaseMask : KeyPressMask,
+			&e);
 	}
 	else if (fw && te->xkey.window != FW_W(fw))
 	{
@@ -1981,8 +2030,26 @@ void HandleKeyPress(const evh_args_t *ea)
 
 		e = *te;
 		e.xkey.window = FW_W(fw);
-		FSendEvent(dpy, FW_W(fw), False, KeyPressMask, &e);
+		FSendEvent(
+			dpy, e.xkey.window, False,
+			(is_release == True) ? KeyReleaseMask : KeyPressMask,
+			&e);
 	}
+	XAllowEvents(dpy, AsyncKeyboard, CurrentTime);
+
+	return;
+}
+
+void HandleKeyPress(const evh_args_t *ea)
+{
+	__handle_key_event(ea, False);
+
+	return;
+}
+
+void HandleKeyRelease(const evh_args_t *ea)
+{
+	__handle_key_event(ea, True);
 
 	return;
 }
@@ -2103,6 +2170,12 @@ void HandleMapNotify(const evh_args_t *ea)
 		}
 		return;
 	}
+	if (te->xmap.window == FW_W_FRAME(fw))
+	{
+		/* Now that we know the frame is mapped after capturing the
+		 * window we do not need StructureNotifyMask events anymore. */
+		XSelectInput(dpy, FW_W_FRAME(fw), XEVMASK_FRAMEW);
+	}
 	/* Except for identifying over-ride redirect window mappings, we
 	 * don't need or want windows associated with the
 	 * SubstructureNotifyMask */
@@ -2900,6 +2973,7 @@ void HandleUnmapNotify(const evh_args_t *ea)
 	if (te->xunmap.window == FW_W_FRAME(fw))
 	{
 		SET_ICONIFY_PENDING(fw , 0);
+		return;
 	}
 	if (must_return)
 	{
@@ -3058,6 +3132,7 @@ void InitEventHandlerJumpTable(void)
 	EventHandlerJumpTable[ClientMessage] =    HandleClientMessage;
 	EventHandlerJumpTable[PropertyNotify] =   HandlePropertyNotify;
 	EventHandlerJumpTable[KeyPress] =         HandleKeyPress;
+	EventHandlerJumpTable[KeyRelease] =       HandleKeyRelease;
 	EventHandlerJumpTable[VisibilityNotify] = HandleVisibilityNotify;
 	EventHandlerJumpTable[ColormapNotify] =   HandleColormapNotify;
 	if (FShapesSupported)
@@ -3806,205 +3881,6 @@ void CMD_XSynchronize(F_CMD_ARGS)
 	return;
 }
 
-#if 0
-/* experimental code */
-int addkbsubinstoarray(Window parentw, Window *winlist, int offset)
-{
-	Window *children;
-	int i;
-	int n;
-	int s;
-	Window JunkW;
-	static int depth = 0;
-
-	depth++;
-	if (XQueryTree(dpy, parentw, &JunkW, &JunkW, &children, &n) == 0)
-	{
-		depth--;
-		return offset;
-	}
-/*fprintf(stderr,"%d: 0x%08x has %d children\n", depth, (int)parentw, n);*/
-	if (n == 0)
-	{
-		depth--;
-		return offset;
-	}
-	for (i = 0; i < n; i++)
-	{
-		XWindowAttributes xwa;
-
-		if (XGetWindowAttributes(dpy, children[i], &xwa) == 0)
-		{
-			continue;
-		}
-#if 0
-		if (xwa.map_state != IsViewable)
-		{
-			continue;
-		}
-#endif
-#define KBWIN_MIN_WIDTH 100
-#define KBWIN_MIN_HEIGHT 23
-#define MASK (KeyPressMask|KeyReleaseMask)
-		s = addkbsubinstoarray(children[i], winlist, offset);
-		if ((xwa.all_event_masks & MASK) &&
-		    xwa.width >= KBWIN_MIN_WIDTH &&
-		    xwa.height >= KBWIN_MIN_HEIGHT &&
-		    s == offset)
-		{
-fprintf(stderr, "w %d (%d), h %d (%d)\n", xwa.width, KBWIN_MIN_WIDTH, xwa.height, KBWIN_MIN_HEIGHT);
-			winlist[offset] = children[i];
-			offset++;
-fprintf(stderr,"%d: + 0x%08x offset %d\n", depth, (int)children[i], offset);
-		}
-#if 1
-		else if (depth == 1 && i + 1 == n)
-		{
-			offset = s;
-			winlist[offset] = parentw;
-			offset++;
-fprintf(stderr,"%d: * 0x%08x offset %d\n", depth, (int)parentw, offset);
-		}
-#endif
-		else
-		{
-/*fprintf(stderr,"%d: v 0x%08x s %d offset %d ev 0x%x 0x%x 0x%x\n", depth, (int)children[i], s, offset, (int)xwa.all_event_masks, (int)MASK, (int)(xwa.all_event_masks & MASK));*/
-			offset = s;
-		}
-	}
-	XFree(children);
-	depth--;
-
-	return offset;
-}
-
-int findcurrentkbwininlist(Window start_win, Window *winlist, int n)
-{
-	int x;
-	int y;
-	int new_x;
-	int new_y;
-	int i;
-	int found = -1;
-	Window child;
-	Window new_child;
-
-/*fprintf(stderr,"find sw = 0x%08x\n", (int)start_win);*/
-	if (FQueryPointer(
-		    dpy, start_win, &JunkRoot, &child, &JunkX, &JunkY, &x, &y,
-		    &JunkMask) == False)
-	{
-		return None;
-	}
-	while (start_win != None)
-	{
-/*fprintf(stderr,"checking 0x%08x %d\n", (int)start_win, n);*/
-		for (i = 0; i < n; i++)
-		{
-			if (winlist[i] == start_win)
-			{
-fprintf(stderr,"is in 0x%08x\n", (int)start_win);
-				found = i;
-				break;
-			}
-		}
-		if (child != None && XTranslateCoordinates(
-			    dpy, start_win, child, x, y, &new_x, &new_y,
-			    &new_child) == False)
-		{
-			break;
-		}
-		start_win = child;
-		child = new_child;
-		x = new_x;
-		y = new_y;
-	}
-
-	return found;
-}
-
-void getkbsubwins(F_CMD_ARGS)
-{
-	Window win;
-	Window focus_win;
-	Window winlist[1000];
-	static Window prev_win = None;
-	static Window last_win = None;
-	int n;
-	int i;
-	int j;
-	char cmd[256];
-	int x;
-	int y;
-	int wid;
-	int hei;
-
-	if (!fw)
-	{
-		return;
-	}
-	win = FW_W(fw);
-fprintf(stderr,"start 0x%08x prev 0x%08x last 0x%08x\n", (int)win, (int)prev_win, (int)last_win);
-	n = addkbsubinstoarray(win, &(winlist[0]), 0);
-	if (n == 0)
-	{
-		return;
-	}
-	XGetInputFocus(dpy, &focus_win, &i);
-/*fprintf(stderr,"focus = 0x%08x\n",(int)focus_win);*/
-	if (focus_win == None)
-	{
-		return;
-	}
-	win = focus_win;
-	i = findcurrentkbwininlist(win, &(winlist[0]), n);
-	if (i < 0)
-	{
-		XBell(dpy, 0);
-		return;
-	}
-	if (winlist[i] != last_win && last_win != None && XGetGeometry(
-		    dpy, last_win, &JunkRoot, &JunkX, &JunkY, &JunkWidth,
-		    &JunkHeight, &JunkBW, &JunkMask) != 0)
-	{
-		for (j = 0; j < n; j++)
-		{
-			if (winlist[j] == last_win)
-			{
-				i = j;
-				break;
-			}
-		}
-	}
-	prev_win = winlist[i];
-XGetGeometry(dpy, winlist[i], &JunkRoot, &x,&y,&wid,&hei,&JunkBW, &JunkMask);
-fprintf(stderr,"  %d (%d %d %dx%d) -> ", i, x,y,wid,hei);
-	i++;
-	if (i >= n)
-	{
-		i = 0;
-	}
-	if (winlist[i] == focus_win && n > 1)
-	{
-		i++;
-		if (i >= n)
-		{
-			i = 0;
-		}
-	}
-	win = winlist[i];
-XGetGeometry(dpy, win, &JunkRoot, &x,&y,&wid,&hei,&JunkBW, &JunkMask);
-fprintf(stderr," %d (%d %d %dx%d)\n", i, x,y,wid,hei);
-	sprintf(cmd, "WindowId 0x%x WarpToWindow 99 99", (int)win);
-	fprintf(stderr, "%s\n", cmd);
-	execute_function(NULL, exc, cmd, 0);
-	last_win = win;
-	/*!!!*/
-
-	return;
-}
-#endif
-
 void CMD_XSync(F_CMD_ARGS)
 {
 	XSync(dpy, 0);
