@@ -24,9 +24,8 @@
 #include <X11/Xlib.h>
 #include <X11/Xmd.h>
 
-#include "libs/fvwmlib.h"
 #include "fvwm.h"
-#include "cursor.h"
+#include "execcontext.h"
 #include "functions.h"
 #include "misc.h"
 #include "screen.h"
@@ -34,10 +33,8 @@
 #include "commands.h"
 #include "update.h"
 #include "style.h"
-#include "focus.h"
 #include "stack.h"
 #include "events.h"
-#include "externs.h"
 #include "ewmh.h"
 #include "ewmh_intern.h"
 
@@ -56,8 +53,10 @@ int ewmh_CurrentDesktop(EWMH_CMD_ARGS)
 int ewmh_DesktopGeometry(EWMH_CMD_ARGS)
 {
   char action[256];
-  sprintf(action, "%ld %ld", ev->xclient.data.l[0], ev->xclient.data.l[1]);
-  CMD_DesktopSize(NULL, None, NULL, C_ROOT, action, NULL);
+  sprintf(action, "DesktoSize %ld %ld", ev->xclient.data.l[0],
+	  ev->xclient.data.l[1]);
+  execute_function_override_window(NULL, NULL, action, 0, NULL);
+
   return -1;
 }
 
@@ -72,7 +71,7 @@ int ewmh_NumberOfDesktops(EWMH_CMD_ARGS)
   int d = ev->xclient.data.l[0];
 
   /* not a lot of sinification for FVWM */
-  if (d > 0 && (d <= ewmhc.MaxDesktops || ewmhc.MaxDesktops == 0)) 
+  if (d > 0 && (d <= ewmhc.MaxDesktops || ewmhc.MaxDesktops == 0))
   {
     ewmhc.NumberOfDesktops = d;
     EWMH_SetNumberOfDesktops();
@@ -88,8 +87,9 @@ int ewmh_ActiveWindow(EWMH_CMD_ARGS)
 {
   if (ev == NULL)
     return 0;
-  
-  old_execute_function("EWMHActivateWindowFunc", fwin, ev, C_WINDOW, -1,0,NULL);
+  execute_function_override_window(
+	  NULL, NULL, "EWMHActivateWindowFunc", 0, fwin);
+
   return 0;
 }
 
@@ -97,8 +97,8 @@ int ewmh_CloseWindow(EWMH_CMD_ARGS)
 {
   if (ev == NULL)
     return 0;
+  execute_function_override_window(NULL, NULL, "Close", 0, fwin);
 
-  CMD_Close(ev, fwin->w, fwin, C_WINDOW, "", NULL);
   return 0;
 }
 
@@ -109,22 +109,22 @@ int ewmh_WMDesktop(EWMH_CMD_ARGS)
     /* client message */
     unsigned long d = (unsigned long)ev->xclient.data.l[0];
 
-    /* the spec says that if d = 0xFFFFFFFF then we have to Stick the window 
-    *  however KDE use 0xFFFFFFFE :o) */ 
+    /* the spec says that if d = 0xFFFFFFFF then we have to Stick the window
+    *  however KDE use 0xFFFFFFFE :o) */
     if (d == 0xFFFFFFFE || d == 0xFFFFFFFF)
     {
-      CMD_Stick(ev, fwin->w, fwin, C_WINDOW, "On", NULL);
+      execute_function_override_window(NULL, NULL, "Stick on", 0, fwin);
     }
     else if (d >= 0)
     {
-      if (IS_STICKY(fwin))
-	 CMD_Stick(ev, fwin->w, fwin, C_WINDOW, "Off", NULL);
+      if (IS_STICKY_ON_PAGE(fwin) || IS_STICKY_ON_DESK(fwin))
+	 execute_function_override_window(NULL, NULL, "Stick off", 0, fwin);
       if (fwin->Desk != d)
 	do_move_window_to_desk(fwin, (int)d);
     }
     return 0;
   }
-  
+
   if (style != NULL && ev == NULL)
   {
     /* start on desk */
@@ -134,7 +134,7 @@ int ewmh_WMDesktop(EWMH_CMD_ARGS)
     if (HAS_EWMH_INIT_WM_DESKTOP(fwin) != EWMH_STATE_UNDEFINED_HINT)
       return 0;
 
-    val = ewmh_AtomGetByName(fwin->w,
+    val = ewmh_AtomGetByName(FW_W(fwin),
 			     "_NET_WM_DESKTOP",
 			     EWMH_ATOM_LIST_CLIENT_WIN,
 			     &size);
@@ -146,16 +146,19 @@ int ewmh_WMDesktop(EWMH_CMD_ARGS)
     }
 #if DEBUG_EWMH_INIT_STATE
     fprintf(stderr,"ewmh WM_DESKTOP hint for window 0x%lx  (%i,%lu,%lu)\n",
-	    fwin->w, HAS_EWMH_INIT_WM_DESKTOP(fwin),
+	    FW_W(fwin), HAS_EWMH_INIT_WM_DESKTOP(fwin),
 	    fwin->ewmh_hint_desktop, val[0]);
 #endif
     if (!DO_EWMH_IGNORE_STATE_HINTS(style))
     {
       if ((val[0] == 0xFFFFFFFE || val[0] == 0xFFFFFFFF))
       {
-	SFSET_IS_STICKY(*style, 1);
-	SMSET_IS_STICKY(*style, 1);
-	SCSET_IS_STICKY(*style, 1);
+	S_SET_IS_STICKY_ON_PAGE(SCF(*style), 1);
+	S_SET_IS_STICKY_ON_PAGE(SCM(*style), 1);
+	S_SET_IS_STICKY_ON_PAGE(SCC(*style), 1);
+	S_SET_IS_STICKY_ON_DESK(SCF(*style), 1);
+	S_SET_IS_STICKY_ON_DESK(SCM(*style), 1);
+	S_SET_IS_STICKY_ON_DESK(SCC(*style), 1);
       }
       else if (val[0] < 256)
       {
@@ -215,19 +218,19 @@ int ewmh_MoveResize(EWMH_CMD_ARGS)
 	move = True;
 	break;
     }
-    sprintf(cmd,"%i %i",x_warp,y_warp);
-    CMD_WarpToWindow(ev, fwin->w, fwin, C_WINDOW, cmd, NULL);
+    sprintf(cmd, "WarpToWindow %i %i",x_warp,y_warp);
+    execute_function_override_window(NULL, NULL, cmd, 0, fwin);
     if (move)
-      CMD_Move(ev, fwin->w, fwin, C_WINDOW, "", NULL);
+      execute_function_override_window(NULL, NULL, "Move", 0, fwin);
     else
-      CMD_Resize(ev, fwin->w, fwin, C_WINDOW, "", NULL);
+      execute_function_override_window(NULL, NULL, "Resize", 0, fwin);
 
     return 0;
   }
 }
 
 /* ************************************************************************* *
- * WM_STATE* 
+ * WM_STATE*
  * ************************************************************************* */
 int ewmh_WMState(EWMH_CMD_ARGS)
 {
@@ -257,7 +260,7 @@ int ewmh_WMState(EWMH_CMD_ARGS)
     ewmh_atom *list = ewmh_atom_wm_state;
     int has_hint = 0;
 
-    val = ewmh_AtomGetByName(fwin->w,
+    val = ewmh_AtomGetByName(FW_W(fwin),
 			     "_NET_WM_STATE",
 			     EWMH_ATOM_LIST_CLIENT_WIN,
 			     &size);
@@ -267,12 +270,12 @@ int ewmh_WMState(EWMH_CMD_ARGS)
 
 #if DEBUG_EWMH_INIT_STATE
     if (size != 0)
-      fprintf(stderr,"Window 0x%lx has an init _NET_WM_STATE hint\n",fwin->w);
+      fprintf(stderr,"Window 0x%lx has an init _NET_WM_STATE hint\n",FW_W(fwin));
 #endif
     while(list->name != NULL)
     {
       has_hint = 0;
-      for(i = 0; i < (size / (sizeof(Atom))); i++) 
+      for(i = 0; i < (size / (sizeof(Atom))); i++)
       {
 	if (list->atom == val[i])
 	  has_hint = 1;
@@ -290,12 +293,12 @@ int ewmh_WMState(EWMH_CMD_ARGS)
     int max_vert = (maximize & EWMH_MAXIMIZE_VERT)? 100:0;
     int max_horiz = (maximize & EWMH_MAXIMIZE_HORIZ)? 100:0;
     char cmd[256];
-    
+
     if (maximize & EWMH_MAXIMIZE_REMOVE)
-      sprintf(cmd,"%s", "Off");
+      sprintf(cmd,"Maximize off");
     else
-      sprintf(cmd,"%s %i %i", "On", max_horiz, max_vert);
-    CMD_Maximize(ev, fwin->w, fwin, C_WINDOW, cmd, NULL);
+      sprintf(cmd,"Maximize on %i %i", max_horiz, max_vert);
+    execute_function_override_window(NULL, NULL, cmd, 0, fwin);
   }
   return 0;
 }
@@ -340,6 +343,14 @@ int ewmh_WMStateHidden(EWMH_CMD_ARGS)
 {
   if (ev == NULL && style == NULL)
   {
+    unsigned long do_restore = any;
+
+    if (do_restore)
+    {
+      if (HAS_EWMH_INIT_HIDDEN_STATE(fwin) == EWMH_STATE_HAS_HINT)
+	return True;
+      return False;
+    }
     return IS_ICONIFIED(fwin);
   }
 
@@ -360,9 +371,9 @@ int ewmh_WMStateHidden(EWMH_CMD_ARGS)
     }
     if (!DO_EWMH_IGNORE_STATE_HINTS(style))
     {
-      SFSET_DO_START_ICONIC(*style, 1);
-      SMSET_DO_START_ICONIC(*style, 1);
-      SCSET_DO_START_ICONIC(*style, 1);
+      style->flags.do_start_iconic = 1;
+      style->flag_mask.do_start_iconic = 1;
+      style->change_mask.do_start_iconic = 1;
     }
     SET_HAS_EWMH_INIT_HIDDEN_STATE(fwin, EWMH_STATE_HAS_HINT);
     return 0;
@@ -371,21 +382,21 @@ int ewmh_WMStateHidden(EWMH_CMD_ARGS)
   if (ev != NULL)
   {
     /* client message */
-    char cmd[8];
+    char cmd[16];
     int bool_arg = ev->xclient.data.l[0];
 
     if ((bool_arg == NET_WM_STATE_TOGGLE && !IS_ICONIFIED(fwin)) ||
 	bool_arg == NET_WM_STATE_ADD)
     {
       /* iconify */
-      sprintf(cmd,"%s", "On");
+      sprintf(cmd, "Iconify on");
     }
     else
     {
       /* deiconify */
-      sprintf(cmd,"%s", "Off");
+      sprintf(cmd, "Iconify off");
     }
-    CMD_Iconify(ev, fwin->w, fwin, C_WINDOW, cmd, NULL);
+    execute_function_override_window(NULL, NULL, cmd, 0, fwin);
   }
   return 0;
 }
@@ -483,7 +494,7 @@ int ewmh_WMStateModal(EWMH_CMD_ARGS)
       if (HAS_EWMH_INIT_MODAL_STATE(fwin) == EWMH_STATE_HAS_HINT)
 	return True;
       return False;
-    } 
+    }
     return IS_EWMH_MODAL(fwin);
   }
 
@@ -494,7 +505,7 @@ int ewmh_WMStateModal(EWMH_CMD_ARGS)
     if (has_hint)
       fprintf(stderr,"\t Modal %i\n", HAS_EWMH_INIT_MODAL_STATE(fwin));
 #endif
-    if (!has_hint && 
+    if (!has_hint &&
 	HAS_EWMH_INIT_MODAL_STATE(fwin) != EWMH_STATE_HAS_HINT)
     {
       SET_HAS_EWMH_INIT_MODAL_STATE(fwin, EWMH_STATE_NO_HINT);
@@ -507,18 +518,18 @@ int ewmh_WMStateModal(EWMH_CMD_ARGS)
       SET_EWMH_MODAL(fwin, True);
       /* the window is a modal transient window so we grab the focus
        * it will be good to raise it but ... */
-      SFSET_DO_GRAB_FOCUS_WHEN_TRANSIENT_CREATED(*style, 1);
-      SMSET_DO_GRAB_FOCUS_WHEN_TRANSIENT_CREATED(*style, 1);
-      SCSET_DO_GRAB_FOCUS_WHEN_TRANSIENT_CREATED(*style, 1);
+      FPS_GRAB_FOCUS_TRANSIENT(S_FOCUS_POLICY(SCF(*style)), 1);
+      FPS_GRAB_FOCUS_TRANSIENT(S_FOCUS_POLICY(SCM(*style)), 1);
+      FPS_GRAB_FOCUS_TRANSIENT(S_FOCUS_POLICY(SCC(*style)), 1);
     }
     else
     {
       SET_EWMH_MODAL(fwin, False);
-      if (!DO_GRAB_FOCUS_TRANSIENT(style))
+      if (!FP_DO_GRAB_FOCUS_TRANSIENT(S_FOCUS_POLICY(SCF(*style))))
       {
-	SFSET_DO_GRAB_FOCUS_WHEN_TRANSIENT_CREATED(*style, 0);
-	SMSET_DO_GRAB_FOCUS_WHEN_TRANSIENT_CREATED(*style, 1);
-	SCSET_DO_GRAB_FOCUS_WHEN_TRANSIENT_CREATED(*style, 1);
+	FPS_GRAB_FOCUS_TRANSIENT(S_FOCUS_POLICY(SCF(*style)), 0);
+	FPS_GRAB_FOCUS_TRANSIENT(S_FOCUS_POLICY(SCM(*style)), 1);
+	FPS_GRAB_FOCUS_TRANSIENT(S_FOCUS_POLICY(SCC(*style)), 1);
       }
     }
     SET_HAS_EWMH_INIT_MODAL_STATE(fwin, EWMH_STATE_HAS_HINT);
@@ -545,6 +556,14 @@ int ewmh_WMStateShaded(EWMH_CMD_ARGS)
 
   if (ev == NULL && style == NULL)
   {
+    unsigned long do_restore = any;
+
+    if (do_restore)
+    {
+      if (HAS_EWMH_INIT_SHADED_STATE(fwin) == EWMH_STATE_HAS_HINT)
+	return True;
+      return False;
+    }
     return IS_SHADED(fwin);
   }
 
@@ -566,6 +585,7 @@ int ewmh_WMStateShaded(EWMH_CMD_ARGS)
     if (!DO_EWMH_IGNORE_STATE_HINTS(style))
     {
       SET_SHADED(fwin, 1);
+      SET_SHADED_DIR(fwin, GET_TITLE_DIR(fwin));
     }
     SET_HAS_EWMH_INIT_SHADED_STATE(fwin, EWMH_STATE_HAS_HINT);
     return 0;
@@ -577,11 +597,11 @@ int ewmh_WMStateShaded(EWMH_CMD_ARGS)
     if ((bool_arg == NET_WM_STATE_TOGGLE && !IS_SHADED(fwin)) ||
 	bool_arg == NET_WM_STATE_ADD)
     {
-      CMD_WindowShade(ev, fwin->w, fwin, C_WINDOW, "True", NULL);
+      execute_function_override_window(NULL, NULL, "Windowshade on", 0, fwin);
     }
     else
     {
-      CMD_WindowShade(ev, fwin->w, fwin, C_WINDOW, "False", NULL);
+      execute_function_override_window(NULL, NULL, "Windowshade off", 0, fwin);
     }
   }
   return 0;
@@ -598,7 +618,7 @@ int ewmh_WMStateSkipPager(EWMH_CMD_ARGS)
       if (HAS_EWMH_INIT_SKIP_PAGER_STATE(fwin) == EWMH_STATE_HAS_HINT)
 	return True;
       return False;
-    } 
+    }
     return DO_SKIP_WINDOW_LIST(fwin);
   }
 
@@ -625,24 +645,24 @@ int ewmh_WMStateSkipPager(EWMH_CMD_ARGS)
 
     if (!DO_EWMH_IGNORE_STATE_HINTS(style))
     {
-      SFSET_DO_WINDOW_LIST_SKIP(*style, 1);
-      SMSET_DO_WINDOW_LIST_SKIP(*style, 1);
-      SCSET_DO_WINDOW_LIST_SKIP(*style, 1);
+      S_SET_DO_WINDOW_LIST_SKIP(SCF(*style), 1);
+      S_SET_DO_WINDOW_LIST_SKIP(SCM(*style), 1);
+      S_SET_DO_WINDOW_LIST_SKIP(SCC(*style), 1);
     }
     else if (!DO_SKIP_WINDOW_LIST(style))
     {
-      SFSET_DO_WINDOW_LIST_SKIP(*style, 0);
-      SMSET_DO_WINDOW_LIST_SKIP(*style, 1);
-      SCSET_DO_WINDOW_LIST_SKIP(*style, 1);
+      S_SET_DO_WINDOW_LIST_SKIP(SCF(*style), 0);
+      S_SET_DO_WINDOW_LIST_SKIP(SCM(*style), 1);
+      S_SET_DO_WINDOW_LIST_SKIP(SCC(*style), 1);
     }
     SET_HAS_EWMH_INIT_SKIP_PAGER_STATE(fwin, EWMH_STATE_HAS_HINT);
     return 0;
   }
-  
+
   {
     /* I do not think we can get such client message */
     int bool_arg = ev->xclient.data.l[0];
-    
+
     if ((bool_arg == NET_WM_STATE_TOGGLE && !DO_SKIP_WINDOW_LIST(fwin)) ||
 	bool_arg == NET_WM_STATE_ADD)
     {
@@ -665,7 +685,7 @@ int ewmh_WMStateSkipTaskBar(EWMH_CMD_ARGS)
       if (HAS_EWMH_INIT_SKIP_TASKBAR_STATE(fwin) == EWMH_STATE_HAS_HINT)
 	return True;
       return False;
-    } 
+    }
     return DO_SKIP_WINDOW_LIST(fwin);
   }
 
@@ -692,24 +712,24 @@ int ewmh_WMStateSkipTaskBar(EWMH_CMD_ARGS)
 
     if (!DO_EWMH_IGNORE_STATE_HINTS(style))
     {
-      SFSET_DO_WINDOW_LIST_SKIP(*style, 1);
-      SMSET_DO_WINDOW_LIST_SKIP(*style, 1);
-      SCSET_DO_WINDOW_LIST_SKIP(*style, 1);
+      S_SET_DO_WINDOW_LIST_SKIP(SCF(*style), 1);
+      S_SET_DO_WINDOW_LIST_SKIP(SCM(*style), 1);
+      S_SET_DO_WINDOW_LIST_SKIP(SCC(*style), 1);
     }
     else if (!DO_SKIP_WINDOW_LIST(style))
     {
-      SFSET_DO_WINDOW_LIST_SKIP(*style, 0);
-      SMSET_DO_WINDOW_LIST_SKIP(*style, 1);
-      SCSET_DO_WINDOW_LIST_SKIP(*style, 1);
+      S_SET_DO_WINDOW_LIST_SKIP(SCF(*style), 0);
+      S_SET_DO_WINDOW_LIST_SKIP(SCM(*style), 1);
+      S_SET_DO_WINDOW_LIST_SKIP(SCC(*style), 1);
     }
     SET_HAS_EWMH_INIT_SKIP_TASKBAR_STATE(fwin, EWMH_STATE_HAS_HINT);
     return 0;
   }
-  
+
   {
     /* I do not think we can get such client message */
     int bool_arg = ev->xclient.data.l[0];
-    
+
     if ((bool_arg == NET_WM_STATE_TOGGLE && !DO_SKIP_WINDOW_LIST(fwin)) ||
 	bool_arg == NET_WM_STATE_ADD)
     {
@@ -791,7 +811,15 @@ int ewmh_WMStateSticky(EWMH_CMD_ARGS)
 
   if (ev == NULL && style == NULL)
   {
-    return IS_STICKY(fwin);
+    unsigned long do_restore = any;
+
+    if (do_restore)
+    {
+      if (HAS_EWMH_INIT_STICKY_STATE(fwin) == EWMH_STATE_HAS_HINT)
+	return True;
+      return False;
+    }
+    return (IS_STICKY_ON_PAGE(fwin) && IS_STICKY_ON_DESK(fwin));
   }
 
   if (ev == NULL && style != NULL)
@@ -811,9 +839,12 @@ int ewmh_WMStateSticky(EWMH_CMD_ARGS)
     }
     if (!DO_EWMH_IGNORE_STATE_HINTS(style))
     {
-      SFSET_IS_STICKY(*style, 1);
-      SMSET_IS_STICKY(*style, 1);
-      SCSET_IS_STICKY(*style, 1);
+      S_SET_IS_STICKY_ON_PAGE(SCF(*style), 1);
+      S_SET_IS_STICKY_ON_PAGE(SCM(*style), 1);
+      S_SET_IS_STICKY_ON_PAGE(SCC(*style), 1);
+      S_SET_IS_STICKY_ON_DESK(SCF(*style), 1);
+      S_SET_IS_STICKY_ON_DESK(SCM(*style), 1);
+      S_SET_IS_STICKY_ON_DESK(SCC(*style), 1);
     }
     SET_HAS_EWMH_INIT_STICKY_STATE(fwin, EWMH_STATE_HAS_HINT);
     return 0;
@@ -822,14 +853,15 @@ int ewmh_WMStateSticky(EWMH_CMD_ARGS)
   {
     /* client message */
     int bool_arg = ev->xclient.data.l[0];
-    if ((bool_arg == NET_WM_STATE_TOGGLE && !IS_STICKY(fwin)) ||
+    if ((bool_arg == NET_WM_STATE_TOGGLE &&
+	 (!IS_STICKY_ON_PAGE(fwin) || !IS_STICKY_ON_DESK(fwin))) ||
 	bool_arg == NET_WM_STATE_ADD)
     {
-      CMD_Stick(ev, fwin->w, fwin, C_WINDOW, "On", NULL);
+      execute_function_override_window(NULL, NULL, "Stick on", 0, fwin);
     }
     else
     {
-      CMD_Stick(ev, fwin->w, fwin, C_WINDOW, "Off", NULL);
+      execute_function_override_window(NULL, NULL, "Stick on", 1, fwin);
     }
   }
   return 0;
@@ -847,7 +879,7 @@ int ewmh_WMIconGeometry(EWMH_CMD_ARGS)
   /* FIXME: After a (un)silde of kicker the geometry are wrong (not because
    * we set the geometry just after the property notify). This does
    * not happen with kwin */
-  val = ewmh_AtomGetByName(fwin->w, "_NET_WM_ICON_GEOMETRY",
+  val = ewmh_AtomGetByName(FW_W(fwin), "_NET_WM_ICON_GEOMETRY",
 			   EWMH_ATOM_LIST_PROPERTY_NOTIFY, &size);
 
   if (val == NULL)
@@ -869,19 +901,19 @@ int ewmh_WMIconGeometry(EWMH_CMD_ARGS)
 }
 
 /**** for animation ****/
-int EWMH_GetIconGeometry(FvwmWindow *fwin, rectangle *icon_rect)
+void EWMH_GetIconGeometry(FvwmWindow *fwin, rectangle *icon_rect)
 {
   if (!IS_ICON_SUPPRESSED(fwin) ||
       (fwin->ewmh_icon_geometry.x == 0 &&
        fwin->ewmh_icon_geometry.y == 0 &&
        fwin->ewmh_icon_geometry.width == 0 &&
        fwin->ewmh_icon_geometry.height == 0))
-    return 0;
+    return;
   icon_rect->x = fwin->ewmh_icon_geometry.x;
   icon_rect->y = fwin->ewmh_icon_geometry.y;
   icon_rect->width = fwin->ewmh_icon_geometry.width;
   icon_rect->height = fwin->ewmh_icon_geometry.height;
-  return 1;
+  return;
 }
 
 int ewmh_WMStrut(EWMH_CMD_ARGS)
@@ -897,7 +929,7 @@ int ewmh_WMStrut(EWMH_CMD_ARGS)
     fwin->dyn_strut.bottom = fwin->strut.bottom = 0;
   }
 
-  val = ewmh_AtomGetByName(fwin->w, "_NET_WM_STRUT",
+  val = ewmh_AtomGetByName(FW_W(fwin), "_NET_WM_STRUT",
 			   EWMH_ATOM_LIST_PROPERTY_NOTIFY, &size);
 
   if (val == NULL)
@@ -929,13 +961,15 @@ int ewmh_WMStrut(EWMH_CMD_ARGS)
 }
 
 /* ************************************************************************* *
- * 
+ *
  * ************************************************************************* */
-Bool EWMH_ProcessClientMessage(FvwmWindow *fwin, XEvent *ev)
+Bool EWMH_ProcessClientMessage(const exec_context_t *exc)
 {
   ewmh_atom *ewmh_a = NULL;
+  FvwmWindow *fwin = exc->w.fw;
+  XEvent *ev = exc->x.elast;
 
-  if ((ewmh_a = 
+  if ((ewmh_a =
        (ewmh_atom *)ewmh_GetEwmhAtomByAtom(ev->xclient.message_type,
 					   EWMH_ATOM_LIST_CLIENT_ROOT)) != NULL)
   {
@@ -961,19 +995,21 @@ Bool EWMH_ProcessClientMessage(FvwmWindow *fwin, XEvent *ev)
 }
 
 /* ************************************************************************* *
- * 
+ *
  * ************************************************************************* */
-void EWMH_ProcessPropertyNotify(FvwmWindow *fwin, XEvent *ev)
+void EWMH_ProcessPropertyNotify(const exec_context_t *exc)
 {
   ewmh_atom *ewmh_a = NULL;
+  FvwmWindow *fwin = exc->w.fw;
+  XEvent *ev = exc->x.elast;
 
-  if ((ewmh_a = 
+  if ((ewmh_a =
        (ewmh_atom *)ewmh_GetEwmhAtomByAtom(ev->xproperty.atom,
 				EWMH_ATOM_LIST_PROPERTY_NOTIFY)) != NULL)
   {
     if (ewmh_a->action != None)
     {
-      flush_property_notify(ewmh_a->atom, fwin->w);
+      flush_property_notify(ewmh_a->atom, FW_W(fwin));
       ewmh_a->action(fwin, ev, NULL, 0);
     }
   }
