@@ -25,6 +25,7 @@
 #include "libs/FShape.h"
 #include "libs/PictureBase.h"
 #include "libs/Picture.h"
+#include "libs/PictureUtils.h"
 #include "libs/PictureImageLoader.h"
 #include "libs/FRenderInit.h"
 #include "fvwm.h"
@@ -183,7 +184,6 @@ CARD32 *ewmh_SetWmIconFromPixmap(FvwmWindow *fwin,
 				 Bool is_mini_icon)
 {
 	CARD32 *new_icon = NULL;
-	unsigned char *c_new_icon;
 	int keep_start = 0, keep_length = 0;
 	int width = 0, height = 0;
 	unsigned int i,j,k,l,m;
@@ -200,6 +200,9 @@ CARD32 *ewmh_SetWmIconFromPixmap(FvwmWindow *fwin,
 	Pixmap save_icon_pixmap = None;
 	Pixmap save_icon_mask = None;
 	Pixmap save_icon_alpha = None;
+	int save_icon_nalloc_pixels = 0;
+	Pixel *save_icon_alloc_pixels = NULL;
+	int save_icon_no_limit = 0;
 	Window save_icon_pixmap_w = None;
 	Bool is_pixmap_ours = False;
 	Bool is_icon_ours = False;
@@ -230,6 +233,9 @@ CARD32 *ewmh_SetWmIconFromPixmap(FvwmWindow *fwin,
 		save_icon_pixmap = fwin->iconPixmap;
 		save_icon_mask = fwin->icon_maskPixmap;
 		save_icon_alpha = fwin->icon_alphaPixmap;
+		save_icon_nalloc_pixels = fwin->icon_nalloc_pixels;
+		save_icon_alloc_pixels = fwin->icon_alloc_pixels;
+		save_icon_no_limit = fwin->icon_no_limit;
 		save_icon_pixmap_w =  FW_W_ICON_PIXMAP(fwin);
 		is_pixmap_ours = IS_PIXMAP_OURS(fwin);
 		is_icon_ours = IS_ICON_OURS(fwin);
@@ -242,6 +248,17 @@ CARD32 *ewmh_SetWmIconFromPixmap(FvwmWindow *fwin,
 		alpha = fwin->icon_alphaPixmap;
 		width = fwin->icon_g.picture_w_g.width;
 		height = fwin->icon_g.picture_w_g.height;
+		if (fwin->icon_alloc_pixels != NULL)
+		{
+			if (fwin->icon_nalloc_pixels != 0)
+			{
+				PictureFreeColors(
+					dpy, Pcmap, fwin->icon_alloc_pixels,
+					fwin->icon_nalloc_pixels, 0,
+					fwin->icon_no_limit);
+			}
+			free(fwin->icon_alloc_pixels);
+		}
 
 		fwin->icon_g.picture_w_g.width = save_picture_w_g_width;
 		fwin->icon_g.picture_w_g.height = save_picture_w_g_height;
@@ -249,6 +266,9 @@ CARD32 *ewmh_SetWmIconFromPixmap(FvwmWindow *fwin,
 		fwin->iconPixmap = save_icon_pixmap;
 		fwin->icon_maskPixmap = save_icon_mask;
 		fwin->icon_alphaPixmap = save_icon_alpha;
+		fwin->icon_nalloc_pixels = save_icon_nalloc_pixels;
+		fwin->icon_alloc_pixels = save_icon_alloc_pixels;
+		fwin->icon_no_limit = save_icon_no_limit;
 		FW_W_ICON_PIXMAP(fwin) = save_icon_pixmap_w;
 		SET_ICON_OURS(fwin, is_icon_ours);
 		SET_PIXMAP_OURS(fwin, is_pixmap_ours);
@@ -327,9 +347,8 @@ CARD32 *ewmh_SetWmIconFromPixmap(FvwmWindow *fwin,
 	new_icon[keep_length] = width;
 	new_icon[1+keep_length] = height;
 
-	c_new_icon = (unsigned char *)new_icon;
 	k = 0;
-	l = 4 * (2 + keep_length);
+	l = (2 + keep_length);
 	m = 0;
 
 	switch(image->depth)
@@ -337,18 +356,17 @@ CARD32 *ewmh_SetWmIconFromPixmap(FvwmWindow *fwin,
 	case 1:
 	{
 		XColor colors[2];
-		unsigned short fg[3];
-		unsigned short bg[3];
+		CARD32 fg,bg;
 
 		colors[0].pixel = fwin->colors.fore;
 		colors[1].pixel = fwin->colors.back;
 		XQueryColors(dpy, Pcmap, colors, 2);
-		fg[0] = colors[0].blue >> 8;
-		fg[1] = colors[0].green >> 8;
-		fg[2] = colors[0].red >> 8;
-		bg[0] = colors[1].blue >> 8;
-		bg[1] = colors[1].green >> 8;
-		bg[2] = colors[1].red >> 8;
+		fg = 0xff000000 + (((colors[0].red >> 8) & 0xff) << 16) +
+			(((colors[0].green >> 8) & 0xff) << 8) +
+			((colors[0].blue >> 8) & 0xff);
+		bg = 0xff000000 + (((colors[1].red >> 8) & 0xff) << 16) +
+			(((colors[1].green >> 8) & 0xff) << 8) +
+			((colors[1].blue >> 8) & 0xff);
 		for (j = 0; j < height; j++)
 		{
 			for (i = 0; i < width; i++)
@@ -356,26 +374,17 @@ CARD32 *ewmh_SetWmIconFromPixmap(FvwmWindow *fwin,
 				if (m_image != NULL &&
 				    (XGetPixel(m_image, i, j) == 0))
 				{
-					c_new_icon[l++] = 0;
-					c_new_icon[l++] = 0;
-					c_new_icon[l++] = 0;
-					c_new_icon[l++] = 0;
+					new_icon[l++] = 0;
 				}
 				else
 				{
 					if (XGetPixel(image, i, j) == 0)
 					{
-						c_new_icon[l++] = bg[0];
-						c_new_icon[l++] = bg[1];
-						c_new_icon[l++] = bg[2];
-						c_new_icon[l++] = 255;
+						new_icon[l++] = bg;
 					}
 					else
 					{
-						c_new_icon[l++] = fg[0];
-						c_new_icon[l++] = fg[1];
-						c_new_icon[l++] = fg[2];
-						c_new_icon[l++] = 255;
+						new_icon[l++] = fg;
 					}
 				}
 			}
@@ -425,18 +434,18 @@ CARD32 *ewmh_SetWmIconFromPixmap(FvwmWindow *fwin,
 			{
 				if (cm[m] > 0)
 				{
-					c_new_icon[l++] = colors[k].blue >> 8;
-					c_new_icon[l++] = colors[k].green >> 8;
-					c_new_icon[l++] = colors[k].red >> 8;
-					c_new_icon[l++] = cm[m];
+					new_icon[l++] =
+						((cm[m] & 0xff) << 24) +
+						(((colors[k].red >> 8) & 0xff)
+						 << 16) +
+						(((colors[k].green >> 8) & 0xff)
+						 << 8) +
+						((colors[k].blue >> 8) & 0xff);
 					k++;
 				}
 				else
 				{
-					c_new_icon[l++] = 0;
-					c_new_icon[l++] = 0;
-					c_new_icon[l++] = 0;
-					c_new_icon[l++] = 0;
+					new_icon[l++] = 0;
 				}
 				m++;
 			}
@@ -447,22 +456,6 @@ CARD32 *ewmh_SetWmIconFromPixmap(FvwmWindow *fwin,
 	}
 	} /* switch */
 
-#if 0 /* this is dramatically slow */
-	for (j = 0; j < height; j++)
-		for (i = 0; i < width; i++)
-		{
-			c.pixel = XGetPixel(image, i, j);
-			XQueryColor(dpy, Pcmap, &c);
-			k = 4*(i + j*width +2 +keep_length);
-			c_new_icon[k] = (unsigned short)c.blue >> 8;
-			c_new_icon[k+1] = (unsigned short)c.green >> 8;
-			c_new_icon[k+2] = (unsigned short)c.red >> 8;
-			if (m_image != None && (XGetPixel(m_image, i, j) == 0))
-				c_new_icon[k+3] = 0;
-			else
-				c_new_icon[k+3] = 255;
-		}
-#endif
 
 	if (is_mini_icon)
 	{
@@ -661,6 +654,9 @@ int EWMH_SetIconFromWMIcon(FvwmWindow *fwin, CARD32 *list, unsigned int size,
 	Pixmap alpha = None;
 	Bool free_list = False;
 	int have_alpha;
+	int nalloc_pixels;
+	Pixel *alloc_pixels;
+	int no_limit;
 	FvwmPictureAttributes fpa;
 
 	if (list == NULL)
@@ -710,8 +706,9 @@ int EWMH_SetIconFromWMIcon(FvwmWindow *fwin, CARD32 *list, unsigned int size,
 			FRenderGetAlphaDepth());
 	}
 	if (!PImageCreatePixmapFromArgbData(
-		dpy, Scr.Root, (unsigned char *)list, start, width, height,
-		pixmap, mask, alpha, &have_alpha, fpa) || pixmap == None)
+		dpy, Scr.Root, list, start, width, height,
+		pixmap, mask, alpha, &have_alpha, &nalloc_pixels, &alloc_pixels,
+		&no_limit, fpa) || pixmap == None)
 	{
 		fvwm_msg(ERR, "EWMH_SetIconFromWMIcon",
 			 "fail to create a pixmap\n");
@@ -780,7 +777,10 @@ int EWMH_SetIconFromWMIcon(FvwmWindow *fwin, CARD32 *list, unsigned int size,
 							      Scr.NoFocusWin,
 							      name,
 							      pixmap,mask,alpha,
-							      width, height);
+							      width, height,
+							      nalloc_pixels,
+							      alloc_pixels,
+							      no_limit);
 		if (fwin->mini_icon != NULL)
 		{
 			fwin->mini_pixmap_file = name;
@@ -797,6 +797,9 @@ int EWMH_SetIconFromWMIcon(FvwmWindow *fwin, CARD32 *list, unsigned int size,
 		fwin->iconPixmap = pixmap;
 		fwin->icon_maskPixmap = mask;
 		fwin->icon_alphaPixmap = alpha;
+		fwin->icon_nalloc_pixels = nalloc_pixels;
+		fwin->icon_alloc_pixels = alloc_pixels;
+		fwin->icon_no_limit = no_limit;
 		fwin->icon_g.picture_w_g.width = width;
 		fwin->icon_g.picture_w_g.height = height;
 		fwin->iconDepth = Pdepth;
