@@ -25,25 +25,25 @@
 /* ---------------------------- included header files ----------------------- */
 
 #include <config.h>
+
 #include <stdio.h>
 
 #include "libs/fvwmlib.h"
-#include <libs/gravity.h>
 #include "fvwm.h"
 #include "externs.h"
-#include "cursor.h"
-#include "functions.h"
+#include "execcontext.h"
+#include "eventhandler.h"
 #include "bindings.h"
 #include "misc.h"
 #include "screen.h"
 #include "focus.h"
-#include "icons.h"
 #include "borders.h"
 #include "frame.h"
 #include "virtual.h"
 #include "stack.h"
 #include "geometry.h"
 #include "colormaps.h"
+#include "add_window.h"
 
 /* ---------------------------- local definitions --------------------------- */
 
@@ -80,26 +80,6 @@ static FvwmWindow *LastScreenFocus = NULL;
 
 /* ---------------------------- local functions ----------------------------- */
 
-static void SetPointerEventPosition(XEvent *eventp, int x, int y)
-{
-	switch (eventp->type)
-	{
-	case ButtonPress:
-	case ButtonRelease:
-	case KeyPress:
-	case KeyRelease:
-	case MotionNotify:
-		eventp->xbutton.x_root = x;
-		eventp->xbutton.y_root = y;
-		eventp->xmotion.same_screen = True;
-		break;
-	default:
-		break;
-	}
-
-	return;
-}
-
 static Bool focus_get_fpol_context_flag(
 	fpol_context_t *fpol_context, int context)
 {
@@ -124,20 +104,19 @@ static Bool focus_get_fpol_context_flag(
 /********************************************************************
  * Helper functions for setting the focus
  ********************************************************************/
-static void __try_program_focus(Window w, FvwmWindow *fw)
+static void __try_program_focus(Window w, const FvwmWindow *fw)
 {
-	extern Time lastTimestamp;
-
 	if (fw && WM_TAKES_FOCUS(fw) &&
 	    FP_DO_FOCUS_BY_PROGRAM(FW_FOCUS_POLICY(fw)))
 	{
-		send_clientmessage(dpy, w, _XA_WM_TAKE_FOCUS, lastTimestamp);
+		send_clientmessage(dpy, w, _XA_WM_TAKE_FOCUS, fev_get_evtime());
 	}
 
 	return;
 }
 
-static Bool __try_forbid_user_focus(Window w, FvwmWindow *fw)
+static Bool __try_forbid_user_focus(
+	Window w, FvwmWindow *fw)
 {
 	if (fw == NULL ||
 	    fpol_query_allow_user_focus(&FW_FOCUS_POLICY(fw)) == True)
@@ -187,8 +166,7 @@ static Bool __check_allow_focus(
 }
 
 static void __update_windowlist(
-	FvwmWindow *fw, fpol_set_focus_by_t set_by,
-	int is_focus_by_focus_cmd)
+	FvwmWindow *fw, fpol_set_focus_by_t set_by, int is_focus_by_focus_cmd)
 {
 	lastFocusType = !is_focus_by_focus_cmd;
 	if (fw == NULL || focus_is_focused(fw) || fw == &Scr.FvwmRoot)
@@ -264,7 +242,7 @@ static void __update_windowlist(
 	return;
 }
 
-static Bool __try_other_screen_focus(FvwmWindow *fw)
+static Bool __try_other_screen_focus(const FvwmWindow *fw)
 {
 	if (fw == NULL && !Scr.flags.is_pointer_on_this_screen)
 	{
@@ -302,7 +280,7 @@ static void __set_focus_to_fwin(
 	{
 		return;
 	}
-	__update_windowlist(fw, args->is_focus_by_focus_cmd, args->set_by);
+	__update_windowlist(fw, args->set_by, args->is_focus_by_focus_cmd);
 	if (__try_other_screen_focus(fw) == True)
 	{
 		return;
@@ -437,12 +415,13 @@ static void set_focus_to_fwin(
  *
  *************************************************************************/
 static void warp_to_fvwm_window(
-	XEvent *eventp, FvwmWindow *t, int warp_x, int x_unit, int warp_y,
+	const exec_context_t *exc, int warp_x, int x_unit, int warp_y,
 	int y_unit)
 {
 	int dx,dy;
 	int cx,cy;
 	int x,y;
+	FvwmWindow *t = exc->w.fw;
 
 	if (t == (FvwmWindow *)0 ||
 	    (IS_ICONIFIED(t) && FW_W_ICON_TITLE(t) == None))
@@ -532,21 +511,21 @@ static void warp_to_fvwm_window(
 				(t->frame_g.height - 1) * (100 + warp_y) / 100;
 		}
 	}
-	XWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, x, y);
-	SetPointerEventPosition(eventp, x, y);
+	FWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, x, y);
 	RaiseWindow(t);
 
 	/* If the window is still not visible, make it visible! */
 	if (t->frame_g.x + t->frame_g.width  < 0 ||
 	    t->frame_g.y + t->frame_g.height < 0 ||
-	    t->frame_g.x >= Scr.MyDisplayWidth	 ||
+	    t->frame_g.x >= Scr.MyDisplayWidth ||
 	    t->frame_g.y >= Scr.MyDisplayHeight)
 	{
 		frame_setup_window(
 			t, 0, 0, t->frame_g.width, t->frame_g.height, False);
-		XWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, 2,2);
-		SetPointerEventPosition(eventp, 2, 2);
+		FWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, 2, 2);
 	}
+
+	return;
 }
 
 static Bool focus_query_grab_buttons(FvwmWindow *fw, Bool client_entered)
@@ -580,7 +559,7 @@ static Bool focus_query_grab_buttons(FvwmWindow *fw, Bool client_entered)
 }
 
 static FvwmWindow *__restore_focus_after_unmap(
-	FvwmWindow *fw, Bool do_skip_marked_transients)
+	const FvwmWindow *fw, Bool do_skip_marked_transients)
 {
 	FvwmWindow *t = NULL;
 	FvwmWindow *set_focus_to = NULL;
@@ -640,12 +619,8 @@ static void __activate_window_by_command(
 	int cy;
 	Bool do_not_warp;
 	sftfwin_args_t sf_args;
+	FvwmWindow * const fw = exc->w.fw;
 
-	if (DeferExecution(
-		    eventp, &w, &fw, &context, CRS_SELECT, ButtonRelease))
-	{
-		return;
-	}
 	sf_args.do_allow_force_broadcast = 1;
 	sf_args.is_focus_by_focus_cmd = !!is_focus_by_focus_cmd;
 	sf_args.set_by = FOCUS_SET_BY_FUNCTION;
@@ -703,7 +678,7 @@ static void __activate_window_by_command(
 			if (FP_DO_WARP_POINTER_ON_FOCUS_FUNC(
 				    FW_FOCUS_POLICY(fw)))
 			{
-				XWarpPointer(
+				FWarpPointer(
 					dpy, None, Scr.Root, 0, 0, 0, 0, 2, 2);
 			}
 		}
@@ -733,54 +708,26 @@ static void __activate_window_by_command(
 static void __focus_grab_one_button(
 	FvwmWindow *fw, int button, int grab_buttons)
 {
-	register Bool do_grab;
-	register unsigned int mods;
-	register unsigned int max = GetUnusedModifiers();
-	register unsigned int living_modifiers = ~max;
+	Bool do_grab;
 
-#if 0
-	/* RBW - Set flag for grab or ungrab according to how we were called. */
-	if (!is_focused)
-	{
-		do_grab = !!(grab_buttons & (1 << button));
-	}
-	else
-	{
-		do_grab = !(grab_buttons & (1 << button));
-	}
-#else
-	if ((grab_buttons & (1 << button)) ==
-	    (fw->grabbed_buttons & (1 << button)))
+	do_grab = (grab_buttons & (1 << button));
+	if ((do_grab & (1 << button)) == (fw->grabbed_buttons & (1 << button)))
 	{
 		return;
 	}
-	do_grab = !!(grab_buttons & (1 << button));
-#endif
-
-	/* handle all bindings for the dead modifiers */
-	for (mods = 0; mods <= max; mods++)
+	if (do_grab)
 	{
-		/* Since mods starts with 1 we don't need to test if
-		 * mods contains a dead modifier. Otherwise both, dead
-		 * and living modifiers would be zero ==> mods == 0 */
-		if (mods & living_modifiers)
-		{
-			continue;
-		}
-		if (do_grab)
-		{
-			XGrabButton(
-				dpy, button + 1, mods, FW_W_PARENT(fw), True,
-				ButtonPressMask, GrabModeSync, GrabModeAsync,
-				None, None);
-			/* Set window flags accordingly as we grab or ungrab. */
-			fw->grabbed_buttons |= (1 << button);
-		}
-		else
-		{
-			XUngrabButton(dpy, button + 1, mods, FW_W_PARENT(fw));
-			fw->grabbed_buttons &= ~(1 << button);
-		}
+		XGrabButton(
+			dpy, button + 1, AnyModifier, FW_W_PARENT(fw), True,
+			ButtonPressMask, GrabModeSync, GrabModeAsync, None,
+			None);
+		/* Set window flags accordingly as we grab or ungrab. */
+		fw->grabbed_buttons |= (1 << button);
+	}
+	else
+	{
+		XUngrabButton(dpy, button + 1, AnyModifier, FW_W_PARENT(fw));
+		fw->grabbed_buttons &= ~(1 << button);
 	}
 
 	return;
@@ -788,13 +735,13 @@ static void __focus_grab_one_button(
 
 /* ---------------------------- interface functions ------------------------- */
 
-Bool focus_does_accept_input_focus(FvwmWindow *fw)
+Bool focus_does_accept_input_focus(const FvwmWindow *fw)
 {
 	return (!fw || !fw->wmhints ||
 		!(fw->wmhints->flags & InputHint) || fw->wmhints->input);
 }
 
-Bool focus_is_focused(FvwmWindow *fw)
+Bool focus_is_focused(const FvwmWindow *fw)
 {
 	return (fw && fw == ScreenFocus);
 }
@@ -841,14 +788,7 @@ Bool focus_query_open_grab_focus(FvwmWindow *fw, FvwmWindow *focus_win)
 		/* Don't steal the focus from the current window */
 		return False;
 	}
-	if (IS_TRANSIENT(fw) && !XGetGeometry(
-		    dpy, FW_W_TRANSIENTFOR(fw), &JunkRoot, &JunkX, &JunkY,
-		    &JunkWidth, &JunkHeight, &JunkBW, &JunkDepth))
-	{
-		/* Gee, the transientfor does not exist! These evil application
-		 * programmers must hate us a lot ;-) */
-		FW_W_TRANSIENTFOR(fw) = Scr.Root;
-	}
+	validate_transientfor(fw);
 	if (IS_TRANSIENT(fw) && FW_W_TRANSIENTFOR(fw) != Scr.Root)
 	{
 		if (focus_win != NULL &&
@@ -883,7 +823,7 @@ Bool focus_query_open_grab_focus(FvwmWindow *fw, FvwmWindow *focus_win)
 
 /* Returns true if the focus has to be restored to a different window after
  * unmapping. */
-Bool focus_query_close_release_focus(FvwmWindow *fw)
+Bool focus_query_close_release_focus(const FvwmWindow *fw)
 {
 	if (fw == NULL || fw != get_focus_window())
 	{
@@ -904,17 +844,19 @@ Bool focus_query_close_release_focus(FvwmWindow *fw)
 	return False;
 }
 
-void _focus_grab_buttons(FvwmWindow *fw, Bool client_entered)
+static void __focus_grab_buttons(FvwmWindow *fw, Bool client_entered)
 {
 	int i;
 	Bool do_grab_window = False;
-	int grab_buttons = Scr.buttons2grab;
+	int grab_buttons;
 
-	if (fw == NULL || IS_SCHEDULED_FOR_DESTROY(fw))
+	if (fw == NULL || IS_SCHEDULED_FOR_DESTROY(fw) || !IS_MAPPED(fw))
 	{
-		/* It's pointless to grab buttons on dying windows */
+		/* It is pointless to grab buttons on dying windows.  Buttons
+		 * can not be grabbed when the window is unmapped. */
 		return;
 	}
+	grab_buttons = Scr.buttons2grab;
 	do_grab_window = focus_query_grab_buttons(fw, client_entered);
 	if (do_grab_window == True)
 	{
@@ -923,10 +865,10 @@ void _focus_grab_buttons(FvwmWindow *fw, Bool client_entered)
 	if (grab_buttons != fw->grabbed_buttons)
 	{
 		MyXGrabServer(dpy);
-		for (i = 0; i < NUMBER_OF_MOUSE_BUTTONS; i++)
+		for (i = 0; i < NUMBER_OF_EXTENDED_MOUSE_BUTTONS; i++)
 		{
 			__focus_grab_one_button(fw, i, grab_buttons);
-		} /* for */
+		}
 		MyXUngrabServer (dpy);
 	}
 
@@ -935,12 +877,12 @@ void _focus_grab_buttons(FvwmWindow *fw, Bool client_entered)
 
 void focus_grab_buttons(FvwmWindow *fw)
 {
-        return _focus_grab_buttons(fw, False);
+        return __focus_grab_buttons(fw, False);
 }
 
 void focus_grab_buttons_client_entered(FvwmWindow *fw)
 {
-        return _focus_grab_buttons(fw, True);
+        return __focus_grab_buttons(fw, True);
 }
 
 void focus_grab_buttons_on_layer(int layer)
@@ -983,13 +925,13 @@ void _SetFocusWindow(
 	sf_args.do_force = 1;
 	sf_args.set_by = set_by;
         if (client_entered)
-                {
-                        sf_args.client_entered = 1;
-                }
-                else
-                {
-                        sf_args.client_entered = 0;
-                }
+	{
+		sf_args.client_entered = 1;
+	}
+	else
+	{
+		sf_args.client_entered = 0;
+	}
 	set_focus_to_fwin(FW_W(fw), fw, &sf_args);
 
 	return;
@@ -1043,7 +985,7 @@ void _ForceDeleteFocus(void)
 /* When a window is unmapped (or destroyed) this function takes care of
  * adjusting the focus window appropriately. */
 void restore_focus_after_unmap(
-	FvwmWindow *fw, Bool do_skip_marked_transients)
+	const FvwmWindow *fw, Bool do_skip_marked_transients)
 {
 	extern FvwmWindow *colormap_win;
 	FvwmWindow *set_focus_to = NULL;
@@ -1078,7 +1020,7 @@ void focus_grab_buttons_on_pointer_window(void)
 	Window w;
 	FvwmWindow *fw;
 
-	if (!XQueryPointer(
+	if (!FQueryPointer(
 		    dpy, Scr.Root, &JunkRoot, &w, &JunkX, &JunkY, &JunkX,
 		    &JunkY, &JunkMask))
 	{
@@ -1162,7 +1104,7 @@ void set_focus_model(FvwmWindow *fw)
  * applications that use the passive focus model but manage focus in their own
  * sub windows and should thus use the locally active focus model instead.
  * There are many examples like netscape or ddd. */
-void focus_force_refresh_focus(FvwmWindow *fw)
+void focus_force_refresh_focus(const FvwmWindow *fw)
 {
 	XWindowAttributes winattrs;
 
@@ -1180,7 +1122,7 @@ void focus_force_refresh_focus(FvwmWindow *fw)
 	return;
 }
 
-void refresh_focus(FvwmWindow *fw)
+void refresh_focus(const FvwmWindow *fw)
 {
 	Bool do_refresh = False;
 
@@ -1235,22 +1177,16 @@ void CMD_WarpToWindow(F_CMD_ARGS)
 	int val1, val2;
 
 	n = GetTwoArguments(action, &val1, &val2, &val1_unit, &val2_unit);
-	if (context != C_UNMANAGED)
+	if (exc->w.wcontext != C_UNMANAGED)
 	{
-		if (DeferExecution(
-			    eventp, &w, &fw, &context, CRS_SELECT,
-			    ButtonRelease))
-		{
-			return;
-		}
 		if (n == 2)
 		{
 			warp_to_fvwm_window(
-				eventp, fw, val1, val1_unit, val2, val2_unit);
+				exc, val1, val1_unit, val2, val2_unit);
 		}
 		else
 		{
-			warp_to_fvwm_window(eventp, fw, 0, 0, 0, 0);
+			warp_to_fvwm_window(exc, 0, 0, 0, 0);
 		}
 	}
 	else
@@ -1266,8 +1202,8 @@ void CMD_WarpToWindow(F_CMD_ARGS)
 			unsigned int wh;
 
 			if (!XGetGeometry(
-				    dpy, w, &JunkRoot, &wx, &wy, &ww, &wh,
-				    &JunkBW, &JunkDepth))
+				    dpy, exc->w.w, &JunkRoot, &wx, &wy, &ww,
+				    &wh, &JunkBW, &JunkDepth))
 			{
 				return;
 			}
@@ -1296,7 +1232,7 @@ void CMD_WarpToWindow(F_CMD_ARGS)
 				y += wh;
 			}
 		}
-		XWarpPointer(dpy, None, w, 0, 0, 0, 0, x, y);
+		FWarpPointer(dpy, None, exc->w.w, 0, 0, 0, 0, x, y);
 	}
 
 	return;
