@@ -127,7 +127,8 @@ const func_type func_table[] =
 	CMD_ENT("edgethickness", CMD_EdgeThickness, F_NOP, 0, 0, 0),
 	CMD_ENT("emulate", CMD_Emulate, F_EMULATE, 0, 0, 0),
 	CMD_ENT("escapefunc", CMD_EscapeFunc, F_ESCAPE_FUNC, 0, 0, 0),
-	CMD_ENT("ewmhbasestrut", CMD_EwmhBaseStrut, F_EWMH_BASE_STRUT, 0, 0, 0),
+	CMD_ENT("ewmhbasestruts", CMD_EwmhBaseStruts, F_EWMH_BASE_STRUTS, 0, 0,
+		0),
 	CMD_ENT("ewmhnumberofdesktops", CMD_EwmhNumberOfDesktops,
 		F_EWMH_NUMBER_OF_DESKTOPS, 0, 0, 0),
 	CMD_ENT("exec", CMD_Exec, F_EXEC, 0, 0, 0),
@@ -137,6 +138,7 @@ const func_type func_table[] =
 		FUNC_NEEDS_WINDOW, CRS_SELECT, ButtonRelease),
 	CMD_ENT("focus", CMD_Focus, F_FOCUS,
 		FUNC_NEEDS_WINDOW, CRS_SELECT, ButtonRelease),
+	CMD_ENT("focusstyle", CMD_FocusStyle, F_FOCUSSTYLE, 0, 0, 0),
 	CMD_ENT("function", CMD_Function, F_FUNCTION, 0, 0, 0),
 	CMD_ENT("globalopts", CMD_GlobalOpts, F_GLOBAL_OPTS, 0, 0, 0),
 #ifdef GNOME
@@ -161,6 +163,7 @@ const func_type func_table[] =
 	CMD_ENT("killmodule", CMD_KillModule, F_KILL_MODULE, 0, 0, 0),
 	CMD_ENT("layer", CMD_Layer, F_LAYER,
 		FUNC_NEEDS_WINDOW, CRS_SELECT, ButtonRelease),
+	CMD_ENT("localepath", CMD_LocalePath, F_LOCALE_PATH, 0, 0, 0),
 	CMD_ENT("lower", CMD_Lower, F_LOWER,
 		FUNC_NEEDS_WINDOW, CRS_SELECT, ButtonRelease),
 	CMD_ENT("maximize", CMD_Maximize, F_MAXIMIZE,
@@ -224,6 +227,8 @@ const func_type func_table[] =
 	CMD_ENT("resizemovemaximize", CMD_ResizeMoveMaximize,
 		F_RESIZEMOVE_MAXIMIZE, FUNC_NEEDS_WINDOW, CRS_RESIZE,
 		ButtonPress),
+	CMD_ENT("restacktransients", CMD_RestackTransients, F_RESTACKTRANSIENTS,
+		FUNC_NEEDS_WINDOW, CRS_SELECT, ButtonRelease),
 	CMD_ENT("restart", CMD_Restart, F_RESTART, 0, 0, 0),
 	CMD_ENT("savequitsession", CMD_SaveQuitSession, F_SAVE_QUIT_SESSION,
 		0, 0, 0),
@@ -250,6 +255,10 @@ const func_type func_table[] =
 		FUNC_NEEDS_WINDOW, CRS_SELECT, ButtonRelease),
 	CMD_ENT("stick", CMD_Stick, F_STICK,
 		FUNC_NEEDS_WINDOW, CRS_SELECT, ButtonRelease),
+	CMD_ENT("stickdesk", CMD_StickDesk, F_STICKDESK,
+		FUNC_NEEDS_WINDOW, CRS_SELECT, ButtonRelease),
+	CMD_ENT("stickpage", CMD_StickPage, F_STICKPAGE,
+		FUNC_NEEDS_WINDOW, CRS_SELECT, ButtonRelease),
 #ifdef HAVE_STROKE
 	CMD_ENT("stroke", CMD_Stroke, F_STROKE, 0, 0, 0),
 	CMD_ENT("strokefunc", CMD_StrokeFunc, F_STROKE_FUNC, 0, 0, 0),
