@@ -46,6 +46,7 @@
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
+#include "libs/PictureBase.h"
 #include "libs/PictureUtils.h"
 #include "libs/FRenderInit.h"
 #include "fvwm.h"
@@ -74,6 +75,7 @@
 #include "move_resize.h"
 #include "frame.h"
 #include "menus.h"
+#include "libs/FGettext.h"
 
 /* ---------------------------- local definitions --------------------------- */
 
@@ -589,7 +591,7 @@ void Done(int restart, char *command)
 					"Restart command parsing error in"
 					" (%s): [%s]", command, errorMsg);
 			}
-			else if (StrEquals(my_argv[0], "--pass-args"))
+			else if (strcmp(my_argv[0], "--pass-args") == 0)
 			{
 				if (n != 2)
 				{
@@ -834,6 +836,7 @@ InstallSignals(void)
 static void LoadDefaultLeftButton(DecorFace *df, int i)
 {
 	struct vector_coords *v = &df->u.vector;
+	int j = 0;
 
 	memset(&df->style, 0, sizeof(df->style));
 	DFS_FACE_TYPE(df->style) = DefaultVectorButton;
@@ -844,6 +847,8 @@ static void LoadDefaultLeftButton(DecorFace *df, int i)
 		v->num = 5;
 		v->x = safemalloc(sizeof(char) * v->num);
 		v->y = safemalloc(sizeof(char) * v->num);
+		v->xoff = safemalloc(sizeof(char) * v->num);
+		v->yoff = safemalloc(sizeof(char) * v->num);
 		v->c = safecalloc(v->num, sizeof(char));
 		v->x[0] = 22;
 		v->y[0] = 39;
@@ -863,6 +868,8 @@ static void LoadDefaultLeftButton(DecorFace *df, int i)
 		v->num = 5;
 		v->x = safemalloc(sizeof(char) * v->num);
 		v->y = safemalloc(sizeof(char) * v->num);
+		v->xoff = safemalloc(sizeof(char) * v->num);
+		v->yoff = safemalloc(sizeof(char) * v->num);
 		v->c = safecalloc(v->num, sizeof(char));
 		v->x[0] = 32;
 		v->y[0] = 45;
@@ -881,6 +888,8 @@ static void LoadDefaultLeftButton(DecorFace *df, int i)
 		v->num = 5;
 		v->x = safemalloc(sizeof(char) * v->num);
 		v->y = safemalloc(sizeof(char) * v->num);
+		v->xoff = safemalloc(sizeof(char) * v->num);
+		v->yoff = safemalloc(sizeof(char) * v->num);
 		v->c = safecalloc(v->num, sizeof(char));
 		v->x[0] = 49;
 		v->y[0] = 49;
@@ -900,6 +909,8 @@ static void LoadDefaultLeftButton(DecorFace *df, int i)
 		v->num = 5;
 		v->x = safemalloc(sizeof(char) * v->num);
 		v->y = safemalloc(sizeof(char) * v->num);
+		v->xoff = safemalloc(sizeof(char) * v->num);
+		v->yoff = safemalloc(sizeof(char) * v->num);
 		v->c = safecalloc(v->num, sizeof(char));
 		v->x[0] = 32;
 		v->y[0] = 45;
@@ -916,6 +927,12 @@ static void LoadDefaultLeftButton(DecorFace *df, int i)
 		v->c[4] = 1;
 		break;
 	}
+	/* set offsets to 0, for all buttons */
+	for(j = 0 ; j < v->num ; j++)
+	{
+	  v->xoff[j] = 0;
+	  v->yoff[j] = 0;
+	}
 
 	return;
 }
@@ -929,6 +946,7 @@ static void LoadDefaultLeftButton(DecorFace *df, int i)
 static void LoadDefaultRightButton(DecorFace *df, int i)
 {
 	struct vector_coords *v = &df->u.vector;
+	int j = 0;
 
 	memset(&df->style, 0, sizeof(df->style));
 	DFS_FACE_TYPE(df->style) = DefaultVectorButton;
@@ -939,6 +957,8 @@ static void LoadDefaultRightButton(DecorFace *df, int i)
 		v->num = 5;
 		v->x = safemalloc(sizeof(char) * v->num);
 		v->y = safemalloc(sizeof(char) * v->num);
+		v->xoff = safemalloc(sizeof(char) * v->num);
+		v->yoff = safemalloc(sizeof(char) * v->num);
 		v->c = safecalloc(v->num, sizeof(char));
 		v->x[0] = 25;
 		v->y[0] = 25;
@@ -958,6 +978,8 @@ static void LoadDefaultRightButton(DecorFace *df, int i)
 		v->num = 5;
 		v->x = safemalloc(sizeof(char) * v->num);
 		v->y = safemalloc(sizeof(char) * v->num);
+		v->xoff = safemalloc(sizeof(char) * v->num);
+		v->yoff = safemalloc(sizeof(char) * v->num);
 		v->c = safecalloc(v->num, sizeof(char));
 		v->x[0] = 39;
 		v->y[0] = 39;
@@ -977,6 +999,8 @@ static void LoadDefaultRightButton(DecorFace *df, int i)
 		v->num = 5;
 		v->x = safemalloc(sizeof(char) * v->num);
 		v->y = safemalloc(sizeof(char) * v->num);
+		v->xoff = safemalloc(sizeof(char) * v->num);
+		v->yoff = safemalloc(sizeof(char) * v->num);
 		v->c = safecalloc(v->num, sizeof(char));
 		v->x[0] = 49;
 		v->y[0] = 49;
@@ -996,6 +1020,8 @@ static void LoadDefaultRightButton(DecorFace *df, int i)
 		v->num = 5;
 		v->x = safemalloc(sizeof(char) * v->num);
 		v->y = safemalloc(sizeof(char) * v->num);
+		v->xoff = safemalloc(sizeof(char) * v->num);
+		v->yoff = safemalloc(sizeof(char) * v->num);
 		v->c = safecalloc(v->num, sizeof(char));
 		v->x[0] = 36;
 		v->y[0] = 36;
@@ -1012,6 +1038,12 @@ static void LoadDefaultRightButton(DecorFace *df, int i)
 		v->c[4] = 1;
 		break;
 	}
+	/* set offsets to 0, for all buttons */
+	for(j = 0 ; j < v->num ; j++)
+	{
+	  v->xoff[j] = 0;
+	  v->yoff[j] = 0;
+	}
 
 	return;
 }
@@ -1039,7 +1071,7 @@ static void CreateGCs(void)
 	Scr.ScratchGC3 = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
 	Scr.ScratchGC4 = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
 	Scr.TitleGC = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
-	Scr.TileGC = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
+	Scr.BordersGC = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
 	Scr.TransMaskGC = fvwmlib_XCreateGC(dpy, Scr.NoFocusWin, gcm, &gcv);
 	c.pixel = GetColor(DEFAULT_FORE_COLOR);
 	Scr.ScratchMonoPixmap = XCreatePixmap(dpy, Scr.Root, 1, 1, 1);
@@ -1099,7 +1131,8 @@ static void InitVariables(void)
 	/* Multiple desks are available even in non-virtual
 	 * compilations */
 	Scr.CurrentDesk = 0;
-	Scr.EdgeScrollX = Scr.EdgeScrollY = DEFAULT_EDGE_SCROLL;
+	Scr.EdgeScrollX = DEFAULT_EDGE_SCROLL * Scr.MyDisplayWidth / 100;
+	Scr.EdgeScrollY = DEFAULT_EDGE_SCROLL * Scr.MyDisplayHeight / 100;
 	Scr.ScrollResistance = DEFAULT_SCROLL_RESISTANCE;
 	Scr.MoveResistance = DEFAULT_MOVE_RESISTANCE;
 	Scr.XiMoveResistance = DEFAULT_XIMOVE_RESISTANCE;
@@ -1193,25 +1226,51 @@ static void InitVariables(void)
 	return;
 }
 
-static void usage(void)
+static void usage(int is_verbose)
 {
-	fprintf(stderr, "\nfvwm %s Usage:\n\n", VERSION);
-	fprintf(stderr, "  %s"
-		" [-d dpy]"
-		" [-debug]"
-		" [-f config_file]"
-		" [-cmd config_cmd]"
+	fprintf(stderr, "usage: %s", g_argv[0]);
+	fprintf(stderr,
+		" [-d display]"
+		" [-f cfgfile]"
+		" [-c cmd]"
 		" [-s]"
-		" [--version]"
-		" [-h]"
-		" [-replace]"
-		" [-clientId id]"
-		" [-restore file]"
-		" [-visualId id]"
-		" [-visual class]"
-		" [-color-limit]"
-		"\n\n",g_argv[0]);
-	exit(1);
+		" [-I vis-id | -C vis-class]"
+		" [-l colors"
+		" [-L|A|S|P] ...]"
+		" [-r]"
+		" [OTHER OPTIONS] ..."
+		"\n");
+	if (!is_verbose)
+	{
+		fprintf(
+			stderr, "Try '%s --help' for more information.\n",
+			g_argv[0]);
+		return;
+	}
+	fprintf(stderr,
+		" -A:           allocate palette\n"
+		" -c cmd:       preprocess configuration file with <cmd>\n"
+		" -C vis-class: use visual class <vis-class>\n"
+		" -d display:   run fvwm on <display>\n"
+		" -D:           enable debug oputput\n"
+		" -f cfgfile:   read configuration from <cfgfile>\n"
+		" -F file:      used internally for session management\n"
+		" -h, -?:       print this help message\n"
+		" -i client-id: used internally for session management\n"
+		" -I vis-id:    use visual <vis-id>\n"
+		" -l colors:    try to use no more than <colors> colors\n"
+		" -L:           strict color limit\n"
+		" -P:           visual palette\n"
+		" -r:           replace running window manager\n"
+		" -s:           manage a single screen\n"
+		" -S:           static palette\n"
+		" -V:           print version information\n"
+		);
+	fprintf(
+		stderr, "Try 'man %s' for more information.\n",
+		PACKAGE);
+
+		return;
 }
 
 static void setVersionInfo(void)
@@ -1291,28 +1350,35 @@ static void SetRCDefaults(void)
 {
 	int i;
 	/* set up default colors, fonts, etc */
-	char *defaults[] = {
-		"XORValue 0",
-		"DefaultFont",
-		"DefaultColors black grey",
-		DEFAULT_MENU_STYLE,
-		"TitleStyle Centered -- Raised",
-		"Style * Color lightgrey/dimgrey",
-		"Style * HilightFore black, HilightBack grey",
-		"AddToMenu MenuFvwmRoot \"Builtin Menu\" Title",
-		"+ \"&1. XTerm\" Exec xterm",
-		"+ \"&2. Setup Form\" Module FvwmForm FvwmForm-Setup",
-		"+ \"&3. Setup 95 Script\" Module FvwmScript"
-		" FvwmScript-Setup95",
-		"+ \"&4. Issue FVWM commands\" Module FvwmConsole",
-		"+ \"&R. Restart FVWM\" Restart",
-		"+ \"&X. Exit FVWM\" Quit",
-		"Mouse 0 R N Menu MenuFvwmRoot",
-		"Read "FVWM_DATADIR"/ConfigFvwmDefaults",
-		NULL
+	const char *defaults[][3] = {
+		{"XORValue 0","",""},
+		{"DefaultFont","",""},
+		{"DefaultColors black grey","",""},
+		{DEFAULT_MENU_STYLE,"",""},
+		{"TitleStyle Centered -- Raised"},
+		{"Style * Color lightgrey/dimgrey","",""},
+		{"Style * HilightFore black, HilightBack grey","",""},
+		{"AddToMenu MenuFvwmRoot \"",
+		 _("Builtin Menu"), "\" Title"},
+		{"+ \"&1. XTerm\" Exec xterm","",""},
+		{"+ \"&2. ",
+		_("Setup Form"),
+		"\" Module FvwmForm FvwmForm-Setup"},
+		{"+ \"&3. ",
+		 _("Setup 95 Script"), 
+		 "\" Module FvwmScript FvwmScript-Setup95"},
+		{"+ \"&4. ",
+		 _("Issue FVWM commands"),
+		"\" Module FvwmConsole"},
+		{"+ \"&R. ",
+		 _("Restart FVWM"), "\" Restart"},
+		{"+ \"&X. ", gettext("Exit FVWM"), "\" Quit"},
+		{"Mouse 0 R N Menu MenuFvwmRoot","",""},
+		{"Read "FVWM_DATADIR"/ConfigFvwmDefaults","",""},
+		{NULL,NULL,NULL}
 	};
 
-	for (i = 0; defaults[i]; i++)
+	for (i = 0; defaults[i][0]; i++)
 	{
 		const exec_context_t *exc;
 		exec_context_changes_t ecc;
@@ -1320,7 +1386,10 @@ static void SetRCDefaults(void)
 		ecc.type = EXCT_INIT;
 		ecc.w.wcontext = C_ROOT;
 		exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
-		execute_function(NULL, exc, defaults[i], 0);
+		execute_function(
+			NULL, exc,
+			CatString3(defaults[i][0],defaults[i][1],defaults[i][2]),
+			0);
 		exc_destroy_context(exc);
 	}
 
@@ -1592,13 +1661,12 @@ int main(int argc, char **argv)
 	int i;
 	int len;
 	char *display_string;
-	char *opt_color_limit = NULL;
 	char message[255];
 	Bool do_force_single_screen = False;
 	Bool replace_wm = False;
-	Bool option_error = False;
 	int visualClass = -1;
 	int visualId = -1;
+	PictureColorLimitOption colorLimitop = {-1, -1, -1, -1, -1};
 	const exec_context_t *exc;
 	exec_context_changes_t ecc;
 
@@ -1613,9 +1681,11 @@ int main(int argc, char **argv)
 		g_argv[i] = argv[i];
 	}
 	g_argv[g_argc] = NULL;
-	setVersionInfo();
-	FlocaleInit(LC_CTYPE, "", "", "FVWM");
 
+	FlocaleInit(LC_CTYPE, "", "", "FVWM");
+	FGettextInit("fvwm", LOCALEDIR, "FVWM");
+
+	setVersionInfo();
 	/* Put the default module directory into the environment so it can be
 	 * used later by the config file, etc.  */
 	putenv("FVWM_MODULEDIR=" FVWM_MODULEDIR);
@@ -1661,51 +1731,64 @@ int main(int argc, char **argv)
 
 	for (i = 1; i < argc; i++)
 	{
-		if (StrEquals(argv[i], "-debug_stack_ring") ||
-		    StrEquals(argv[i], "--debug_stack_ring"))
+		if (strcmp(argv[i], "-debug_stack_ring") == 0 ||
+		    strcmp(argv[i], "--debug-stack-ring") == 0)
 		{
 			debugging_stack_ring = True;
 		}
-		else if (StrEquals(argv[i], "-debug") ||
-			 StrEquals(argv[i], "--debug"))
+		else if (strcmp(argv[i], "-D") == 0 ||
+			 strcmp(argv[i], "-debug") == 0 ||
+			 strcmp(argv[i], "--debug") == 0)
 		{
 			debugging = True;
 		}
-		else if (StrEquals(argv[i], "-clientId") ||
-			 StrEquals(argv[i], "--clientId"))
+		else if (strcmp(argv[i], "-i") == 0 ||
+			 strcmp(argv[i], "-clientid") == 0 ||
+			 strcmp(argv[i], "--clientid") == 0 ||
+			 strcmp(argv[i], "-clientId") == 0 ||
+			 strcmp(argv[i], "--clientId") == 0)
 		{
 			if (++i >= argc)
 			{
-				usage();
+				usage(0);
+				exit(1);
 			}
 			SetClientID(argv[i]);
 		}
-		else if (StrEquals(argv[i], "-restore") ||
-			 StrEquals(argv[i], "--restore"))
+		else if (strcmp(argv[i], "-F") == 0 ||
+			 strcmp(argv[i], "-restore") == 0 ||
+			 strcmp(argv[i], "--restore") == 0)
 		{
 			if (++i >= argc)
 			{
-				usage();
+				usage(0);
+				exit(1);
 			}
 			state_filename = argv[i];
 		}
-		else if (StrEquals(argv[i], "-s"))
+		else if (strcmp(argv[i], "-s") == 0 ||
+			 strcmp(argv[i], "-single-screen") == 0 ||
+			 strcmp(argv[i], "--single-screen") == 0)
 		{
 			do_force_single_screen = True;
 		}
-		else if (StrEquals(argv[i], "-d"))
+		else if (strcmp(argv[i], "-d") == 0 ||
+			 strcmp(argv[i], "-display") == 0 ||
+			 strcmp(argv[i], "--display") == 0)
 		{
 			if (++i >= argc)
 			{
-				usage();
+				usage(0);
+				exit(1);
 			}
 			display_name = argv[i];
 		}
-		else if (StrEquals(argv[i], "-f"))
+		else if (strcmp(argv[i], "-f") == 0)
 		{
 			if (++i >= argc)
 			{
-				usage();
+				usage(0);
+				exit(1);
 			}
 			if (num_config_commands < MAX_CFG_CMDS)
 			{
@@ -1725,11 +1808,15 @@ int main(int argc, char **argv)
 					MAX_CFG_CMDS);
 			}
 		}
-		else if (StrEquals(argv[i], "-cmd") ||
-			 StrEquals(argv[i], "--cmd"))
+		else if (strcmp(argv[i], "-c") == 0 ||
+			 strcmp(argv[i], "-cmd") == 0 ||
+			 strcmp(argv[i], "--cmd") == 0)
 		{
 			if (++i >= argc)
-				usage();
+			{
+				usage(0);
+				exit(1);
+			}
 			if (num_config_commands < MAX_CFG_CMDS)
 			{
 				config_commands[num_config_commands] =
@@ -1744,12 +1831,14 @@ int main(int argc, char **argv)
 					MAX_CFG_CMDS);
 			}
 		}
-		else if (StrEquals(argv[i], "-h"))
+		else if (strcmp(argv[i], "-h") == 0 ||
+			 strcmp(argv[i], "-?") == 0 ||
+			 strcmp(argv[i], "--help") == 0)
 		{
-			usage();
+			usage(1);
 			exit(0);
 		}
-		else if (StrEquals(argv[i], "-blackout"))
+		else if (strcmp(argv[i], "-blackout") == 0)
 		{
 			/* obsolete option */
 			fvwm_msg(
@@ -1757,35 +1846,43 @@ int main(int argc, char **argv)
 				"The -blackout option is obsolete, it may be "
 				"removed in the future.");
 		}
-		else if (StrEquals(argv[i], "-replace") ||
-			 StrEquals(argv[i], "--replace"))
+		else if (strcmp(argv[i], "-r") == 0 ||
+			 strcmp(argv[i], "-replace") == 0 ||
+			 strcmp(argv[i], "--replace") == 0)
 		{
 			replace_wm = True;
 		}
 		/* check for visualId before visual to remove ambiguity */
-		else if (StrEquals(argv[i], "-visualId") ||
-			 StrEquals(argv[i], "--visualId"))
+		else if (strcmp(argv[i], "-I") == 0 ||
+			 strcmp(argv[i], "-visualid") == 0 ||
+			 strcmp(argv[i], "--visualid") == 0 ||
+			 strcmp(argv[i], "-visualId") == 0 ||
+			 strcmp(argv[i], "--visualId") == 0)
 		{
 			visualClass = -1;
 			if (++i >= argc)
 			{
-				usage();
+				usage(0);
+				exit(1);
 			}
 			if (sscanf(argv[i], "0x%x", &visualId) == 0)
 			{
 				if (sscanf(argv[i], "%d", &visualId) == 0)
 				{
-					usage();
+					usage(0);
+					exit(1);
 				}
 			}
 		}
-		else if (StrEquals(argv[i], "-visual") ||
-			 StrEquals(argv[i], "--visual"))
+		else if (strcmp(argv[i], "-C") == 0 ||
+			 strcmp(argv[i], "-visual") == 0 ||
+			 strcmp(argv[i], "--visual") == 0)
 		{
 			visualId = None;
 			if (++i >= argc)
 			{
-				usage();
+				usage(0);
+				exit(1);
 			}
 			if (strncasecmp(argv[i], "staticg", 7) == 0)
 			{
@@ -1813,20 +1910,48 @@ int main(int argc, char **argv)
 			}
 			else
 			{
-				usage();
+				usage(0);
+				exit(1);
 			}
 		}
-		else if (StrEquals(argv[i], "-color-limit") ||
-			 StrEquals(argv[i], "--color-limit"))
+		else if (strcmp(argv[i], "-l") == 0 ||
+			 strcmp(argv[i], "-color-limit") == 0 ||
+			 strcmp(argv[i], "--color-limit") == 0)
 		{
 			if (++i >= argc)
 			{
-				usage();
+				usage(0);
+				exit(1);
 			}
-			CopyString(&opt_color_limit,argv[i]);
+			colorLimitop.color_limit = atoi(argv[i]);
 		}
-		else if (StrEquals(argv[i], "-version") ||
-			 StrEquals(argv[i], "--version"))
+		else if (strcmp(argv[i], "-L") == 0 ||
+			 strcmp(argv[i], "-strict-color-limit") == 0 ||
+			 strcmp(argv[i], "--strict-color-limit") == 0)
+		{
+			colorLimitop.strict = True;
+		}
+		else if (strcmp(argv[i], "-A") == 0 ||
+			 strcmp(argv[i], "-allocate-palette") == 0 ||
+			 strcmp(argv[i], "--allocate-palette") == 0)
+		{
+			colorLimitop.allocate = True;
+		}
+		else if (strcmp(argv[i], "-S") == 0 ||
+			 strcmp(argv[i], "-static-palette") == 0 ||
+			 strcmp(argv[i], "--static-palette") == 0)
+		{
+			colorLimitop.not_dynamic = True;
+		}
+		else if (strcmp(argv[i], "-P") == 0 ||
+			 strcmp(argv[i], "-visual-palette") == 0 ||
+			 strcmp(argv[i], "--visual-palette") == 0)
+		{
+			colorLimitop.use_named_table = True;
+		}
+		else if (strcmp(argv[i], "-V") == 0 ||
+			 strcmp(argv[i], "-version") == 0 ||
+			 strcmp(argv[i], "--version") == 0)
 		{
 			printf("%s\n%s\n\n%s\n", Fvwm_VersionInfo,
 			       Fvwm_SupportInfo, Fvwm_LicenseInfo);
@@ -1834,18 +1959,14 @@ int main(int argc, char **argv)
 		}
 		else
 		{
-			fvwm_msg(ERR, "main", "Unknown option: `%s'", argv[i]);
-			option_error = TRUE;
+			usage(0);
+			fprintf(stderr, "invalid option -- %s\n", argv[i]);
+			exit(1);
 		}
 	}
 
 	DBUG("main", "Done parsing args");
 
-	if (option_error)
-	{
-		usage();
-	}
-
 	DBUG("main", "Installing signal handlers");
 	InstallSignals();
 
@@ -2106,11 +2227,7 @@ int main(int argc, char **argv)
 	Scr.ColorLimit = 0;
 	PUseDynamicColors = 0;
 	Scr.ColorLimit = PictureInitColors(
-		PICTURE_CALLED_BY_FVWM, True, opt_color_limit, True, True);
-	if (opt_color_limit)
-	{
-		free(opt_color_limit);
-	}
+		PICTURE_CALLED_BY_FVWM, True, &colorLimitop, True, True);
 
 	FShapeInit(dpy);
 	FRenderInit(dpy);
@@ -2250,8 +2367,8 @@ int main(int argc, char **argv)
 			    CatString3(FVWM_CONFDIR, "/system", FVWMRC), exc))
 		{
 			fvwm_msg(
-				ERR, "main", "Cannot read startup file, tried: "
-				"\n\t%s/%s\n\t%s/%s\n\t%s/%s\n\t%s/system"
+				ERR, "main", "Cannot read startup file, tried:"
+				" \n\t%s/%s\n\t%s/%s\n\t%s/%s\n\t%s/system"
 				"%s\n\t%s/system%s", fvwm_userdir, FVWMRC,
 				home_dir, FVWMRC, FVWM_DATADIR, FVWMRC,
 				FVWM_DATADIR, FVWMRC, FVWM_CONFDIR, FVWMRC);
