@@ -87,7 +87,8 @@ static int count_nonsticky_in_hashtab (void *arg)
   WinData *win = (WinData *)arg;
   WinManager *man = the_manager;
 
-  if (!(IS_STICKY(win)) && win->complete && win->manager == man)
+  if (!(IS_STICKY_ON_DESK(win) || IS_STICKY_ON_PAGE(win)) &&
+      win->complete && win->manager == man)
     return 1;
   return 0;
 }
@@ -145,9 +146,21 @@ int win_in_viewport (WinData *win)
   WinManager *manager = win->manager;
   int flag = 0;
   int reverse = 0;
+  rectangle g;
 
   assert (manager);
 
+  if (IS_ICONIFIED(win) && !IS_ICON_SUPPRESSED(win))
+  {
+	  g = win->icon_g;
+  }
+  else
+  {
+	  g.x = win->x;
+	  g.y = win->y;
+	  g.width = win->width;
+	  g.height = win->height;
+  }
   switch (manager->res) {
   case SHOW_GLOBAL:
     flag = 1;
@@ -157,7 +170,7 @@ int win_in_viewport (WinData *win)
     reverse = 1;
     /* fall through to next case */
   case SHOW_DESKTOP:
-    if (IS_STICKY(win) || win->desknum == globals.desknum)
+    if (IS_STICKY_ON_PAGE(win) || win->desknum == globals.desknum)
       flag = 1;
     break;
 
@@ -165,12 +178,12 @@ int win_in_viewport (WinData *win)
     reverse = 1;
     /* fall through to next case */
   case SHOW_PAGE:
-    if (IS_STICKY(win)) {
+    if (IS_STICKY_ON_PAGE(win)) {
       flag = 1;
     } else if (win->desknum == globals.desknum) {
       /* win and screen intersect if they are not disjoint in x and y */
       flag = RECTANGLES_INTERSECT(
-	win->x, win->y, win->width, win->height,
+	g.x, g.y, g.width, g.height,
 	manager->managed_g.x, manager->managed_g.y,
 	manager->managed_g.width, manager->managed_g.height);
     }
@@ -183,7 +196,7 @@ int win_in_viewport (WinData *win)
     if (win->desknum == globals.desknum) {
       /* win and screen intersect if they are not disjoint in x and y */
       flag = RECTANGLES_INTERSECT(
-	win->x, win->y, win->width, win->height,
+	g.x, g.y, g.width, g.height,
 	manager->managed_g.x, manager->managed_g.y,
 	manager->managed_g.width, manager->managed_g.height);
     }
@@ -198,6 +211,7 @@ int win_in_viewport (WinData *win)
 static WinData *id_to_win (Ulong id)
 {
   WinData *win;
+
   win = find_win_hashtab (id);
   if (win == NULL) {
     win = new_windata ();
@@ -205,6 +219,7 @@ static WinData *id_to_win (Ulong id)
     win->app_id_set = 1;
     insert_win_hashtab (win);
   }
+
   return win;
 }
 
@@ -215,6 +230,8 @@ static void set_win_configuration (WinData *win, FvwmPacketBody *body)
   win->y = body->add_config_data.frame_y;
   win->width = body->add_config_data.frame_width;
   win->height = body->add_config_data.frame_height;
+  win->iconified =  IS_ICONIFIED(&(body->add_config_data));
+  win->state = PLAIN_CONTEXT;
   win->geometry_set = 1;
   memcpy(&(win->flags), &(body->add_config_data.flags), sizeof(win->flags));
 }
@@ -269,7 +286,7 @@ static void focus_change (FvwmPacketBody *body)
   ConsoleDebug (FVWM, "Focus Change\n");
   ConsoleDebug (FVWM, "\tID: %ld\n", app_id);
 
-  if (fvwm_focus_win)
+  if (fvwm_focus_win && win != fvwm_focus_win)
   {
     del_win_state (fvwm_focus_win, FOCUS_CONTEXT);
     if (fvwm_focus_win->manager && fvwm_focus_win->manager->focus_button)
@@ -283,8 +300,8 @@ static void focus_change (FvwmPacketBody *body)
       win->manager->window_up  &&
       win->manager->followFocus) {
     win->manager->focus_button = win->button;
-    add_win_state (win, FOCUS_CONTEXT);
   }
+  add_win_state (win, FOCUS_CONTEXT);
 
   globals.focus_win = win;
   fvwm_focus_win = win;
@@ -358,7 +375,8 @@ static void visible_icon_name (FvwmPacketBody *body)
 
   win = id_to_win (app_id);
 
-  if (win->visible_icon_name && !strcmp (win->visible_icon_name, (char *)name)) {
+  if (win->visible_icon_name && !strcmp (win->visible_icon_name, (char *)name))
+  {
     ConsoleDebug (FVWM, "No icon change: %s %s\n", win->iconname, name);
     return;
   }
@@ -434,22 +452,12 @@ static void new_window (FvwmPacketBody *body)
 {
   WinData *win;
 
-  win = new_windata();
+  win = id_to_win (body->add_config_data.w);
   memcpy(&(win->flags), &(body->add_config_data.flags), sizeof(win->flags));
-  if (!(IS_TRANSIENT(win)))
-  {
-    win->app_id = body->add_config_data.w;
-    win->app_id_set = 1;
-    set_win_configuration (win, body);
-
-    insert_win_hashtab (win);
-    check_win_complete (win);
-    check_in_window (win);
-  }
-  else
-  {
-    Free(win);
-  }
+  set_win_configuration (win, body);
+  got_configure (win->manager);
+  check_win_complete (win);
+  check_in_window (win);
 }
 
 static void destroy_window (FvwmPacketBody *body)
@@ -495,17 +503,62 @@ static void mini_icon (FvwmPacketBody *body)
 		win->pic.width, win->pic.height, win->pic.depth);
 }
 
-static void iconify (FvwmPacketBody *body, int dir)
+static void icon_location(FvwmPacketBody *body)
 {
-  Ulong app_id = body->minimal_data.app_id;
-  WinData *win;
+	Ulong app_id = body->minimal_data.app_id;
+	WinData *win;
 
-  win = id_to_win (app_id);
+	win = id_to_win(app_id);
+	if (win == NULL)
+	{
+		return;
+	}
+	win->icon_g.x = body->icon_data.xpos;
+	win->icon_g.y = body->icon_data.ypos;
+	win->icon_g.width = body->icon_data.icon_width;
+	win->icon_g.height = body->icon_data.icon_height;
+	check_in_window (win);
+}
 
-  set_win_iconified (win, dir);
+static void iconify(FvwmPacketBody *body, int dir)
+{
+	Ulong app_id = body->minimal_data.app_id;
+	WinData *win;
 
-  check_win_complete (win);
-  check_in_window (win);
+	win = id_to_win(app_id);
+	set_win_iconified(win, dir);
+	icon_location(body);
+	check_win_complete(win);
+	check_in_window (win);
+}
+
+static void window_shade(FvwmPacketBody *body, int do_shade)
+{
+	Window w;
+	WinManager *man;
+
+	w = (Window)(body->minimal_data.app_id);
+	man = find_windows_manager(w);
+	if (man == NULL)
+	{
+		return;
+	}
+	if (do_shade)
+	{
+		man->flags.needs_resize_after_unshade = 0;
+		man->flags.is_shaded = 1;
+	}
+	else
+	{
+		man->flags.is_shaded = 0;
+		if (man->flags.needs_resize_after_unshade)
+		{
+			draw_manager(man);
+			man->flags.needs_resize_after_unshade = 0;
+		}
+	}
+
+	return;
 }
 
 /* only used by remanage_winlist */
@@ -540,34 +593,42 @@ static void sendtomodule (FvwmPacketBody *body)
 
 static void property_change (FvwmPacketBody *body)
 {
-  WinManager *man = NULL;
-  int j;
+	WinManager *man = NULL;
+	int j;
 
-  if (body->property_data.id != 0)
-  {
-    if (!(man = find_windows_manager(body->property_data.id)))
-      return;
-  }
+	if (body->property_data.id != 0)
+	{
+		if (!(man = find_windows_manager(body->property_data.id)))
+			return;
+	}
 
-  if (body->property_data.arg == MX_PROPERTY_CHANGE_BACKGROUND)
-  {
-    if (man != NULL && man->pixmap[DEFAULT] == ParentRelative)
-    {
-      XClearArea(theDisplay, man->theWindow, 0, 0, 0, 0, True);
-    }
-    if (man != NULL)
-      return;
-    for (j = 0; j < globals.num_managers; j++)
-    {
-      man = &globals.managers[j];
-      if (man->pixmap[DEFAULT] == ParentRelative)
-	XClearArea(theDisplay, man->theWindow, 0, 0, 0, 0, True);
-    }
-  }
-  else if  (body->property_data.arg == MX_PROPERTY_CHANGE_SWALLOW && man != NULL)
-  {
-    man->swallowed = body->property_data.toggle;
-  }
+	if (body->property_data.arg == MX_PROPERTY_CHANGE_BACKGROUND)
+	{
+		if (man != NULL &&
+		    CSET_IS_TRANSPARENT_PR(man->colorsets[DEFAULT]))
+		{
+			recreate_background(man, DEFAULT);
+			force_manager_redraw(man);
+		}
+		if (man != NULL)
+		{
+			return;
+		}
+		for (j = 0; j < globals.num_managers; j++)
+		{
+			man = &globals.managers[j];
+			if (CSET_IS_TRANSPARENT_PR(man->colorsets[DEFAULT]))
+			{
+				recreate_background(man, DEFAULT);
+				force_manager_redraw(man);
+			}
+		}
+	}
+	else if  (body->property_data.arg == MX_PROPERTY_CHANGE_SWALLOW &&
+		  man != NULL)
+	{
+		man->swallowed = body->property_data.toggle;
+	}
 }
 
 static void ProcessMessage (Ulong type, FvwmPacketBody *body)
@@ -654,6 +715,10 @@ static void ProcessMessage (Ulong type, FvwmPacketBody *body)
     SendUnlockNotification(Fvwm_fd);
     break;
 
+  case M_ICON_LOCATION:
+    icon_location(body);
+    break;
+
   case M_END_WINDOWLIST:
     ConsoleDebug (FVWM, "DEBUG::M_END_WINDOWLIST\n");
     ConsoleDebug (FVWM,
@@ -704,6 +769,14 @@ static void ProcessMessage (Ulong type, FvwmPacketBody *body)
     property_change(body);
     break;
 
+  case M_WINDOWSHADE:
+    window_shade(body, 1);
+    break;
+
+  case M_DEWINDOWSHADE:
+    window_shade(body, 0);
+    break;
+
   default:
     break;
   }
