@@ -355,7 +355,7 @@ int main(int argc, char **argv)
   ParseOptions();
   if (is_transient)
   {
-    if (XQueryPointer(
+    if (FQueryPointer(
 	  dpy, Scr.Root, &JunkRoot, &JunkChild, &window_x, &window_y, &JunkX,
 	  &JunkY, &JunkMask) == False)
     {
@@ -990,8 +990,8 @@ void list_new_desk(unsigned long *body)
   }
 
   MovePage(True);
-  DrawGrid(oldDesk - desk1,1);
-  DrawGrid(Scr.CurrentDesk - desk1,1);
+  DrawGrid(oldDesk - desk1, 1, None, NULL);
+  DrawGrid(Scr.CurrentDesk - desk1, 1, None, NULL);
   MoveStickyWindows();
 /*
   Hilight(FocusWin,False);
@@ -1395,7 +1395,7 @@ void list_config_info(unsigned long *body)
     {
       val = val - desk1;
     }
-    DrawGrid(val, True);
+    DrawGrid(val, True, None, NULL);
   }
 }
 
@@ -1404,14 +1404,9 @@ void list_property_change(unsigned long *body)
   if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND)
   {
 
-    if (default_pixmap == ParentRelative &&
-	((!Swallowed && body[2] == 0) || (Swallowed && body[2] == Scr.Pager_w)))
+    if (((!Swallowed && body[2] == 0) || (Swallowed && body[2] == Scr.Pager_w)))
     {
-      ReConfigure();
-    }
-    if (BalloonView != None)
-    {
-      XClearArea(dpy, Scr.balloon_w, 0, 0, 0, 0, True);
+	    update_pr_transparent_windows();
     }
   }
   else if  (body[0] == MX_PROPERTY_CHANGE_SWALLOW && body[2] == Scr.Pager_w)
@@ -1430,9 +1425,9 @@ int My_XNextEvent(Display *dpy, XEvent *event)
   fd_set in_fdset;
   static int miss_counter = 0;
 
-  if(XPending(dpy))
+  if(FPending(dpy))
   {
-    XNextEvent(dpy,event);
+    FNextEvent(dpy,event);
     return 1;
   }
 
@@ -1444,9 +1439,9 @@ int My_XNextEvent(Display *dpy, XEvent *event)
   {
   if(FD_ISSET(x_fd, &in_fdset))
     {
-      if(XPending(dpy))
+      if(FPending(dpy))
 	{
-	  XNextEvent(dpy,event);
+	  FNextEvent(dpy,event);
 	  miss_counter = 0;
 	  return 1;
 	}
@@ -1591,7 +1586,7 @@ void ParseOptions(void)
 
   Scr.MyDisplayWidth = DisplayWidth(dpy, Scr.screen);
   Scr.MyDisplayHeight = DisplayHeight(dpy, Scr.screen);
-  
+
   fpa.mask = 0;
   if (Pdepth <= 8)
   {
@@ -1603,7 +1598,6 @@ void ParseOptions(void)
     int g_x, g_y, flags;
     unsigned width,height;
     char *resource;
-    char *resource_string;
     char *arg1;
     char *arg2;
     char *tline2;
@@ -1611,7 +1605,7 @@ void ParseOptions(void)
     char *next;
     Bool MoveThresholdSetForModule = False;
 
-    resource_string = arg1 = arg2 = NULL;
+    arg1 = arg2 = NULL;
 
     token = PeekToken(tline, &next);
     if (StrEquals(token, "Colorset"))
