@@ -131,7 +131,7 @@ static void discard_events(long event_type, Window w, XEvent *last_ev)
   XEvent e;
 
   XSync(dpy, 0);
-  while (XCheckTypedWindowEvent(dpy, w, event_type, &e))
+  while (FCheckTypedWindowEvent(dpy, w, event_type, &e))
   {
     /* do nothing */
     if (last_ev)
@@ -242,30 +242,44 @@ static void CalcGeom(PagerWindow *t, int win_w, int win_h,
  ***********************************************************************/
 void initialize_viz_pager(void)
 {
-  XSetWindowAttributes attr;
-  XGCValues xgcv;
+	XSetWindowAttributes attr;
+	XGCValues xgcv;
 
-  if (Pdefault) {
-    Scr.Pager_w = Scr.Root;
-    Scr.NormalGC = DefaultGC(dpy, Scr.screen);
-  } else {
-    attr.background_pixmap = None;
-    attr.border_pixel = 0;
-    attr.colormap = Pcmap;
-    Scr.Pager_w = XCreateWindow(dpy, Scr.Root, -10, -10, 10, 10, 0, Pdepth,
-				InputOutput, Pvisual,
-				CWBackPixmap|CWBorderPixel|CWColormap, &attr);
-    Scr.NormalGC = fvwmlib_XCreateGC(dpy, Scr.Pager_w, 0, &xgcv);
-  }
-  xgcv.plane_mask = AllPlanes;
-  Scr.MiniIconGC = fvwmlib_XCreateGC(dpy, Scr.Pager_w, GCPlaneMask, &xgcv);
-  Scr.black = GetColor("Black");
+	/* FIXME: I think that we only need that Pdepth ==
+	 * DefaultDepth(dpy, Scr.screen) to use the Scr.Root for Scr.Pager_w */
+	if (Pdefault)
+	{
+		Scr.Pager_w = Scr.Root;
+	}
+	else
+	{
+		attr.background_pixmap = None;
+		attr.border_pixel = 0;
+		attr.colormap = Pcmap;
+		Scr.Pager_w = XCreateWindow(
+			dpy, Scr.Root, -10, -10, 10, 10, 0, Pdepth,
+			InputOutput, Pvisual,
+			CWBackPixmap|CWBorderPixel|CWColormap, &attr);
+		Scr.NormalGC = fvwmlib_XCreateGC(dpy, Scr.Pager_w, 0, &xgcv);
+	}
+	Scr.NormalGC = fvwmlib_XCreateGC(dpy, Scr.Pager_w, 0, NULL);
 
-  /* Transparent background are only allowed when the depth matched the root */
-  if (Pdepth == DefaultDepth(dpy, Scr.screen))
-    default_pixmap = ParentRelative;
+	xgcv.plane_mask = AllPlanes;
+	Scr.MiniIconGC = fvwmlib_XCreateGC(
+		dpy, Scr.Pager_w, GCPlaneMask, &xgcv);
+	Scr.black = GetColor("Black");
+
+	/* Transparent background are only allowed when the depth matched the
+	 * root */
+	if (Pdepth == DefaultDepth(dpy, Scr.screen))
+	{
+		default_pixmap = ParentRelative;
+	}
+
+	return;
 }
 
+/* see also change colorset */
 void draw_desk_background(int i, int page_w, int page_h)
 {
 	if (Desks[i].colorset > -1)
@@ -280,12 +294,26 @@ void draw_desk_background(int i, int page_w, int page_h)
 			dpy, Desks[i].DashedGC,Colorset[Desks[i].colorset].fg);
 		if (uselabel)
 		{
-			SetWindowBackground(
-				dpy, Desks[i].title_w, desk_w, desk_h + label_h,
-				&Colorset[Desks[i].colorset], Pdepth,
-				Scr.NormalGC, True);
+			if (CSET_IS_TRANSPARENT(Desks[i].colorset))
+			{
+					SetWindowBackground(
+						dpy, Desks[i].title_w,
+						desk_w, label_h,
+						&Colorset[Desks[i].colorset],
+						Pdepth,
+						Scr.NormalGC, True);
+			}
+			else
+			{
+				SetWindowBackground(
+					dpy, Desks[i].title_w, desk_w,
+					desk_h + label_h,
+					&Colorset[Desks[i].colorset], Pdepth,
+					Scr.NormalGC, True);
+			}
 		}
-		if (label_h != 0 && uselabel && !LabelsBelow)
+		if (label_h != 0 && uselabel && !LabelsBelow &&
+		    !CSET_IS_TRANSPARENT(Desks[i].colorset))
 		{
 			SetWindowBackgroundWithOffset(
 				dpy, Desks[i].w, 0, -label_h, desk_w,
@@ -294,12 +322,24 @@ void draw_desk_background(int i, int page_w, int page_h)
 		}
 		else
 		{
-			SetWindowBackground(
-				dpy, Desks[i].w, desk_w, desk_h + label_h,
-				&Colorset[Desks[i].colorset], Pdepth,
-				Scr.NormalGC, True);
+			if (CSET_IS_TRANSPARENT(Desks[i].colorset))
+			{
+				SetWindowBackground(
+					dpy, Desks[i].w, desk_w, desk_h,
+					&Colorset[Desks[i].colorset], Pdepth,
+					Scr.NormalGC, True);
+			}
+			else
+			{
+				SetWindowBackground(
+					dpy, Desks[i].w, desk_w,
+					desk_h + label_h,
+					&Colorset[Desks[i].colorset],
+					Pdepth, Scr.NormalGC, True);
+			}
 		}
 	}
+	XClearArea(dpy,Desks[i].w, 0, 0, 0, 0,True);
 	if (Desks[i].highcolorset > -1)
 	{
 		XSetForeground(
@@ -315,7 +355,6 @@ void draw_desk_background(int i, int page_w, int page_h)
 				Scr.NormalGC, True);
 		}
 	}
-	XClearArea(dpy,Desks[i].w, 0, 0, 0, 0,True);
 	if (uselabel)
 	{
 		XClearArea(dpy,Desks[i].title_w, 0, 0, 0, 0,True);
@@ -808,7 +847,7 @@ void initialize_pager(void)
     }
 
     Desks[i].w = XCreateWindow(
-      dpy, Desks[i].title_w, x - 1, LabelsBelow ? -1 : label_h - 1, w, desk_h,
+	    dpy, Desks[i].title_w, x - 1, LabelsBelow ? -1 : label_h - 1, w, desk_h,
       1, CopyFromParent, InputOutput, CopyFromParent, valuemask, &attributes);
 
     if (HilightDesks)
@@ -989,8 +1028,8 @@ void DispatchEvent(XEvent *Event)
       if (do_move_page)
       {
 	char command[64];
-	sprintf(command,"Scroll %d %d\n", dx, dy);
-	SendInfo(fd, command, 0);
+	sprintf(command,"Scroll %d %d", dx, dy);
+	SendText(fd, command, 0);
       }
     }
     break;
@@ -1006,7 +1045,7 @@ void DispatchEvent(XEvent *Event)
       {
 	if(Event->xany.window == Desks[i].w)
 	{
-	  if (XQueryPointer(dpy, Desks[i].w, &JunkRoot, &JunkChild,
+	  if (FQueryPointer(dpy, Desks[i].w, &JunkRoot, &JunkChild,
 			    &JunkX, &JunkY,&x, &y, &JunkMask) == False)
 	  {
 	    /* pointer is on a different screen - that's okay here */
@@ -1016,7 +1055,7 @@ void DispatchEvent(XEvent *Event)
       }
       if(Event->xany.window == icon_win)
       {
-	if (XQueryPointer(dpy, icon_win, &JunkRoot, &JunkChild,
+	if (FQueryPointer(dpy, icon_win, &JunkRoot, &JunkChild,
 			  &JunkX, &JunkY,&x, &y, &JunkMask) == False)
 	{
 	  /* pointer is on a different screen - that's okay here */
@@ -1062,7 +1101,7 @@ void DispatchEvent(XEvent *Event)
       {
 	if(Event->xany.window == Desks[i].w)
 	{
-	  if (XQueryPointer(dpy, Desks[i].w, &JunkRoot, &JunkChild,
+	  if (FQueryPointer(dpy, Desks[i].w, &JunkRoot, &JunkChild,
 			    &JunkX, &JunkY,&x, &y, &JunkMask) == False)
 	  {
 	    /* pointer is on a different screen - that's okay here */
@@ -1078,7 +1117,7 @@ void DispatchEvent(XEvent *Event)
       }
       if(Event->xany.window == icon_win)
       {
-	if (XQueryPointer(dpy, icon_win, &JunkRoot, &JunkChild,
+	if (FQueryPointer(dpy, icon_win, &JunkRoot, &JunkChild,
 			  &JunkX, &JunkY,&x, &y, &JunkMask) == False)
 	{
 	  /* pointer is on a different screen - that's okay here */
@@ -1089,7 +1128,7 @@ void DispatchEvent(XEvent *Event)
     break;
   case MotionNotify:
     do_ignore_next_button_release = False;
-    while(XCheckMaskEvent(dpy, PointerMotionMask | ButtonMotionMask,Event))
+    while(FCheckMaskEvent(dpy, PointerMotionMask | ButtonMotionMask,Event))
       ;
 
     if(Event->xmotion.state & Button3MotionMask)
@@ -1098,7 +1137,7 @@ void DispatchEvent(XEvent *Event)
       {
 	if(Event->xany.window == Desks[i].w)
 	{
-	  if (XQueryPointer(dpy, Desks[i].w, &JunkRoot, &JunkChild,
+	  if (FQueryPointer(dpy, Desks[i].w, &JunkRoot, &JunkChild,
 			    &JunkX, &JunkY,&x, &y, &JunkMask) == False)
 	  {
 	    /* pointer is on a different screen - that's okay here */
@@ -1108,7 +1147,7 @@ void DispatchEvent(XEvent *Event)
       }
       if(Event->xany.window == icon_win)
       {
-	if (XQueryPointer(dpy, icon_win, &JunkRoot, &JunkChild,
+	if (FQueryPointer(dpy, icon_win, &JunkRoot, &JunkChild,
 			  &JunkX, &JunkY,&x, &y, &JunkMask) == False)
 	{
 	  /* pointer is on a different screen - that's okay here */
@@ -1166,51 +1205,78 @@ void HandleEnterNotify(XEvent *Event)
 
   if (do_focus_on_enter)
   {
-    SendInfo(fd, "Silent FlipFocus NoWarp", t->w);
+    SendText(fd, "Silent FlipFocus NoWarp", t->w);
   }
 
 }
 
+XRectangle get_expose_bound(XEvent *Event)
+{
+	XRectangle r;
+	int ex, ey, ex2, ey2;
+
+	ex = Event->xexpose.x;
+	ey = Event->xexpose.y;
+	ex2 = Event->xexpose.x + Event->xexpose.width;
+	ey2 = Event->xexpose.y + Event->xexpose.height;
+	while (FCheckTypedWindowEvent(dpy, Event->xany.window, Expose, Event))
+	{
+		ex = min(ex, Event->xexpose.x);
+		ey = min(ey, Event->xexpose.y);
+		ex2 = max(ex2, Event->xexpose.x + Event->xexpose.width);
+		ey2= max(ey2 , Event->xexpose.y + Event->xexpose.height);
+	}
+	r.x = ex;
+	r.y = ey;
+	r.width = ex2-ex;
+	r.height = ey2-ey;
+	return r;
+}
+
 void HandleExpose(XEvent *Event)
 {
-  int i;
-  PagerWindow *t;
+	int i;
+	PagerWindow *t;
+	XRectangle r;
 
-  for(i=0;i<ndesks;i++)
-  {
-    /* ric@giccs.georgetown.edu */
-    if (Event->xany.window == Desks[i].w ||
-	Event->xany.window == Desks[i].title_w)
-    {
-      DrawGrid(i, 0);
-      return;
-    }
-  }
-  if (Event->xany.window == Scr.balloon_w)
-  {
-    DrawInBalloonWindow(Scr.balloon_desk);
-    return;
-  }
-  if(Event->xany.window == icon_win)
-    DrawIconGrid(0);
+	/* it will be good to have full "clipping redraw". Do that for
+	 * desk label only for now */
+	for(i=0;i<ndesks;i++)
+	{
+		/* ric@giccs.georgetown.edu */
+		if (Event->xany.window == Desks[i].w ||
+		    Event->xany.window == Desks[i].title_w)
+		{
+			r = get_expose_bound(Event);
+			DrawGrid(i, 0, Event->xany.window, &r);
+			return;
+		}
+	}
+	if (Event->xany.window == Scr.balloon_w)
+	{
+		DrawInBalloonWindow(Scr.balloon_desk);
+		return;
+	}
+	if(Event->xany.window == icon_win)
+		DrawIconGrid(0);
 
-  for (t = Start; t != NULL; t = t->next)
-  {
-    if (t->PagerView == Event->xany.window)
-    {
-      LabelWindow(t);
-      PictureWindow(t);
-      BorderWindow(t);
-    }
-    else if(t->IconView == Event->xany.window)
-    {
-      LabelIconWindow(t);
-      PictureIconWindow(t);
-      BorderIconWindow(t);
-    }
-  }
+	for (t = Start; t != NULL; t = t->next)
+	{
+		if (t->PagerView == Event->xany.window)
+		{
+			LabelWindow(t);
+			PictureWindow(t);
+			BorderWindow(t);
+		}
+		else if(t->IconView == Event->xany.window)
+		{
+			LabelIconWindow(t);
+			PictureIconWindow(t);
+			BorderIconWindow(t);
+		}
+	}
 
-  discard_events(Expose, Event->xany.window, NULL);
+	discard_events(Expose, Event->xany.window, NULL);
 }
 
 /****************************************************************************
@@ -1282,7 +1348,11 @@ void ReConfigure(void)
 	  dpy,Desks[i].w, -1, (LabelsBelow) ? -1 : label_h - 1, desk_w,desk_h);
 	if (!is_size_changed)
 	{
-	  continue;
+		if (CSET_IS_TRANSPARENT(Desks[i].colorset))
+		{
+			draw_desk_background(i, w, h);
+		}
+		continue;
 	}
 	if (HilightDesks)
 	{
@@ -1299,9 +1369,142 @@ void ReConfigure(void)
   ReConfigureAll();
 }
 
+/****************************************************************************
+ *
+ * Respond to a "background" change: update the Parental Relative cset
+ *
+ ****************************************************************************/
+
+/* layout:
+ * Root -> pager window (pr) -> title window -> desk window -> window view
+ *  |                                                |-> hilight desk
+ *  |-> icon_window -> icon view
+ *  |-> ballon window
+ */
+
+/* update the hilight desk and the windows: desk color change */
+static
+void update_pr_transparent_subwindows(int i)
+{
+	int cset;
+	int n,m,w,h;
+	PagerWindow *t;
+
+	n = Scr.VxMax / Scr.MyDisplayWidth;
+	m = Scr.VyMax / Scr.MyDisplayHeight;
+	w = (desk_w - n)/(n+1);
+	h = (desk_h - m)/(m+1);
+
+	if (CSET_IS_TRANSPARENT_PR(Desks[i].highcolorset) && HilightDesks)
+	{
+		SetWindowBackground(
+			dpy, Desks[i].CPagerWin, w, h,
+			&Colorset[Desks[i].highcolorset],
+			Pdepth, Scr.NormalGC, True);
+	}
+
+	t = Start;
+	for(t = Start; t != NULL; t = t->next)
+	{
+		cset = (t != FocusWin) ? windowcolorset : activecolorset;
+		if (t->desk != i && !CSET_IS_TRANSPARENT_PR(cset))
+		{
+			continue;
+		}
+		if (t->PagerView != None)
+		{
+			SetWindowBackground(
+				dpy, t->PagerView, t->pager_view_width,
+				t->pager_view_height,
+				&Colorset[cset], Pdepth, Scr.NormalGC, True);
+		}
+		if (t->IconView)
+		{
+			SetWindowBackground(
+				dpy, t->IconView, t->icon_view_width,
+				t->icon_view_height,
+				&Colorset[cset], Pdepth, Scr.NormalGC, True);
+		}
+	}
+}
+
+/* update all the parental relative windows: pr background change */
+void update_pr_transparent_windows(void)
+{
+	int i,j,k,cset;
+	int n,m,w,h;
+	PagerWindow *t;
+
+	n = Scr.VxMax / Scr.MyDisplayWidth;
+	m = Scr.VyMax / Scr.MyDisplayHeight;
+	w = (desk_w - n)/(n+1);
+	h = (desk_h - m)/(m+1);
+
+	for(k=0;k<Rows;k++)
+	{
+		for(j=0;j<Columns;j++)
+		{
+			i = k*Columns+j;
+			if (i < ndesks)
+			{
+				if (CSET_IS_TRANSPARENT_PR(Desks[i].colorset))
+				{
+					draw_desk_background(i, w, h);
+				}
+				else if (CSET_IS_TRANSPARENT_PR(
+					Desks[i].highcolorset) && HilightDesks)
+				{
+					SetWindowBackground(
+						dpy, Desks[i].CPagerWin, w, h,
+						&Colorset[Desks[i].highcolorset],
+						Pdepth, Scr.NormalGC, True);
+				}
+			}
+		}
+	}
+	/* subordinate windows with a pr parent desk */
+	t = Start;
+	for(t = Start; t != NULL; t = t->next)
+	{
+		cset = (t != FocusWin) ? windowcolorset : activecolorset;
+		if (!CSET_IS_TRANSPARENT_PR(cset) ||
+		    (fAlwaysCurrentDesk &&
+		     !CSET_IS_TRANSPARENT_PR(Desks[0].colorset)) ||
+		    (!fAlwaysCurrentDesk &&
+		     !CSET_IS_TRANSPARENT_PR(Desks[t->desk].colorset)))
+		{
+			continue;
+		}
+		if (t->PagerView != None)
+		{
+			SetWindowBackground(
+				dpy, t->PagerView, t->pager_view_width,
+				t->pager_view_height,
+				&Colorset[cset], Pdepth, Scr.NormalGC, True);
+		}
+		if (t->desk && t->IconView)
+		{
+			SetWindowBackground(
+				dpy, t->IconView, t->icon_view_width,
+				t->icon_view_height,
+				&Colorset[cset], Pdepth, Scr.NormalGC, True);
+		}
+	}
+
+	/* ballon */
+	if (BalloonView != None)
+	{
+		cset = Desks[Scr.balloon_desk].ballooncolorset;
+		if (CSET_IS_TRANSPARENT_PR(cset))
+		{
+			XClearArea(dpy, Scr.balloon_w, 0, 0, 0, 0, True);
+		}
+	}
+}
+
 void MovePage(Bool is_new_desk)
 {
-  int n1,m1,x,y,n,m,i;
+  int n1,m1,x,y,n,m,i,w,h;
   XTextProperty name;
   char str[100],*sptr;
   static int icon_desk_shown = -1000;
@@ -1314,6 +1517,9 @@ void MovePage(Bool is_new_desk)
 
   x = (desk_w - n) * Scr.Vx / Scr.VWidth + n1;
   y = (desk_h - m) * Scr.Vy / Scr.VHeight + m1;
+  w = (desk_w - n)/(n+1);
+  h = (desk_h - m)/(m+1);
+
   for(i=0;i<ndesks;i++)
   {
     if (HilightDesks)
@@ -1322,9 +1528,13 @@ void MovePage(Bool is_new_desk)
       {
 	XMoveWindow(dpy, Desks[i].CPagerWin, x,y);
 	XLowerWindow(dpy,Desks[i].CPagerWin);
-	if (Desks[i].highcolorset > -1 &&
-	    Colorset[Desks[i].highcolorset].pixmap == ParentRelative)
-	  XClearWindow(dpy, Desks[i].CPagerWin);
+	if (CSET_IS_TRANSPARENT(Desks[i].highcolorset))
+	{
+		SetWindowBackground(
+			dpy, Desks[i].CPagerWin, w, h,
+			&Colorset[Desks[i].highcolorset], Pdepth,
+			Scr.NormalGC, True);
+	}
       }
       else
       {
@@ -1344,7 +1554,7 @@ void MovePage(Bool is_new_desk)
       sptr = Desks[Scr.CurrentDesk -desk1].label;
     else
     {
-      sprintf(str,"GoToDesk %d",Scr.CurrentDesk);
+      sprintf(str, "GotoDesk %d", Scr.CurrentDesk);
       sptr = &str[0];
     }
 
@@ -1395,90 +1605,153 @@ void ReConfigureIcons(Bool do_reconfigure_desk_only)
  * Draw grid lines for desk #i
  *
  ****************************************************************************/
-void DrawGrid(int desk, int erase)
+void DrawGrid(int desk, int erase, Window ew, XRectangle *r)
 {
-  int y, y1, y2, x, x1, x2,d,w;
-  char str[15], *ptr;
-  int cs;
+	int y, y1, y2, x, x1, x2,d,w;
+	char str[15], *ptr;
+	int cs;
+	XRectangle bound;
+	Region region = 0;
 
-  if((desk < 0 ) || (desk >= ndesks))
-    return;
+	if((desk < 0 ) || (desk >= ndesks))
+		return;
 
-  x = Scr.MyDisplayWidth;
-  y1 = 0;
-  y2 = desk_h;
-  while (x < Scr.VWidth)
-  {
-    x1 = (x * desk_w) / Scr.VWidth;
-    if (!use_no_separators)
-      XDrawLine(dpy,Desks[desk].w,Desks[desk].DashedGC,x1,y1,x1,y2);
-    x += Scr.MyDisplayWidth;
-  }
+	/* desk grid */
+	if (!ew || ew == Desks[desk].w)
+	{
+		x = Scr.MyDisplayWidth;
+		y1 = 0;
+		y2 = desk_h;
+		while (x < Scr.VWidth)
+		{
+			x1 = (x * desk_w) / Scr.VWidth;
+			if (!use_no_separators)
+			{
+				XDrawLine(
+					dpy,Desks[desk].w,Desks[desk].DashedGC,
+					x1,y1,x1,y2);
+			}
+			x += Scr.MyDisplayWidth;
+		}
+		y = Scr.MyDisplayHeight;
+		x1 = 0;
+		x2 = desk_w;
+		while(y < Scr.VHeight)
+		{
+			y1 = (y * desk_h) / Scr.VHeight;
+			if (!use_no_separators)
+			{
+				XDrawLine(
+					dpy,Desks[desk].w,Desks[desk].DashedGC,
+					x1,y1,x2,y1);
+			}
+			y += Scr.MyDisplayHeight;
+		}
+	}
 
-  y = Scr.MyDisplayHeight;
-  x1 = 0;
-  x2 = desk_w;
-  while(y < Scr.VHeight)
-  {
-    y1 = (y * desk_h) / Scr.VHeight;
-    if (!use_no_separators)
-      XDrawLine(dpy,Desks[desk].w,Desks[desk].DashedGC,x1,y1,x2,y1);
-    y += Scr.MyDisplayHeight;
-  }
-  if (FftSupport && Ffont->fftf.fftfont != NULL)
-    erase = True;
-  if(((Scr.CurrentDesk - desk1) == desk) && !ShapeLabels)
-  {
-    if(uselabel)
-      XFillRectangle(dpy,Desks[desk].title_w,Desks[desk].HiliteGC,
-		     0,(LabelsBelow ? desk_h : 0),desk_w,label_h);
-  }
-  else
-  {
-    if(uselabel && erase)
-      XClearArea(dpy,Desks[desk].title_w,
-		 0,(LabelsBelow ? desk_h : 0),desk_w,label_h,False);
-  }
+	if (ew && ew != Desks[desk].title_w)
+	{
+		return;
+	}
 
-  d = desk1+desk;
-  ptr = Desks[desk].label;
-  w = FlocaleTextWidth(Ffont,ptr,strlen(ptr));
-  if( w > desk_w)
-  {
-    sprintf(str,"%d",d);
-    ptr = str;
-    w = FlocaleTextWidth(Ffont,ptr,strlen(ptr));
-  }
-  if((w <= desk_w)&&(uselabel))
-  {
-    FwinString->str = ptr;
-    FwinString->win = Desks[desk].title_w;
-    if(desk == (Scr.CurrentDesk - desk1))
-    {
-      cs = Desks[desk].highcolorset;
-      FwinString->gc = Desks[desk].rvGC;
-    }
-    else
-    {
-      cs = Desks[desk].highcolorset;
-      FwinString->gc = Desks[desk].NormalGC;
-    }
+	/* desk label */
+	if (r)
+	{
+		bound.x = r->x;
+		bound.y = r->y;
+		bound.width = r->width;
+		bound.height = r->height;
+		region = XCreateRegion();
+		XUnionRectWithRegion (&bound, region, region);
+	}
+	else
+	{
+		bound.x = 0;
+		bound.y = (LabelsBelow ? desk_h : 0);
+		bound.width = desk_w;
+		bound.height = label_h;
+	}
 
-    FwinString->flags.has_colorset = False;
-    if (cs >= 0)
-    {
-      FwinString->colorset = &Colorset[cs];
-      FwinString->flags.has_colorset = True;
-    }
-    FwinString->x = (desk_w -w)/2;
-    FwinString->y = (LabelsBelow ?
-		     desk_h + Ffont->ascent + 1 : Ffont->ascent + 1);
-    FlocaleDrawString(dpy, Ffont, FwinString, 0);
-  }
-  if (FShapesSupported)
-  {
-    UpdateWindowShape ();
-  }
+	if (FftSupport && Ffont->fftf.fftfont != NULL)
+	{
+		erase = True;
+	}
+	if(((Scr.CurrentDesk - desk1) == desk) && !ShapeLabels)
+	{
+		if (uselabel)
+		{
+			XFillRectangle(
+				dpy,Desks[desk].title_w,Desks[desk].HiliteGC,
+				bound.x, bound.y, bound.width, bound.height);
+		}
+	}
+	else
+	{
+		if(uselabel && erase)
+		{
+			XClearArea(dpy,Desks[desk].title_w,
+				   bound.x, bound.y, bound.width, bound.height,
+				   False);
+		}
+	}
+
+	d = desk1+desk;
+	ptr = Desks[desk].label;
+	w = FlocaleTextWidth(Ffont,ptr,strlen(ptr));
+	if( w > desk_w)
+	{
+		sprintf(str,"%d",d);
+		ptr = str;
+		w = FlocaleTextWidth(Ffont,ptr,strlen(ptr));
+	}
+	if((w <= desk_w)&&(uselabel))
+	{
+		FwinString->str = ptr;
+		FwinString->win = Desks[desk].title_w;
+		if(desk == (Scr.CurrentDesk - desk1))
+		{
+			cs = Desks[desk].highcolorset;
+			FwinString->gc = Desks[desk].rvGC;
+		}
+		else
+		{
+			cs = Desks[desk].colorset;
+			FwinString->gc = Desks[desk].NormalGC;
+		}
+
+		FwinString->flags.has_colorset = False;
+		if (cs >= 0)
+		{
+			FwinString->colorset = &Colorset[cs];
+			FwinString->flags.has_colorset = True;
+		}
+		FwinString->x = (desk_w - w)/2;
+		FwinString->y = (LabelsBelow ?
+				 desk_h + Ffont->ascent + 1 : Ffont->ascent + 1);
+		if (region)
+		{
+			FwinString->flags.has_clip_region = True;
+			FwinString->clip_region = region;
+			XSetRegion(dpy, FwinString->gc, region);
+		}
+		else
+		{
+			FwinString->flags.has_clip_region = False;
+		}
+		FlocaleDrawString(dpy, Ffont, FwinString, 0);
+		if (region)
+		{
+			XDestroyRegion(region);
+			FwinString->flags.has_clip_region = False;
+			FwinString->clip_region = None;
+			XSetClipMask(dpy, FwinString->gc, None);
+		}
+	}
+
+	if (FShapesSupported)
+	{
+		UpdateWindowShape ();
+	}
 }
 
 
@@ -1568,8 +1841,8 @@ void SwitchToDesk(int Desk)
 {
   char command[256];
 
-  sprintf(command,"GoToDesk 0 %d\n",Desk+desk1);
-  SendInfo(fd,command,0);
+  sprintf(command, "GotoDesk 0 %d", Desk + desk1);
+  SendText(fd,command,0);
 }
 
 
@@ -1585,8 +1858,8 @@ void SwitchToDeskAndPage(int Desk, XEvent *Event)
     vy = Event->xbutton.y * Scr.VHeight / (desk_h * Scr.MyDisplayHeight);
     Scr.Vx = vx * Scr.MyDisplayWidth;
     Scr.Vy = vy * Scr.MyDisplayHeight;
-    sprintf(command, "GoToDeskAndPage %d %d %d\n", Desk + desk1, vx, vy);
-    SendInfo(fd, command, 0);
+    sprintf(command, "GotoDeskAndPage %d %d %d", Desk + desk1, vx, vy);
+    SendText(fd, command, 0);
 
   }
   else
@@ -1605,8 +1878,8 @@ void SwitchToDeskAndPage(int Desk, XEvent *Event)
       x = Scr.VxMax / Scr.MyDisplayWidth;
     if (y * Scr.MyDisplayHeight > Scr.VyMax)
       y = Scr.VyMax / Scr.MyDisplayHeight;
-    sprintf(command, "GotoPage %d %d\n", x, y);
-    SendInfo(fd, command, 0);
+    sprintf(command, "GotoPage %d %d", x, y);
+    SendText(fd, command, 0);
   }
   Wait = 1;
 }
@@ -1615,10 +1888,10 @@ void IconSwitchPage(XEvent *Event)
 {
   char command[34];
 
-  sprintf(command,"GotoPage %d %d\n",
+  sprintf(command,"GotoPage %d %d",
 	  Event->xbutton.x * Scr.VWidth / (icon_w * Scr.MyDisplayWidth),
 	  Event->xbutton.y * Scr.VHeight / (icon_h * Scr.MyDisplayHeight));
-  SendInfo(fd, command, 0);
+  SendText(fd, command, 0);
   Wait = 1;
 }
 
@@ -1730,8 +2003,7 @@ void ChangeDeskForWindow(PagerWindow *t,long newdesk)
     if (size_changed)
       XResizeWindow(dpy, t->PagerView, w, h);
     cset = (t != FocusWin) ? windowcolorset : activecolorset;
-    if (cset > -1 &&
-	(size_changed || Colorset[cset].pixmap == ParentRelative))
+    if (cset > -1 && (size_changed || CSET_IS_TRANSPARENT(cset)))
     {
       SetWindowBackground(
 	dpy, t->PagerView, t->pager_view_width, t->pager_view_height,
@@ -1757,8 +2029,7 @@ void ChangeDeskForWindow(PagerWindow *t,long newdesk)
 
     XMoveResizeWindow(dpy,t->IconView,x,y,w,h);
     cset = (t != FocusWin) ? windowcolorset : activecolorset;
-    if (cset > -1 &&
-	(size_changed || Colorset[cset].pixmap == ParentRelative))
+    if (cset > -1 && (size_changed || CSET_IS_TRANSPARENT(cset)))
     {
       SetWindowBackground(
 	dpy, t->IconView, t->icon_view_width, t->icon_view_height,
@@ -1805,8 +2076,7 @@ void MoveResizePagerView(PagerWindow *t, Bool do_force_redraw)
 
       XMoveResizeWindow(dpy, t->PagerView, x, y, w, h);
       cset = (t != FocusWin) ? windowcolorset : activecolorset;
-      if (cset > -1 &&
-	  (size_changed || Colorset[cset].pixmap == ParentRelative))
+      if (cset > -1 && (size_changed || CSET_IS_TRANSPARENT(cset)))
       {
 	SetWindowBackground(
 	  dpy, t->PagerView, t->pager_view_width, t->pager_view_height,
@@ -1834,8 +2104,7 @@ void MoveResizePagerView(PagerWindow *t, Bool do_force_redraw)
 
     XMoveResizeWindow(dpy, t->IconView, x, y, w, h);
     cset = (t != FocusWin) ? windowcolorset : activecolorset;
-    if (cset > -1 &&
-	(size_changed || Colorset[windowcolorset].pixmap == ParentRelative))
+    if (cset > -1 && (size_changed || CSET_IS_TRANSPARENT(cset)))
     {
       SetWindowBackground(
 	dpy, t->IconView, t->icon_view_width, t->icon_view_height,
@@ -1856,8 +2125,8 @@ void MoveStickyWindows(void)
   t = Start;
   while(t!= NULL)
   {
-    if(((IS_ICONIFIED(t))&&(IS_ICON_STICKY(t)))||
-       (IS_STICKY(t)))
+    if(((IS_ICONIFIED(t))&&(IS_ICON_STICKY_ON_DESK(t)))||
+       (IS_STICKY_ON_DESK(t)))
     {
       if(t->desk != Scr.CurrentDesk)
       {
@@ -1888,7 +2157,7 @@ void Hilight(PagerWindow *t, int on)
     }
     else
     {
-      if(IS_STICKY(t))
+      if(IS_STICKY_ON_DESK(t) || IS_STICKY_ON_PAGE(t))
       {
 	if(t->PagerView != None)
 	  XSetWindowBackgroundPixmap(dpy,t->PagerView,
@@ -2008,8 +2277,8 @@ void Scroll(int window_w, int window_h, int x, int y, int Desk,
 	}
 	if (Wait == 0 || last_sx != sx || last_sy != sy)
 	{
-		sprintf(command,"Scroll %d %d\n",sx,sy);
-		SendInfo(fd,command,0);
+		sprintf(command, "Scroll %d %d", sx, sy);
+		SendText(fd, command, 0);
 		Wait = 1;
 	}
 	if (Wait == 0)
@@ -2081,7 +2350,7 @@ void MoveWindow(XEvent *Event)
   yi = y1;
   while(!finished)
   {
-    XMaskEvent(dpy,ButtonReleaseMask | ButtonMotionMask|ExposureMask,Event);
+    FMaskEvent(dpy,ButtonReleaseMask | ButtonMotionMask|ExposureMask,Event);
 
     if(Event->type == MotionNotify)
     {
@@ -2137,7 +2406,7 @@ void MoveWindow(XEvent *Event)
       XDestroyWindow(dpy,t->PagerView);
       t->PagerView = None;
     }
-    if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+    if (FQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
 		      &x, &y, &JunkX, &JunkY, &JunkMask) == False)
     {
       /* pointer is on a different screen */
@@ -2147,15 +2416,15 @@ void MoveWindow(XEvent *Event)
     XUngrabPointer(dpy,CurrentTime);
     XSync(dpy,0);
     sprintf(command, "Silent Move %dp %dp", x, y);
-    SendInfo(fd,command,t->w);
+    SendText(fd, command, t->w);
     if (NewDesk != t->desk)
     {
       sprintf(command, "Silent MoveToDesk 0 %d", NewDesk);
-      SendInfo(fd, command, t->w);
+      SendText(fd, command, t->w);
       t->desk = NewDesk;
     }
-    SendInfo(fd,"Silent Raise",t->w);
-    SendInfo(fd,"Silent Move Pointer",t->w);
+    SendText(fd, "Silent Raise", t->w);
+    SendText(fd, "Silent Move Pointer", t->w);
     return;
   }
   else
@@ -2199,7 +2468,7 @@ void MoveWindow(XEvent *Event)
       x = Scr.VWidth - Scr.Vx - 1;
     if (y + Scr.Vy >= Scr.VHeight)
       y = Scr.VHeight - Scr.Vy - 1;
-    if(((IS_ICONIFIED(t))&&(IS_ICON_STICKY(t)))||(IS_STICKY(t)))
+    if((IS_ICONIFIED(t) && IS_ICON_STICKY_ON_DESK(t)) || (IS_STICKY_ON_DESK(t)))
     {
       NewDesk = Scr.CurrentDesk - desk1;
       if (x > Scr.MyDisplayWidth - 16)
@@ -2213,7 +2482,8 @@ void MoveWindow(XEvent *Event)
     }
     if(NewDesk + desk1 != t->desk)
     {
-      if(((IS_ICONIFIED(t))&&(IS_ICON_STICKY(t)))||(IS_STICKY(t)))
+      if((IS_ICONIFIED(t) && IS_ICON_STICKY_ON_DESK(t)) ||
+	 (IS_STICKY_ON_DESK(t)))
       {
 	NewDesk = Scr.CurrentDesk - desk1;
 	if(t->desk != Scr.CurrentDesk)
@@ -2221,8 +2491,8 @@ void MoveWindow(XEvent *Event)
       }
       else if (NewDesk + desk1 != Scr.CurrentDesk)
       {
-	sprintf(command,"Silent MoveToDesk 0 %d", NewDesk + desk1);
-	SendInfo(fd,command,t->w);
+	sprintf(command, "Silent MoveToDesk 0 %d", NewDesk + desk1);
+	SendText(fd, command, t->w);
 	t->desk = NewDesk + desk1;
       }
       else
@@ -2257,18 +2527,18 @@ void MoveWindow(XEvent *Event)
 	    ty = ty + t->height - Scr.MyDisplayHeight;
 	  }
 	  sprintf(buf, "Silent Move %dp %dp", tx, ty);
-	  SendInfo(fd, buf, t->w);
+	  SendText(fd, buf, t->w);
 	}
 	XSync(dpy,0);
       }
       else
 	MoveResizePagerView(t, True);
-      SendInfo(fd, "Silent Raise", t->w);
+      SendText(fd, "Silent Raise", t->w);
     }
     if (do_switch_desk_later)
     {
       sprintf(command,"Silent MoveToDesk 0 %d", NewDesk + desk1);
-      SendInfo(fd,command,t->w);
+      SendText(fd, command, t->w);
       t->desk = NewDesk + desk1;
     }
     if(Scr.CurrentDesk == t->desk)
@@ -2277,7 +2547,7 @@ void MoveWindow(XEvent *Event)
       usleep(5000);
       XSync(dpy,0);
 
-      SendInfo(fd, "Silent FlipFocus NoWarp", t->w);
+      SendText(fd, "Silent FlipFocus NoWarp", t->w);
     }
   }
   if (is_transient)
@@ -2408,38 +2678,81 @@ void LabelIconWindow(PagerWindow *t)
   do_label_window(t, t->IconView);
 }
 
-static void do_picture_window(PagerWindow *t, Window w, int width, int height)
+static void do_picture_window(
+	PagerWindow *t, Window w, int width, int height)
 {
-  int iconX;
-  int iconY;
+	int iconX;
+	int iconY;
+	int src_x = 0;
+	int src_y = 0;
+	int dest_w, dest_h;
+	int cs;
+	FvwmRenderAttributes fra;
 
-  if (MiniIcons)
-  {
-    if (t->mini_icon.picture && w != None)
-    {
-      if (width > t->mini_icon.width)
-	iconX = (width - t->mini_icon.width) / 2;
-      else if (width < t->mini_icon.width)
-	iconX = -((t->mini_icon.width - width) / 2);
-      else
-	iconX = 0;
-      if (height > t->mini_icon.height)
-	iconY = (height - t->mini_icon.height) / 2;
-      else if (height < t->mini_icon.height)
-	iconY = -((t->mini_icon.height - height) / 2);
-      else
-	iconY = 0;
-      if (t->mini_icon.alpha != None)
-      {
-	      XClearArea(dpy, w, iconX, iconY,
-			 t->mini_icon.width,
-			 t->mini_icon.height, False);
-      }
-      PGraphicsCopyFvwmPicture(dpy, &t->mini_icon, w,  Scr.MiniIconGC, 0, 0,
-			       t->mini_icon.width, t->mini_icon.height,
-			       iconX, iconY);
-    }
-  }
+	if (MiniIcons)
+	{
+		if (t->mini_icon.picture && w != None)
+		{
+			dest_w = t->mini_icon.width;
+			dest_h = t->mini_icon.height;
+			if (width > t->mini_icon.width)
+			{
+				iconX = (width - t->mini_icon.width) / 2;
+			}
+			else if (width < t->mini_icon.width)
+			{
+				iconX = 0;
+				src_x = (t->mini_icon.width - width) / 2;
+				dest_w = width;
+			}
+			else
+			{
+				iconX = 0;
+			}
+			if (height > t->mini_icon.height)
+			{
+				iconY = (height - t->mini_icon.height) / 2;
+			}
+			else if (height < t->mini_icon.height)
+			{
+				iconY = 0;
+				src_y = (t->mini_icon.height - height) / 2;
+				dest_h = height;
+			}
+			else
+			{
+				iconY = 0;
+			}
+			fra.mask = FRAM_DEST_IS_A_WINDOW;
+			if (t == FocusWin)
+			{
+				cs =  activecolorset;
+			}
+			else
+			{
+				cs = windowcolorset;
+			}
+			if (t->mini_icon.alpha != None ||
+			    (cs >= 0 &&
+			     Colorset[cs].icon_alpha_percent < 100))
+			{
+				XClearArea(dpy, w, iconX, iconY,
+					   t->mini_icon.width,
+					   t->mini_icon.height, False);
+			}
+			if (cs >= 0)
+			{
+				fra.mask |= FRAM_HAVE_ICON_CSET;
+				fra.colorset = &Colorset[cs];
+			}
+			PGraphicsRenderPicture(
+				dpy, w, &t->mini_icon, &fra, w,
+				Scr.MiniIconGC, None, None, src_x, src_y,
+				t->mini_icon.width - src_x,
+				t->mini_icon.height - src_y,
+				iconX, iconY, dest_w, dest_h, False);
+		}
+	}
 }
 
 void PictureWindow (PagerWindow *t)
@@ -2479,7 +2792,7 @@ void IconMoveWindow(XEvent *Event,PagerWindow *t)
 			Event->xbutton.x, Event->xbutton.y, &x1, &y1, &dumwin);
   while(!finished)
   {
-    XMaskEvent(dpy,ButtonReleaseMask | ButtonMotionMask|ExposureMask,Event);
+    FMaskEvent(dpy,ButtonReleaseMask | ButtonMotionMask|ExposureMask,Event);
 
     if(Event->type == MotionNotify)
     {
@@ -2524,7 +2837,7 @@ void IconMoveWindow(XEvent *Event,PagerWindow *t)
   {
     char command[48];
 
-    if (XQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
+    if (FQueryPointer(dpy, Scr.Root, &JunkRoot, &JunkChild,
 		      &x, &y, &JunkX, &JunkY, &JunkMask) == False)
     {
       /* pointer is on a different screen */
@@ -2534,9 +2847,9 @@ void IconMoveWindow(XEvent *Event,PagerWindow *t)
     XUngrabPointer(dpy,CurrentTime);
     XSync(dpy,0);
     sprintf(command, "Silent Move %dp %dp", x, y);
-    SendInfo(fd,command,t->w);
-    SendInfo(fd,"Silent Raise",t->w);
-    SendInfo(fd,"Silent Move Pointer",t->w);
+    SendText(fd, command, t->w);
+    SendText(fd, "Silent Raise", t->w);
+    SendText(fd, "Silent Move Pointer", t->w);
   }
   else
   {
@@ -2555,7 +2868,7 @@ void IconMoveWindow(XEvent *Event,PagerWindow *t)
       x = Scr.VWidth - Scr.Vx - 1;
     if (y + Scr.Vy >= Scr.VHeight)
       y = Scr.VHeight - Scr.Vy - 1;
-    if(((IS_ICONIFIED(t))&&(IS_ICON_STICKY(t)))||(IS_STICKY(t)))
+    if((IS_ICONIFIED(t) && IS_ICON_STICKY_ON_DESK(t)) || IS_STICKY_ON_DESK(t))
     {
       if (x > Scr.MyDisplayWidth - 16)
 	x = Scr.MyDisplayWidth - 16;
@@ -2576,8 +2889,8 @@ void IconMoveWindow(XEvent *Event,PagerWindow *t)
     }
     else
       MoveResizePagerView(t, True);
-    SendInfo(fd, "Silent Raise", t->w);
-    SendInfo(fd, "Silent FlipFocus NoWarp", t->w);
+    SendText(fd, "Silent Raise", t->w);
+    SendText(fd, "Silent FlipFocus NoWarp", t->w);
   }
   if (is_transient)
   {
@@ -2875,7 +3188,7 @@ void DrawInBalloonWindow (int i)
 }
 
 static void set_window_colorset_background(
-  PagerWindow *t, colorset_struct *csetp)
+  PagerWindow *t, colorset_t *csetp)
 {
   if (t->PagerView != None)
   {
@@ -2890,6 +3203,7 @@ static void set_window_colorset_background(
   return;
 }
 
+/* should be in sync with  draw_desk_background */
 void change_colorset(int colorset)
 {
   int i;
@@ -2933,7 +3247,8 @@ void change_colorset(int colorset)
 	  dpy, Desks[i].title_w, desk_w, desk_h + label_h, &Colorset[colorset],
 	  Pdepth, Scr.NormalGC, True);
       }
-      if (label_h != 0 && uselabel && !LabelsBelow)
+      if (label_h != 0 && uselabel && !LabelsBelow &&
+	  !CSET_IS_TRANSPARENT_PR(Desks[i].colorset))
       {
 	SetWindowBackgroundWithOffset(
 	  dpy, Desks[i].w, 0, -label_h, desk_w, desk_h + label_h,
@@ -2945,7 +3260,7 @@ void change_colorset(int colorset)
 	  dpy, Desks[i].w, desk_w, desk_h + label_h,
 	  &Colorset[Desks[i].colorset], Pdepth, Scr.NormalGC, True);
       }
-
+      update_pr_transparent_subwindows(i);
     }
     else if (Desks[i].highcolorset == colorset && uselabel)
     {
@@ -2963,7 +3278,7 @@ void change_colorset(int colorset)
 
   if (windowcolorset == colorset)
   {
-    colorset_struct *csetp = &Colorset[colorset];
+    colorset_t *csetp = &Colorset[colorset];
 
     XSetForeground(dpy, Scr.whGC, csetp->hilite);
     XSetForeground(dpy, Scr.wsGC, csetp->shadow);
@@ -2981,7 +3296,7 @@ void change_colorset(int colorset)
   }
   if (activecolorset == colorset)
   {
-    colorset_struct *csetp = &Colorset[colorset];
+    colorset_t *csetp = &Colorset[colorset];
 
     focus_fore_pix = csetp->fg;
     XSetForeground(dpy, Scr.ahGC, csetp->hilite);
