@@ -1044,7 +1044,7 @@ static void ShowObj (int NbArg,long *TabArg)
 
   tabxobj[IdItem]->flags[0] = False;
   XMapWindow(dpy,tabxobj[IdItem]->win);
-  tabxobj[IdItem]->DrawObj(tabxobj[IdItem]);
+  tabxobj[IdItem]->DrawObj(tabxobj[IdItem], NULL);
   free(arg[0]);
 }
 
@@ -1062,7 +1062,7 @@ static void ChangeValue (int NbArg,long *TabArg)
   /* On redessine l'objet pour le mettre a jour */
   if (tabxobj[TabIdObj[atoi(arg[0])]]->TypeWidget != SwallowExec)
     XClearWindow(dpy, tabxobj[TabIdObj[atoi(arg[0])]]->win);
-  tabxobj[TabIdObj[atoi(arg[0])]]->DrawObj(tabxobj[TabIdObj[atoi(arg[0])]]);
+  tabxobj[TabIdObj[atoi(arg[0])]]->DrawObj(tabxobj[TabIdObj[atoi(arg[0])]],NULL);
   free(arg[0]);
   free(arg[1]);
 }
@@ -1086,7 +1086,7 @@ static void ChangeValueMax (int NbArg,long *TabArg)
     tabxobj[TabIdObj[j]]->value = atoi(arg[1]);
     if (tabxobj[TabIdObj[j]]->TypeWidget != SwallowExec)
       XClearWindow(dpy, tabxobj[TabIdObj[j]]->win);
-    tabxobj[TabIdObj[j]]->DrawObj(tabxobj[TabIdObj[j]]);
+    tabxobj[TabIdObj[j]]->DrawObj(tabxobj[TabIdObj[j]],NULL);
   }
   free(arg[0]);
   free(arg[1]);
@@ -1111,7 +1111,7 @@ static void ChangeValueMin (int NbArg,long *TabArg)
     tabxobj[TabIdObj[j]]->value = atoi(arg[1]);
     if (tabxobj[TabIdObj[j]]->TypeWidget != SwallowExec)
       XClearWindow(dpy, tabxobj[TabIdObj[j]]->win);
-    tabxobj[TabIdObj[j]]->DrawObj(tabxobj[TabIdObj[j]]);
+    tabxobj[TabIdObj[j]]->DrawObj(tabxobj[TabIdObj[j]],NULL);
   }
   free(arg[0]);
   free(arg[1]);
@@ -1174,7 +1174,7 @@ static void ChangeFont (int NbArg,long *TabArg)
 
   if (tabxobj[IdItem]->TypeWidget != SwallowExec)
     XClearWindow(dpy, tabxobj[IdItem]->win);
-  tabxobj[IdItem]->DrawObj(tabxobj[IdItem]);
+  tabxobj[IdItem]->DrawObj(tabxobj[IdItem],NULL);
   free(arg[0]);
   free(arg[1]);
 }
@@ -1200,7 +1200,7 @@ static void ChangeSize (int NbArg,long *TabArg)
   tabxobj[IdItem]->width = an[1];
   tabxobj[IdItem]->height = an[2];
   XResizeWindow(dpy,tabxobj[IdItem]->win,an[1],an[2]);
-  tabxobj[IdItem]->DrawObj(tabxobj[IdItem]);
+  tabxobj[IdItem]->DrawObj(tabxobj[IdItem],NULL);
   free(arg[0]);
   free(arg[1]);
   free(arg[2]);
@@ -1223,7 +1223,7 @@ static void ChangeTitle (int NbArg,long *TabArg)
   tabxobj[IdItem]->title=safestrdup(arg[1]);
   if (tabxobj[IdItem]->TypeWidget != SwallowExec)
     XClearWindow(dpy, tabxobj[IdItem]->win);
-  tabxobj[IdItem]->DrawObj(tabxobj[IdItem]);
+  tabxobj[IdItem]->DrawObj(tabxobj[IdItem],NULL);
   free(arg[0]);
   free(arg[1]);
 }
@@ -1231,21 +1231,27 @@ static void ChangeTitle (int NbArg,long *TabArg)
 /******* ChangeIcon *******/
 static void ChangeIcon (int NbArg,long *TabArg)
 {
-  int i=0;
-  char *arg[2];
-  int IdItem;
+	int i=0;
+	char *arg[2];
+	int IdItem;
 
-  arg[0] = CalcArg(TabArg,&i);
-  i++;
-  arg[1] = CalcArg(TabArg,&i);
-  IdItem = TabIdObj[atoi(arg[0])];
-  tabxobj[IdItem]->icon = safestrdup(arg[1]);
-  LoadIcon(tabxobj[IdItem]);
-  if (tabxobj[IdItem]->TypeWidget != SwallowExec)
-    XClearWindow(dpy, tabxobj[IdItem]->win);
-  tabxobj[IdItem]->DrawObj(tabxobj[IdItem]);
-  free(arg[0]);
-  free(arg[1]);
+	arg[0] = CalcArg(TabArg,&i);
+	i++;
+	arg[1] = CalcArg(TabArg,&i);
+	IdItem = TabIdObj[atoi(arg[0])];
+	if (tabxobj[IdItem]->icon)
+	{
+		free(tabxobj[IdItem]->icon);
+	}
+	tabxobj[IdItem]->icon = safestrdup(arg[1]);
+	LoadIcon(tabxobj[IdItem]);
+	if (tabxobj[IdItem]->TypeWidget != SwallowExec)
+	{
+		XClearWindow(dpy, tabxobj[IdItem]->win);
+	}
+	tabxobj[IdItem]->DrawObj(tabxobj[IdItem],NULL);
+	free(arg[0]);
+	free(arg[1]);
 }
 
 /******* ChangeForeColor *******/
@@ -1265,8 +1271,11 @@ static void ChangeForeColor (int NbArg,long *TabArg)
     PictureFreeColors(
 	    dpy,Pcmap,(void*)(&(tabxobj[IdItem])->TabColor[fore]),1,0, True);
 
-  tabxobj[IdItem]->forecolor=(char*)safecalloc(100,sizeof(char));
-  sprintf(tabxobj[IdItem]->forecolor,"%s",arg[1]);
+  if (tabxobj[IdItem]->forecolor)
+  {
+	  free(tabxobj[IdItem]->forecolor);
+  }
+  tabxobj[IdItem]->forecolor= safestrdup(arg[1]);
 
   tabxobj[IdItem]->TabColor[fore] = GetColor(tabxobj[IdItem]->forecolor);
   if (tabxobj[IdItem]->colorset >= 0) {
@@ -1278,7 +1287,7 @@ static void ChangeForeColor (int NbArg,long *TabArg)
 
   if (tabxobj[IdItem]->TypeWidget != SwallowExec)
     XClearWindow(dpy, tabxobj[IdItem]->win);
-  tabxobj[IdItem]->DrawObj(tabxobj[IdItem]);
+  tabxobj[IdItem]->DrawObj(tabxobj[IdItem],NULL);
 
   free(arg[0]);
   free(arg[1]);
@@ -1302,8 +1311,11 @@ static void ChangeBackColor (int NbArg,long *TabArg)
     PictureFreeColors(
 	    dpy,Pcmap,(void*)(&(tabxobj[IdItem])->TabColor[back]),1,0,True);
 
-  tabxobj[IdItem]->backcolor = (char*)safecalloc(100,sizeof(char));
-  sprintf(tabxobj[IdItem]->backcolor,"%s",arg[1]);
+  if (tabxobj[IdItem]->backcolor)
+  {
+	  free(tabxobj[IdItem]->backcolor);
+  }
+  tabxobj[IdItem]->backcolor= safestrdup(arg[1]);
 
   tabxobj[IdItem]->TabColor[back] = GetColor(tabxobj[IdItem]->backcolor);
   if (tabxobj[IdItem]->colorset >= 0) {
@@ -1319,7 +1331,7 @@ static void ChangeBackColor (int NbArg,long *TabArg)
 			 tabxobj[IdItem]->TabColor[back]);
     XClearWindow(dpy, tabxobj[IdItem]->win);
   }
-  tabxobj[IdItem]->DrawObj(tabxobj[IdItem]);
+  tabxobj[IdItem]->DrawObj(tabxobj[IdItem],NULL);
 
   free(arg[0]);
   free(arg[1]);
@@ -1384,7 +1396,7 @@ static void ChangeColorset (int NbArg,long *TabArg)
 			tabxobj[IdItem]->gc, False);
     XClearWindow(dpy, tabxobj[IdItem]->win);
   }
-  tabxobj[IdItem]->DrawObj(tabxobj[IdItem]);
+  tabxobj[IdItem]->DrawObj(tabxobj[IdItem],NULL);
 
   free(arg[0]);
   free(arg[1]);
@@ -1784,9 +1796,9 @@ static void Key (int NbArg,long *TabArg)
     return;
   }
 
-  AddBinding(dpy, &BindingsList, KEY_BINDING, STROKE_ARG(0)
-	     0, keysym, key_string, modifier, C_WINDOW,
-	     (void *)action, NULL);
+  AddBinding(
+	  dpy, &BindingsList, BIND_KEYPRESS, STROKE_ARG(0) 0, keysym,
+	  key_string, modifier, C_WINDOW, (void *)action, NULL);
   free(key_string);
   free(in_modifier);
 }
