@@ -49,7 +49,7 @@
 #include <stdio.h>
 #include <signal.h>
 #include <fcntl.h>
-#include <sys/time.h>
+#include "libs/ftime.h"
 
 #ifdef HAVE_SYS_BSDTYPES_H
 #include <sys/bsdtypes.h> /* Saul */
@@ -62,6 +62,11 @@
 #include "libs/fvwmlib.h"
 #include "libs/FScreen.h"
 #include "libs/FShape.h"
+#include "libs/FRenderInit.h"
+#include "libs/Flocale.h"
+#include "libs/gravity.h"
+#include "libs/FRender.h"
+#include "libs/FRenderInterface.h"
 
 #include "FvwmWinList.h"
 #include "ButtonArray.h"
@@ -69,16 +74,6 @@
 #include "Mallocs.h"
 #include "libs/Colorset.h"
 
-#ifdef I18N_MB
-#include <X11/Xlocale.h>
-#ifdef __STDC__
-#define XTextWidth(x,y,z) XmbTextEscapement(x ## set,y,z)
-#else
-#define XTextWidth(x,y,z) XmbTextEscapement(x/**/set,y,z)
-#endif
-#define XDrawString(t,u,v,w,x,y,z) XmbDrawString(t,u,ButtonFontset,v,w,x,y,z)
-#endif
-
 #define MAX_NO_ICON_ACTION_LENGTH (MAX_MODULE_INPUT_TEXT_LEN - 100)
 
 #define GRAB_EVENTS (ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|EnterWindowMask|LeaveWindowMask)
@@ -109,10 +104,8 @@ GC shadow[MAX_COLOUR_SETS];
 GC hilite[MAX_COLOUR_SETS];
 GC background[MAX_COLOUR_SETS];
 Pixmap win_bg;
-XFontStruct *ButtonFont;
-#ifdef I18N_MB
-XFontSet ButtonFontset;
-#endif
+FlocaleFont *FButtonFont;
+FlocaleWinString *FwinString;
 int fontheight;
 int buttonheight;
 static Atom wm_del_win;
@@ -152,7 +145,7 @@ char *ClickAction[NUMBER_OF_BUTTONS] =
   *geometry="";
 int colorset[MAX_COLOUR_SETS];
 Pixmap pixmap[MAX_COLOUR_SETS];
-char *font_string = "fixed";
+char *font_string = NULL;
 Bool UseSkipList = False;
 Bool Anchor = True;
 Bool UseIconNames = False;
@@ -165,6 +158,12 @@ int ReliefWidth = 2;
 long CurrentDesk = 0;
 int ShowCurrentDesk = 0;
 
+struct
+{
+	unsigned is_shaded : 1;
+	unsigned needs_resize_after_unshade : 1;
+} flags = { 0, 0 };
+
 static RETSIGTYPE TerminateHandler(int sig);
 
 static char *AnimCommand = NULL;
@@ -182,6 +181,33 @@ static RETSIGTYPE TerminateHandler(int sig)
   fvwmSetTerminate(sig);
 }
 
+static void window_shade(unsigned long *body, int do_shade)
+{
+	Window w;
+
+	w = (Window)(body[0]);
+	if (w != win)
+	{
+		return;
+	}
+	if (do_shade)
+	{
+		flags.is_shaded = 1;
+		flags.needs_resize_after_unshade = 0;
+	}
+	else
+	{
+		flags.is_shaded = 0;
+		if (flags.needs_resize_after_unshade)
+		{
+			AdjustWindow(True);
+			flags.needs_resize_after_unshade = 0;
+		}
+	}
+
+	return;
+}
+
 /******************************************************************************
   Main - Setup the XConnection,request the window list and loop forever
     Based on main() from FvwmIdent:
@@ -242,9 +268,7 @@ int main(int argc, char **argv)
     Clength = strlen(Module);
   }
 
-#ifdef I18N_MB
-  setlocale(LC_CTYPE, "");
-#endif
+  FlocaleInit(LC_CTYPE, "", "", "FvwmWinList");
 
   Fvwm_fd[0] = atoi(argv[1]);
   Fvwm_fd[1] = atoi(argv[2]);
@@ -266,8 +290,8 @@ int main(int argc, char **argv)
 #else
 #ifdef USE_BSD_SIGNALS
   fvwmSetSignalMask( sigmask(SIGPIPE) |
-                     sigmask(SIGINT)  |
-                     sigmask(SIGTERM) );
+		     sigmask(SIGINT)  |
+		     sigmask(SIGTERM) );
 #endif
   signal(SIGPIPE, TerminateHandler);
   signal(SIGTERM, TerminateHandler);
@@ -283,6 +307,7 @@ int main(int argc, char **argv)
   colorset[0] = colorset[1] = 0;
   colorset[2] = colorset[3] = 1;
   AllocColorset(1);
+  FlocaleAllocateWinString(&FwinString);
 
   /* Setup the XConnection */
   StartMeUp_I();
@@ -300,17 +325,16 @@ int main(int argc, char **argv)
   /* Request a list of all windows,
    * wait for ConfigureWindow packets */
   SetMessageMask(Fvwm_fd,
-#ifdef MINI_ICONS
 		 M_MINI_ICON |
-#endif
 		 M_CONFIGURE_WINDOW | M_ADD_WINDOW | M_DESTROY_WINDOW |
 		 M_VISIBLE_NAME | M_DEICONIFY |
 		 M_ICONIFY | M_END_WINDOWLIST | M_NEW_DESK | M_FOCUS_CHANGE |
-		 M_CONFIG_INFO | M_SENDCONFIG);
+		 M_CONFIG_INFO | M_SENDCONFIG | M_WINDOWSHADE |
+		 M_DEWINDOWSHADE);
   /* extended messages */
-  SetMessageMask(Fvwm_fd, MX_VISIBLE_ICON_NAME);
+  SetMessageMask(Fvwm_fd, MX_VISIBLE_ICON_NAME | MX_PROPERTY_CHANGE);
 
-  SendFvwmPipe(Fvwm_fd, "Send_WindowList",0);
+  SendText(Fvwm_fd, "Send_WindowList",0);
 
   /* Recieve all messages from Fvwm */
   atexit(ShutMeDown);
@@ -346,7 +370,7 @@ void MainEventLoop(void)
     XFlush(dpy);
     if (fvwmSelect(fd_width, &readset, NULL, NULL, NULL) > 0) {
 
-      if (FD_ISSET(x_fd,&readset) || XPending(dpy)) LoopOnEvents();
+      if (FD_ISSET(x_fd,&readset) || FPending(dpy)) LoopOnEvents();
       if (FD_ISSET(Fvwm_fd[1],&readset)) ReadFvwmPipe();
 
     }
@@ -380,10 +404,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
   char *name,*string;
   static int current_focus=-1;
   struct ConfigWinPacket  *cfgpacket;
-
-#ifdef MINI_ICONS
-  Picture p;
-#endif
+  FvwmPicture p;
 
   switch(type)
   {
@@ -397,53 +418,63 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       {
 	win_border_x = win_border_y = cfgpacket->border_width;
 	if (((win_grav == NorthWestGravity || win_grav == NorthEastGravity)
-	     && !HAS_BOTTOM_TITLE(cfgpacket)) ||
+	     && HAS_TITLE_DIR(cfgpacket, DIR_N)) ||
 	    ((win_grav == SouthWestGravity || win_grav == SouthEastGravity)
-	    && HAS_BOTTOM_TITLE(cfgpacket)))
+	    && HAS_TITLE_DIR(cfgpacket, DIR_S)))
+	{
 	  win_border_y +=  cfgpacket->title_height;
+	}
+	if (((win_grav == NorthWestGravity || win_grav == SouthWestGravity)
+	     && HAS_TITLE_DIR(cfgpacket, DIR_W)) ||
+	    ((win_grav == NorthEastGravity || win_grav == SouthEastGravity)
+	     && HAS_TITLE_DIR(cfgpacket, DIR_E)))
+	{
+	  win_border_x +=  cfgpacket->title_height;
+	}
       }
 #endif
       if ((i = FindItem(&windows,cfgpacket->w))!=-1)
       {
 	flagitem=ItemFlags(&windows,cfgpacket->w);
 	if(UpdateItemDesk(&windows, cfgpacket) > 0 ||
-	   IS_STICKY(cfgpacket) != IS_STICKY(flagitem) ||
+	   IS_STICKY_ON_DESK(cfgpacket) != IS_STICKY_ON_DESK(flagitem) ||
 	   DO_SKIP_WINDOW_LIST(cfgpacket) != DO_SKIP_WINDOW_LIST(flagitem))
-        {
+	{
 	  UpdateItemGSFRFlags(&windows, cfgpacket);
-          UpdateButtonDeskFlags(&buttons, i, cfgpacket->desk,
-				IS_STICKY(cfgpacket),
+	  UpdateButtonDeskFlags(&buttons, i, cfgpacket->desk,
+				IS_STICKY_ON_DESK(cfgpacket),
 				DO_SKIP_WINDOW_LIST(cfgpacket));
-          AdjustWindow(False);
-          RedrawWindow(True, True);
-        }
+	  AdjustWindow(False);
+	  RedrawWindow(True, True, NULL);
+	}
 	else
 	  UpdateItemGSFRFlags(&windows, cfgpacket);
 	break;
       }
       else
-        AddItem(&windows, cfgpacket);
+	AddItem(&windows, cfgpacket);
       break;
     case M_DESTROY_WINDOW:
       cfgpacket = (void *) body;
       if ((i=DeleteItem(&windows,cfgpacket->w))==-1) break;
       RemoveButton(&buttons,i);
       if (WindowState)
-        AdjustWindow(False);
+	AdjustWindow(False);
       redraw=1;
       break;
 
-#ifdef MINI_ICONS
   case M_MINI_ICON:
-    if ((i=FindItem(&windows,body[0]))==-1) break;
-
+    if (!FMiniIconsSupported || (i=FindItem(&windows,body[0]))==-1)
+      break;
     if (UpdateButton(&buttons,i,NULL,-1)!=-1)
     {
-      p.width = body[3];
-      p.height = body[4];
-      p.depth = body[5];
-      p.picture = body[6];
-      p.mask = body[7];
+      MiniIconPacket *mip = (MiniIconPacket *) body;
+      p.picture = mip->picture;
+      p.mask    = mip->mask;
+      p.alpha   = mip->alpha;
+      p.width   = mip->width;
+      p.height  = mip->height;
+      p.depth   = mip->depth;
 
       UpdateButtonPicture(&buttons, i, &p);
 
@@ -451,7 +482,6 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       redraw = 1;
     }
     break;
-#endif
 
     case M_VISIBLE_NAME:
     case MX_VISIBLE_ICON_NAME:
@@ -466,8 +496,8 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       {
 	AddButton(&buttons, name, NULL, 1);
 	UpdateButtonSet(&buttons, i, (IS_ICONIFIED(flagitem)?1:0));
-        UpdateButtonDeskFlags(&buttons,i,ItemDesk(&windows, body[0]),
-			      IS_STICKY(flagitem),
+	UpdateButtonDeskFlags(&buttons,i,ItemDesk(&windows, body[0]),
+			      IS_STICKY_ON_DESK(flagitem),
 			      DO_SKIP_WINDOW_LIST(flagitem));
       }
       free(name);
@@ -508,7 +538,7 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	  SET_ICONIFIED(flagitem, !IS_ICONIFIED(flagitem));
 	  string = ItemName(&windows, i);
 	  name = makename(string, IS_ICONIFIED(flagitem));
-          UpdateButtonIconified(&buttons, i, IS_ICONIFIED(flagitem));
+	  UpdateButtonIconified(&buttons, i, IS_ICONIFIED(flagitem));
 	  if (UpdateButton(&buttons, i, name, -1) != -1) redraw = 1;
 	  if (i != current_focus || (IS_ICONIFIED(flagitem)))
 	    if (UpdateButtonSet(&buttons, i, (IS_ICONIFIED(flagitem)) ? 1 : 0)
@@ -523,14 +553,14 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       redraw = 1;
       if ((i=FindItem(&windows,body[0]))!=-1)
       {
-        RadioButton(&buttons,i,ButPressed);
-        if (Follow && i) { /* rearrange order */
-          ReorderList(&windows,i,body[2]);
-          if (WindowState) ReorderButtons(&buttons,i,body[2]);
-        }
+	RadioButton(&buttons,i,ButPressed);
+	if (Follow && i) { /* rearrange order */
+	  ReorderList(&windows,i,body[2]);
+	  if (WindowState) ReorderButtons(&buttons,i,body[2]);
+	}
       }
       else
-        RadioButton(&buttons,-1,ButPressed);
+	RadioButton(&buttons,-1,ButPressed);
       break;
     case M_END_WINDOWLIST:
       if (!WindowState)
@@ -541,27 +571,40 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       CurrentDesk = body[0];
       if(ShowCurrentDesk)
       {
-        AdjustWindow(False);
-        RedrawWindow(True, True);
+	AdjustWindow(False);
+	RedrawWindow(True, True, NULL);
       }
       break;
+    case M_WINDOWSHADE:
+      window_shade(body, 1);
+      break;
+    case M_DEWINDOWSHADE:
+      window_shade(body, 0);
+      break;
+  case MX_PROPERTY_CHANGE:
+    if (body[0] == MX_PROPERTY_CHANGE_BACKGROUND && body[2] == 0 &&
+	WindowState &&  win_bg == ParentRelative)
+    {
+      RedrawWindow(True, True, NULL);
+    }
+    break;
     case M_CONFIG_INFO:
       ParseConfigLine((char *)&body[3]);
       break;
   }
 
-  if (redraw && WindowState==1) RedrawWindow(False, True);
+  if (redraw && WindowState==1) RedrawWindow(False, True, NULL);
 }
 
 
 /******************************************************************************
   RedrawWindow - Update the needed lines and erase any old ones
 ******************************************************************************/
-void RedrawWindow(Bool force, Bool clear_bg)
+void RedrawWindow(Bool force, Bool clear_bg, XEvent *evp)
 {
-  DrawButtonArray(&buttons, force, clear_bg);
+  DrawButtonArray(&buttons, force, clear_bg, evp);
   XFlush(dpy); /** Put here for VMS; ifdef if causes performance problem **/
-  if (XQLength(dpy) && !force) LoopOnEvents();
+  if (FQLength(dpy) && !force) LoopOnEvents();
 }
 
 /******************************************************************************
@@ -624,8 +667,8 @@ ParseConfigLine(char *tline)
     return;
 
   if (strlen(tline) > 1) {
-    if (strncasecmp(tline, CatString3(Module, "Font", ""), Clength + 4)	== 0)
-      CopyString(&font_string, &tline[Clength + 4]);
+    if (strncasecmp(tline, CatString3(Module, "Font", ""), Clength + 4) == 0)
+      CopyStringWithQuotes(&font_string, &tline[Clength + 4]);
     else if (strncasecmp(tline, CatString3(Module, "Fore", ""), Clength + 4)
 	     == 0) {
       CopyString(&ForeColor[0], &tline[Clength + 4]);
@@ -743,16 +786,17 @@ ParseConfigLine(char *tline)
 	/* set the window background */
 	win_bg = None;
 	for (i = 0; i != MAX_COLOUR_SETS; i++)
-	    if (colorset[i] >= 0 &&
-		Colorset[colorset[i]].pixmap == ParentRelative)
+	{
+	    if (CSET_IS_TRANSPARENT_PR(colorset[i]))
 	      win_bg = ParentRelative;
+	}
 	if (old_win_bg != win_bg)
 	  XSetWindowBackgroundPixmap(dpy, win, win_bg);
 	/* refresh if this colorset is used */
 	for (i = 0; i != MAX_COLOUR_SETS; i++) {
 	  if (colorset[i] == cset) {
 	    AdjustWindow(True);
-	    RedrawWindow(True, True);
+	    RedrawWindow(True, True, NULL);
 	    break;
 	  }
 	}
@@ -763,12 +807,6 @@ ParseConfigLine(char *tline)
       FScreenConfigureModule(
 	tline + sizeof(XINERAMA_CONFIG_STRING) - 1);
     }
-    else if (strncasecmp(tline, ROOT_BG_CHANGE_STRING,
-			 sizeof(ROOT_BG_CHANGE_STRING) - 1) == 0)
-    {
-      if (WindowState && win_bg == ParentRelative)
-	RedrawWindow(True, True);
-    }
   }
 }
 
@@ -790,7 +828,7 @@ void LoopOnEvents(void)
   {
     if (Pressed)
     {
-      if (XQueryPointer(
+      if (FQueryPointer(
 	    dpy, win, &dummyroot, &dummychild, &xdummy, &ydummy, &x, &y,
 	    &dummy1) == False)
       {
@@ -812,51 +850,61 @@ void LoopOnEvents(void)
   }
 #endif
 
-  while(XPending(dpy))
+  while(FPending(dpy))
   {
-    XNextEvent(dpy,&Event);
+    FNextEvent(dpy,&Event);
 
     switch(Event.type)
     {
       case ButtonRelease:
-        if (Pressed)
-        {
-          num=WhichButton(&buttons,Event.xbutton.x,Event.xbutton.y);
-          if (num!=-1 && Event.xbutton.button >= 1 &&
+	if (Pressed)
+	{
+	  num=WhichButton(&buttons,Event.xbutton.x,Event.xbutton.y);
+	  if (num!=-1 && Event.xbutton.button >= 1 &&
 	      Event.xbutton.button <= NUMBER_OF_MOUSE_BUTTONS)
 	  {
-            SendFvwmPipe(Fvwm_fd,
-			 ClickAction[(Transient) ? 0:Event.xbutton.button-1],
+	    SendFvwmPipe(Fvwm_fd,
+			 ClickAction[Event.xbutton.button-1],
 			 ItemID(&windows,num));
-            SwitchButton(&buttons,num);
-          }
-        }
-        if (Transient)
+	    SwitchButton(&buttons,num);
+	  }
+	}
+	if (Transient)
 	{
 	  exit(0);
 	}
-        Pressed=0;
-        ButPressed=-1;
-        break;
-      case ButtonPress:
-        num=WhichButton(&buttons,Event.xbutton.x,Event.xbutton.y);
-        if (num != -1)
-        {
-          SwitchButton(&buttons,num);
-          ButPressed=num;
-        } else ButPressed=-1;
-        Pressed=1;
-        break;
-      case Expose:
-	/* have to clear each button with its background,
-	 * let the server compute the intersections */
-	do
-	  ExposeAllButtons(&buttons, &Event);
-	while (XCheckTypedWindowEvent(dpy, win, Expose, &Event));
-        /* No more expose events in the queue draw all icons,
-         * text and shadows, but not the background */
-        RedrawWindow(True, False);
+	Pressed=0;
+	ButPressed=-1;
 	break;
+      case ButtonPress:
+	num=WhichButton(&buttons,Event.xbutton.x,Event.xbutton.y);
+	if (num != -1)
+	{
+	  SwitchButton(&buttons,num);
+	  ButPressed=num;
+	} else ButPressed=-1;
+	Pressed=1;
+	break;
+      case Expose:
+      {
+	      int ex = Event.xexpose.x;
+	      int ey = Event.xexpose.y;
+	      int ex2 = Event.xexpose.x + Event.xexpose.width;
+	      int ey2 = Event.xexpose.y + Event.xexpose.height;
+	      while (FCheckTypedWindowEvent(dpy, win, Expose, &Event))
+	      {
+		      ex = min(ex, Event.xexpose.x);
+		      ey = min(ey, Event.xexpose.y);
+		      ex2 = max(ex2, Event.xexpose.x + Event.xexpose.width);
+		      ey2 = max(ey2 , Event.xexpose.y + Event.xexpose.height);
+	      }
+	      Event.xexpose.x = ex;
+	      Event.xexpose.y = ey;
+	      Event.xexpose.width = ex2 - ex;
+	      Event.xexpose.height = ey2 - ey;
+	      RedrawWindow(True, True, &Event);
+	      break;
+      }
       case ConfigureNotify:
 	{
 	  XEvent event;
@@ -864,7 +912,7 @@ void LoopOnEvents(void)
 	/* Opaque moves cause lots of these to be sent which causes flickering
 	 * It's better to miss out on intermediate steps than to do them all
 	 * and take too long. Look down the event queue and do the last one */
-	  while (XCheckTypedWindowEvent(dpy, win, ConfigureNotify, &event))
+	  while (FCheckTypedWindowEvent(dpy, win, ConfigureNotify, &event))
 	  {
 	    /* if send_event is true it means the user has moved the window or
 	     * winlist has mapped itself, take note of the new position.
@@ -875,55 +923,56 @@ void LoopOnEvents(void)
 	    Event.xconfigure.y = event.xconfigure.y;
 	    Event.xconfigure.send_event = True;
 	  }
+	  if (Event.xconfigure.send_event &&
+	      (win_x != Event.xconfigure.x ||
+	       win_y != Event.xconfigure.y || win_bg == ParentRelative))
+	  {
+		  win_x = Event.xconfigure.x;
+		  win_y = Event.xconfigure.y;
+		  DrawTransparentButtonArray(&buttons);
+	  }
 	}
-        if (Event.xconfigure.send_event && (win_x != Event.xconfigure.x
-            || win_y != Event.xconfigure.y || win_bg == ParentRelative)) {
-	  win_x = Event.xconfigure.x;
-	  win_y = Event.xconfigure.y;
-	  if (win_bg == ParentRelative)
-	    RedrawWindow(True, True);
-	}
-        break;
+	break;
       case KeyPress:
-        num=XLookupString(&Event.xkey,buffer,10,NULL,0);
-        if (num==1)
-        {
-          if (buffer[0]=='q' || buffer[0]=='Q') exit(0);
-          else if (buffer[0]=='i' || buffer[0]=='I') PrintList(&windows);
-          else if (buffer[0]=='b' || buffer[0]=='B') PrintButtons(&buttons);
-        }
-        break;
+	num=XLookupString(&Event.xkey,buffer,10,NULL,0);
+	if (num==1)
+	{
+	  if (buffer[0]=='q' || buffer[0]=='Q') exit(0);
+	  else if (buffer[0]=='i' || buffer[0]=='I') PrintList(&windows);
+	  else if (buffer[0]=='b' || buffer[0]=='B') PrintButtons(&buttons);
+	}
+	break;
       case ClientMessage:
-        if ((Event.xclient.format==32) && (Event.xclient.data.l[0]==wm_del_win))
-          exit(0);
+	if ((Event.xclient.format==32) && (Event.xclient.data.l[0]==wm_del_win))
+	  exit(0);
       case EnterNotify:
-        if (!SomeButtonDown(Event.xcrossing.state)) break;
-        num=WhichButton(&buttons,Event.xcrossing.x,Event.xcrossing.y);
-        if (num!=-1)
-        {
-          SwitchButton(&buttons,num);
-          ButPressed=num;
-        } else ButPressed=-1;
-        Pressed=1;
-        break;
+	if (!SomeButtonDown(Event.xcrossing.state)) break;
+	num=WhichButton(&buttons,Event.xcrossing.x,Event.xcrossing.y);
+	if (num!=-1)
+	{
+	  SwitchButton(&buttons,num);
+	  ButPressed=num;
+	} else ButPressed=-1;
+	Pressed=1;
+	break;
       case LeaveNotify:
-        if (!SomeButtonDown(Event.xcrossing.state)) break;
-        if (ButPressed!=-1) SwitchButton(&buttons,ButPressed);
-        Pressed=0;
-        break;
+	if (!SomeButtonDown(Event.xcrossing.state)) break;
+	if (ButPressed!=-1) SwitchButton(&buttons,ButPressed);
+	Pressed=0;
+	break;
       case MotionNotify:
-        if (!Pressed) break;
-        num=WhichButton(&buttons,Event.xmotion.x,Event.xmotion.y);
-        if (num==ButPressed) break;
-        if (ButPressed!=-1) SwitchButton(&buttons,ButPressed);
-        if (num!=-1)
-        {
-          SwitchButton(&buttons,num);
-          ButPressed=num;
-        }
-        else ButPressed=-1;
+	if (!Pressed) break;
+	num=WhichButton(&buttons,Event.xmotion.x,Event.xmotion.y);
+	if (num==ButPressed) break;
+	if (ButPressed!=-1) SwitchButton(&buttons,ButPressed);
+	if (num!=-1)
+	{
+	  SwitchButton(&buttons,num);
+	  ButPressed=num;
+	}
+	else ButPressed=-1;
 
-        break;
+	break;
     }
   }
 }
@@ -936,9 +985,7 @@ void AdjustWindow(Bool force)
 {
   int new_width=0,new_height=0,tw,i,total,all;
   char *temp;
-#ifdef MINI_ICONS
   int base_miw = 0;
-#endif
 
   total = ItemCountVisible(&windows);
   all = ItemCount(&windows);
@@ -952,29 +999,31 @@ void AdjustWindow(Bool force)
     return;
   }
 
-#ifdef MINI_ICONS
-  for(i=0; i<all; i++)
+  if (FMiniIconsSupported)
   {
-    if(IsButtonIndexVisible(&buttons,i) &&
-       ButtonPicture(&buttons,i)->picture != None)
+    for(i=0; i<all; i++)
     {
-      if (ButtonPicture(&buttons,i)->width > base_miw)
-	base_miw = ButtonPicture(&buttons,i)->width;
-      break;
+      if (IsButtonIndexVisible(&buttons,i) &&
+	  ButtonPicture(&buttons,i)->picture != None)
+      {
+	if (ButtonPicture(&buttons,i)->width > base_miw)
+	  base_miw = ButtonPicture(&buttons,i)->width;
+	break;
+      }
     }
+    base_miw += 2;
   }
-  base_miw += 2;
-#endif
   for(i=0;i<all;i++)
   {
     temp=ItemName(&windows,i);
     if(temp != NULL && IsItemIndexVisible(&windows,i))
     {
-	tw = 8 + XTextWidth(ButtonFont,temp,strlen(temp));
-	tw += XTextWidth(ButtonFont,"()",2);
-#ifdef MINI_ICONS
-	tw += base_miw;
-#endif
+	tw = 8 + FlocaleTextWidth(FButtonFont,temp,strlen(temp));
+	tw += FlocaleTextWidth(FButtonFont,"()",2);
+	if (FMiniIconsSupported)
+	{
+	  tw += base_miw;
+	}
 	new_width=max(new_width,tw);
     }
   }
@@ -1004,12 +1053,23 @@ void AdjustWindow(Bool force)
       if (cset >= 0) {
 	if (pixmap[i])
 	  XFreePixmap(dpy, pixmap[i]);
-	if (Colorset[cset].pixmap) {
-	  pixmap[i] = CreateBackgroundPixmap(dpy, win, new_width, buttonheight,
-					     &Colorset[cset], Pdepth,
-					     background[i], False);
-	  XSetTile(dpy, background[i], pixmap[i]);
-	  XSetFillStyle(dpy, background[i], FillTiled);
+	if (Colorset[cset].pixmap && !CSET_IS_TRANSPARENT_PR(cset)) {
+		if (CSET_IS_TRANSPARENT_ROOT(cset))
+		{
+			pixmap[i] = CreateBackgroundPixmap(
+				dpy, win, new_width, win_height,
+				&Colorset[cset], Pdepth,
+				background[i], False);
+		}
+		else
+		{
+			pixmap[i] = CreateBackgroundPixmap(
+				dpy, win, win_width, buttonheight,
+				&Colorset[cset], Pdepth,
+				background[i], False);
+		}
+		XSetTile(dpy, background[i], pixmap[i]);
+		XSetFillStyle(dpy, background[i], FillTiled);
 	} else {
 	  pixmap[i] = None;
 	  XSetFillStyle(dpy, background[i], FillSolid);
@@ -1017,7 +1077,9 @@ void AdjustWindow(Bool force)
       }
     }
   }
-  if (WindowState && (new_height!=win_height || new_width!=win_width))
+  if (WindowState &&
+      (new_height!=win_height || new_width!=win_width ||
+       (!flags.is_shaded && flags.needs_resize_after_unshade)))
   {
 #ifdef COMPLEX_WINDOW_PLACEMENT
     if (Anchor)
@@ -1049,14 +1111,19 @@ void AdjustWindow(Bool force)
       }
       /* XMoveResizeWindow(dpy, win, win_x, win_y, new_width, new_height); */
     }
-    if (Anchor && WindowState == 2)
+    if (flags.is_shaded)
+      flags.needs_resize_after_unshade = 1;
+    else if (Anchor && WindowState == 2)
       XMoveResizeWindow(dpy, win, win_x, win_y, new_width, new_height);
     else
       XResizeWindow(dpy, win, new_width, new_height);
     /* The new code is sooo much simpler :-) */
 #else
-    /* This relies on fvwm honouring South/East gemoetry when resizing. */
-    XResizeWindow(dpy, win, new_width, new_height);
+    if (flags.is_shaded)
+      flags.needs_resize_after_unshade = 1;
+    else
+      /* This relies on fvwm honouring South/East gemoetry when resizing. */
+      XResizeWindow(dpy, win, new_width, new_height);
 #endif
   }
   UpdateArray(&buttons,new_width);
@@ -1138,7 +1205,7 @@ void MakeMeWindow(void)
   {
     fscreen_scr_arg fscr;
 
-    if (XQueryPointer(
+    if (FQueryPointer(
 	  dpy, Root, &dummyroot, &dummychild, &hints.x, &hints.y, &x, &y,
 	  &dummy1) == False)
     {
@@ -1240,11 +1307,13 @@ void MakeMeWindow(void)
    * the colorsets is transparent which requires help from the server */
   win_bg = None;
   for (i = 0; i != MAX_COLOUR_SETS; i++)
-    if (colorset[i] >= 0 && Colorset[colorset[i]].pixmap == ParentRelative)
-    {
-      win_bg = ParentRelative;
-      break;
-    }
+  {
+	  if (CSET_IS_TRANSPARENT_PR(colorset[i]))
+	  {
+		  win_bg = ParentRelative;
+		  break;
+	  }
+  }
   attr.background_pixmap = win_bg;
   attr.border_pixel = 0;
   attr.colormap = Pcmap;
@@ -1259,6 +1328,8 @@ void MakeMeWindow(void)
   wm_del_win=XInternAtom(dpy,"WM_DELETE_WINDOW",False);
   XSetWMProtocols(dpy,win,&wm_del_win,1);
 
+  FwinString->win = win;
+
   {
     XTextProperty nametext;
     char *list[]={NULL,NULL};
@@ -1273,7 +1344,7 @@ void MakeMeWindow(void)
       exit(1);
     }
     XSetWMProperties(dpy,win,&nametext,&nametext,
-                    NULL,0,&hints,NULL,&classhints);
+		    NULL,0,&hints,NULL,&classhints);
     XFree(nametext.value);
   }
 
@@ -1299,8 +1370,12 @@ void MakeMeWindow(void)
   {
     gcval.foreground=fore[i];
     gcval.background=back[i];
-    gcval.font=ButtonFont->fid;
-    gcmask=GCForeground|GCBackground|GCFont;
+    gcmask=GCForeground|GCBackground;
+    if (FButtonFont->font != NULL)
+    {
+      gcval.font=FButtonFont->font->fid;
+      gcmask |= GCFont;
+    }
     graph[i]=fvwmlib_XCreateGC(dpy,win,gcmask,&gcval);
 
     if(Pdepth < 2)
@@ -1326,13 +1401,25 @@ void MakeMeWindow(void)
     gcmask=GCForeground;
     background[i]=fvwmlib_XCreateGC(dpy,win,gcmask,&gcval);
 
-    if ((colorset[i] >= 0) && Colorset[colorset[i]].pixmap
-        && Colorset[colorset[i]].pixmap != ParentRelative) {
-      pixmap[i] = CreateBackgroundPixmap(dpy, win, win_width, buttonheight,
-					 &Colorset[colorset[i]], Pdepth,
-					 background[i], False);
-      XSetTile(dpy, background[i], pixmap[i]);
-      XSetFillStyle(dpy, background[i], FillTiled);
+    if ((colorset[i] >= 0) && Colorset[colorset[i]].pixmap &&
+	!CSET_IS_TRANSPARENT_PR(colorset[i]))
+    {
+	    if (CSET_IS_TRANSPARENT_ROOT(colorset[i]))
+	    {
+		    pixmap[i] = CreateBackgroundPixmap(
+			    dpy, win, win_width, win_height,
+			    &Colorset[colorset[i]], Pdepth,
+			    background[i], False);
+	    }
+	    else
+	    {
+		    pixmap[i] = CreateBackgroundPixmap(
+			    dpy, win, win_width, buttonheight,
+			    &Colorset[colorset[i]], Pdepth,
+			    background[i], False);
+	    }
+	    XSetTile(dpy, background[i], pixmap[i]);
+	    XSetFillStyle(dpy, background[i], FillTiled);
     }
   }
   AdjustWindow(True);
@@ -1367,7 +1454,7 @@ void MakeMeWindow(void)
       fprintf(stderr,"failed to grab pointer\n");
       exit(1);
     }
-    if (XQueryPointer(
+    if (FQueryPointer(
 	  dpy, Root, &dummyroot, &dummychild, &hints.x, &hints.y, &x, &y,
 	  &dummy1) == False)
     {
@@ -1390,10 +1477,13 @@ void StartMeUp_I(void)
       XDisplayName(""));
     exit (1);
   }
-  InitPictureCMap(dpy);
+XSynchronize(dpy, 1);
+  PictureInitCMap(dpy);
   FScreenInit(dpy);
   AllocColorset(0);
   FShapeInit(dpy);
+  FRenderInit(dpy);
+
   x_fd = XConnectionNumber(dpy);
   screen= DefaultScreen(dpy);
   Root = RootWindow(dpy, screen);
@@ -1403,12 +1493,6 @@ void StartMeUp_I(void)
 
 void StartMeUp_II(void)
 {
-#ifdef I18N_MB
-  char **ml;
-  int mc;
-  char *ds;
-  XFontStruct **fs_list;
-#endif
 
   if (geometry == NULL)
     UpdateString(&geometry, "");
@@ -1419,36 +1503,16 @@ void StartMeUp_II(void)
   FScreenGetScrRect(
     NULL, fscreen, &screen_g.x, &screen_g.y, &screen_g.width, &screen_g.height);
 
-#ifdef I18N_MB
-  if ((ButtonFontset=XCreateFontSet(dpy,font_string,&ml,&mc,&ds)) == NULL) {
-#ifdef STRICTLY_FIXED
-    if ((ButtonFontset=XCreateFontSet(dpy,"fixed",&ml,&mc,&ds)) == NULL)
-    {
-      exit(1);
-    }
-#else
-    if ((ButtonFontset=XCreateFontSet(dpy,"-*-fixed-medium-r-normal-*-14-*-*-*-*-*-*-*",&ml,&mc,&ds)) == NULL)
-    {
-      exit(1);
-    }
-#endif
-  }
-  XFontsOfFontSet(ButtonFontset,&fs_list,&ml);
-  ButtonFont = fs_list[0];
-#else
-  if ((ButtonFont=XLoadQueryFont(dpy,font_string))==NULL)
+  if ((FButtonFont = FlocaleLoadFont(dpy,font_string,Module)) == NULL)
   {
-    if ((ButtonFont=XLoadQueryFont(dpy,"fixed"))==NULL)
-    {
-      exit(1);
-    }
+    fprintf(stderr,"%s: Cannot load font, Exitting\n", Module);
+    exit(1);
   }
-#endif
 
-  fontheight = ButtonFont->ascent+ButtonFont->descent;
+  fontheight = FButtonFont->height;
   buttonheight = fontheight + 3 + 2 * ReliefWidth;
 
-  win_width=XTextWidth(ButtonFont,"XXXXXXXXXXXXXXX",10);
+  win_width=FlocaleTextWidth(FButtonFont,"XXXXXXXXXXXXXXX",10);
 
 }
 
@@ -1528,6 +1592,8 @@ int ErrorHandler(Display *d, XErrorEvent *event)
     return 0;
   if (event->error_code == BadDrawable)
     return 0;
+if (FRenderGetErrorCodeBase() + FRenderBadPicture == event->error_code)
+    return 0;
 
   PrintXErrorAndCoredump(d, event, Module);
   return 0;
